package org.gridschool.unicore.plugins.ajoplugin;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JOptionPane;

import org.unicore.Vsite;
import org.unicore.resources.SoftwareResource;
import org.unicore.upl.ConsignJobReply;
import org.unicore.upl.Reply;

import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.client.trees.ContainerNode;
import com.pallas.unicore.client.util.ExtensionPlugable;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.threadpool.IObserver;
import com.pallas.unicore.utility.UserMessages;

/**
 *  Example for an AJO plugin. This plugin adds a button to the vsite
 *  list in the job group panel. When the button is pushed the name of a
 *  software resource is requested. If the specified software resource is
 *  defined in the IDB at that site it will be executed from a customized
 *  AJO that is send to the NJS by a self-written request. 
 *  
 *@author     Ralf Ratering
 *@version    $Id: AJOPlugin.java,v 1.3.2.1 2004/05/03 08:04:07 rraterin Exp $
 */
public class AJOPlugin extends ExtensionPlugable implements IObserver {
	private final static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.plugins.ajoplugin");
	private JButton startButton;

	/**
	 *  Get a short info message for the plugin
	 *
	 *@return    String with info message
	 */
	public String getPluginInfo() {
		return "UNICORE AJO plugin example\n"
			+ "Demonstrating the use of raw AJOs inside the Client"
			+ "(C)2004 Intel GmbH";
	}

	/**
	 *  Gets the vsiteToolBarComponent attribute of the SmallServicePlugin
	 *  object
	 *
	 *@return    The vsiteToolBarComponent value
	 */
	public Component getVsiteToolBarComponent() {
		// return a new instance each time...
		startButton.setToolTipText("An AJO plugin example");
		return startButton;
	}

	/**
	 *  This method implements the IObserver interface and will be executed
	 *  job submission has returned.
	 *
	 *@param  theObserved  reference to GetOutcome request
	 *@param  changeCode   the job outcome
	 */
	public void observableUpdate(Object theObserved, Object changeCode) {
		//analyze reply
		Reply reply = (Reply) changeCode;
		String message = "AJO has been successfully submitted";
		if (!(reply instanceof ConsignJobReply)) {
			message =
				"Could not submit AJO.\nReason: "
					+ reply.getLastEntry().getComment();
		}

		JOptionPane.showMessageDialog(getClient(), message);
	}

	/**
	 *  Check, if a software resource with a specific name is available at a
	 *  site. For convenience, we do not look for versions here... 
	 *
	 *@param  vsite  site to look at
	 *@return        true, if resource is available
	 */
	private SoftwareResource findResource(String softwareName, Vsite vsite) {
		// get all resources from site
		NamedResourceSet siteResources = ResourceManager.getResourceSet(vsite);
		SoftwareResource serviceSoftware = null;
		// selecte only software resources
		Vector siteSoftwares = new Vector();
		siteResources.addSoftwareResources(siteSoftwares);
		// loop over software resources
		for (int i = 0; i < siteSoftwares.size(); i++) {
			if (siteSoftwares.elementAt(i) instanceof SoftwareResource) {
				SoftwareResource siteSoftware =
					(SoftwareResource) siteSoftwares.elementAt(i);
				if (siteSoftware.getName().equals(softwareName)) {
					// success!
					serviceSoftware = siteSoftware;
				}
			}
		}
		return serviceSoftware;
	}

	/**
	 *  Called to inititialize the plugin
	 */
	public void startPlugin() {
		startButton = new JButton(new ServiceAction());
	}

	/**
	 *  Stop plugin
	 */
	public void stopPlugin() {
		// do nothing
	}

	/**
	 *  Submit the service job
	 *
	 *@param  vsite  Vsite where the job will be submitted
	 */
	private void submitServiceJob(SoftwareResource software, Vsite vsite) {
		AJORequest request = new AJORequest(software, vsite);
		request.addObserver(this);
		request.start();
	}

	private class ServiceAction extends javax.swing.AbstractAction {
		public ServiceAction() {
			super("", ResourceManager.getIcon(ResourceManager.SUBMIT));
		}

		public void actionPerformed(ActionEvent event) {
			// get selected job from jpa tree
			ContainerNode jobNode =
				getClient().getJPATree().getCurrentJobNode();
			if (jobNode == null) {
				return;
			}

			// apply current gui entries to job container
			JPAPanel panel = (JPAPanel) jobNode.getPanel();
			panel.applyValues();

			// get vsite from job container
			JobContainer jobContainer =
				(JobContainer) jobNode.getActionContainer();
			Vsite vsite = jobContainer.getVsite();

			if (vsite == null) {
				UserMessages.info("No virtual site selected.");
				return;
			}

			// ask for resource name
			String softwareName =
				(String) JOptionPane.showInputDialog(
					getClient(),
					"Enter name of Software Resource to be executed.",
					"UNICORE: AJO Plugin",
					JOptionPane.QUESTION_MESSAGE,
					null,
					null,
					"dummySoftware");

			if (softwareName == null) {
				return;
			}

			// if SoftwareResource is not available at the site get out.
			SoftwareResource software = findResource(softwareName, vsite);
			if (software == null) {
				UserMessages.error(
					"A SoftwareResource with name <"
						+ softwareName
						+ "> is not available at site "
						+ vsite);
				return;
			}

			// ask if we should really start now...
			if (JOptionPane
				.showConfirmDialog(
					getClient(),
					"Execute software <"
						+ softwareName
						+ "> at site "
						+ vsite
						+ "?",
					"UNICORE: AJO Plugin",
					JOptionPane.YES_NO_OPTION,
					JOptionPane.QUESTION_MESSAGE)
				== JOptionPane.YES_OPTION) {
				// submit job...
				submitServiceJob(software, vsite);
			}
		}
	}

}
