/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InvalidClassException;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import com.pallas.unicore.client.util.ClientPluginManager;
import com.pallas.unicore.client.util.PluginObjectInputStream;
import com.pallas.unicore.client.xml.XMLObjectReader;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.requests.SubmitJob;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 * @author Thomas Kentemich
 * @version $Id: CommandLineClient.java,v 1.1 2004/05/25 14:58:52 rmenday Exp $
 */
public class CommandLineClient {

	private final static String HELP = "-help";
	private final static String JOBS = "-jobs";
	private final static String KEYSTORE = "-keystore";
	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.client");
	private final static String PASSWORD = "-password";
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.ResourceStrings");
	private Vector jobNames = new Vector();
	private String keystoreName;
	private String keystorePassword = "";
	private ClientPluginManager pluginManager = Client.getPluginManager();
	private Vector runJobs = new Vector();
	private UserDefaults userDefaults;

	/**
	 * Constructor for the CommandLineClient object
	 * 
	 * @param args
	 *            Description of Parameter
	 */
	public CommandLineClient(String[] args) {
		userDefaults = ResourceManager.getUserDefaults();
		keystoreName = userDefaults.getKeyStoreName();
		parseArgs(args);
		File file = new File(keystoreName);
		if (file.exists()) {
			UserMessages.error("Keystore: " + keystoreName + " not found");
		}
		try {
			ResourceManager.loadKeystore(keystoreName, keystorePassword
					.toCharArray());
		}
		catch (Exception e) {
			logger
					.info("Could not unlock keystore, because exception occured: "
							+ e.getMessage());
		}
		pluginManager.stopPlugins();
		pluginManager.scanPlugins(userDefaults.getPluginDirectory());
		for (int i = 0; i < jobNames.size(); i++) {
			JobContainer next = loadJob((String) jobNames.elementAt(i));
			if (next != null) {
				// already done in loadJob
				// next.setFilename((String)jobNames.elementAt(i));
				// next.checkContents();
				if (next.isComplete()) {
					runJobs.add(next);
					logger.info("Loaded job: " + next);
				}
			}
		}
		for (int i = 0; i < runJobs.size(); i++) {
			JobContainer job = (JobContainer) runJobs.elementAt(i);
			SubmitJob submit = new SubmitJob(job);
			submit.start();
			while (job.getState() != ActionContainer.STATE_SUBMITTED) {
				try {
					Thread.sleep(1000);
				}
				catch (InterruptedException e) {
					logger.log(Level.SEVERE, "Thread interrupted", e);
				}
			}
		}
	}

	/**
	 * Load a job from file and insert it into the JPA tree as a new branch.
	 * 
	 * @param filename
	 *            Description of Parameter
	 * @return Description of the Return Value
	 */
	private JobContainer loadJob(final String filename) {
		final JobContainer jc;
		String name = filename;
		if (filename.endsWith(".xml")) {
			jc = readJobFromFileXML(filename);
			name = filename.substring(0, filename.length() - 4) + ".ajo";
		}
		else {
			jc = readJobFromFile(filename);
		}
		// if loading failed return
		if (jc == null) {
			return null;
		}
		//	jc.setFilename(filename);
		jc.setFilename(name);
		jc.checkContents();
		// reset the email in any case; this avoids misdirected mails !!
		// Removed: job shall not be edited
		// recursiveSetUser(jc);
		return jc;
	}

	/**
	 * Description of the Method
	 * 
	 * @param args
	 *            Description of Parameter
	 */
	private void parseArgs(String[] args) {
		boolean passfound = false;
		String usage = "Usage: java -jar client.jar -nogui -password <password> "
				+ "[-keystore <keystore>] [-jobs <job1#job2#...>]";
		for (int i = 0; i < args.length; i++) {
			if (args[i].equals(KEYSTORE)) {
				if ((i + 1) < args.length) {
					keystoreName = args[i + 1];
					logger.info("New keystore: " + keystoreName);
				}
			}
			else if (args[i].equals(PASSWORD)) {
				if ((i + 1) < args.length) {
					keystorePassword = args[i + 1];
					logger.info("New password.");
					passfound = true;
				}
			}
			else if (args[i].equals(JOBS)) {
				if ((i + 1) < args.length) {
					String jobs = args[i + 1];
					logger.info("New jobs: " + jobs);
					StringTokenizer tok = new StringTokenizer(jobs, "#");
					while (tok.hasMoreTokens()) {
						String nextJob = tok.nextToken();
						logger.info("Add job: " + nextJob);
						jobNames.add(nextJob);
					}
				}
			}
			else if (args[i].equals(HELP)) {
				System.out.println(usage);
			}
		}
		if (!passfound) {
			logger.info(usage);
			System.exit(1);
		}
	}

	/**
	 * Read a job from file.
	 * 
	 * @param name
	 *            Description of Parameter
	 * @return new JobContainer
	 */
	private JobContainer readJobFromFile(String name) {
		byte[] in = null;
		try {
			File file = new File(name);
			if (!file.isAbsolute()) {
				if (userDefaults.getJobDirectory() != null) {
					String newName = userDefaults.getJobDirectory()
							+ File.separator + name;
					file = new File(newName);
					logger.info("Switched path to: " + newName);
				}
			}
			FileInputStream f = new FileInputStream(file);
			DataInputStream dis = new DataInputStream(f);
			in = new byte[f.available()];
			dis.readFully(in);
			dis.close();
			f.close();
		}
		catch (IOException e) {
			UserMessages.error("Could not load file: " + name, e.getMessage());
			return null;
		}
		Object o = null;
		if (in != null) {
			PluginObjectInputStream ois = null;
			try {
				ois = new PluginObjectInputStream(new ByteArrayInputStream(in),
						pluginManager);
				o = ois.readObject();
				ois.close();
			}
			// this might be a ClassNotFoundException when we tried to load a
			// plugin task that is not available
			catch (ClassNotFoundException cnfe) {
				UserMessages.error("Plugin not available: " + name, cnfe
						.getMessage());
				return null;
			}
			catch (InvalidClassException cnfe) {
				UserMessages
						.error("Incompatible Java class found while reading saved UNICORE job.\n"
								+ "Possibly conflicting client versions.\n"
								+ "Please contact your administartor.");
				return null;
			}
			catch (Exception e) {
				// display complete exception in dialog
				UserMessages.error("Error while reading job group file: "
						+ name, e.getMessage());
				return null;
			}
			if (o instanceof com.pallas.unicore.container.JobContainer) {
				JobContainer jc = (JobContainer) o;
				// return the JobContainer
				return jc;
			}
		}
		// if we got here something went wrong
		return null;
	}

	/**
	 * Read a job from file in XML format.
	 * 
	 * @param filename
	 *            Description of the Parameter
	 * @return new JobContainer
	 */
	private JobContainer readJobFromFileXML(String filename) {
		if (!filename.endsWith(".xml")) {
			return null;
		}
		Object o = null;
		try {
			File file = new File(filename);
			if (!file.isAbsolute()) {
				if (userDefaults.getJobDirectory() != null) {
					String newName = userDefaults.getJobDirectory()
							+ File.separator + filename;
					file = new File(newName);
					logger.info("Switched path to: " + newName);
				}
			}
			XMLObjectReader din = new XMLObjectReader(file);
			o = din.readObjectXML();
		}
		catch (FileNotFoundException fnfe) {
			UserMessages.error(res.getString("COULD_NOT_LOAD_FILE"), fnfe
					.getMessage());
			logger.log(Level.SEVERE, "", fnfe);
			return null;
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "", e);
			return null;
		}
		if (o instanceof com.pallas.unicore.container.JobContainer) {
			JobContainer jc = (JobContainer) o;
			jc.setName(jc.getName() + "_XML");
			// return the JobContainer
			return jc;
		}
		else {
			// if we got here something went wrong
			return null;
		}
	}
}