/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client;

import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import com.pallas.unicore.utility.UserMessages;

/**
 * Define, load and save system default values
 * 
 * @author Ralf Ratering
 * @version $Id: SystemDefaults.java,v 1.1 2004/05/25 14:58:52 rmenday Exp $
 */
public class SystemDefaults extends Properties {

	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.client");
	/**
	 * String constants defining property keys
	 */
	public final static String USITE_URL_KEY = "systemdefault.usite.url",
			CRL_URL_KEY = "systemdefault.crl.url";
	public final String fallbackCRL = "";
	public final String fallbackURL = "http://www.unicore.de/unicoreSites.xml";

	/**
	 * Constructor for the SystemDefaults object
	 */
	public SystemDefaults() {
		super();
		// override with command line settings
		if (System.getProperty(USITE_URL_KEY) != null) {
			setProperty(USITE_URL_KEY, System.getProperty(USITE_URL_KEY));
		}
		else {
			setProperty(USITE_URL_KEY, fallbackURL);
		}
		if (System.getProperty(CRL_URL_KEY) != null) {
			setProperty(CRL_URL_KEY, System.getProperty(CRL_URL_KEY));
		}
		else {
			setProperty(CRL_URL_KEY, fallbackCRL);
		}
	}

	/**
	 * Gets the CrlUrl attribute of the SystemDefaults object
	 * 
	 * @return The CrlUrl value
	 */
	public String getCrlURL() {
		return getProperty(CRL_URL_KEY);
	}

	/**
	 * Overwrite getProperty method to prevent return value null and return ""
	 * instead. This is important to prevent null pointer execptions from faulty
	 * ScriptDefaults files.
	 * 
	 * @param prop
	 *            property
	 * @return The Property value
	 */
	public String getProperty(String prop) {
		String value = super.getProperty(prop);
		if (value == null) {
			UserMessages.warning("Could not read system default property: "
					+ prop);
			return "";
		}
		else {
			return value;
		}
	}

	/**
	 * Gets the ScriptDirectory attribute of the ScriptDefaults object
	 * 
	 * @return The ScriptDirectory value
	 */
	public String getUsiteURL() {
		return getProperty(USITE_URL_KEY);
	}

	/**
	 * Load system default values from file LIBDIR/SystemDefaults.txt
	 * 
	 * @param filename
	 * @return true if loading was successful, else false
	 */
	public boolean loadFromFile(String filename) {
		boolean retVal = false;
		boolean urlSet = false;
		logger.info("Trying to load: " + filename);
		File defs = new File(filename);
		if (defs.exists()) {
			// try to load file
			try {
				load(new FileInputStream(defs));
				retVal = true;
			}
			// loading failed
			catch (Exception e) {
				// no error message here, but in calling method
				logger.log(Level.SEVERE, "", e);
				retVal = false;
			}
		}
		// no LIBDIR/SystemDefaults.txt found
		else {
			retVal = false;
		}
		// override with command line settings
		if (System.getProperty(USITE_URL_KEY) != null) {
			setProperty(USITE_URL_KEY, System.getProperty(USITE_URL_KEY));
			urlSet = true;
		}
		if (System.getProperty(CRL_URL_KEY) != null) {
			setProperty(CRL_URL_KEY, System.getProperty(CRL_URL_KEY));
			retVal = true && urlSet;
		}
		// loading was successful
		return retVal;
	}
}