/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import java.io.File;
import java.security.cert.X509Certificate;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.plaf.metal.DefaultMetalTheme;
import org.unicore.User;
import org.unicore.Vsite;
import com.pallas.unicore.client.themes.LookAndFeelManager;
import com.pallas.unicore.client.themes.UserTheme;
import com.pallas.unicore.client.util.Defaults;
import com.pallas.unicore.connection.ProxyManager;
import com.pallas.unicore.extensions.Usite;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 * Define, load and save user default values
 * 
 * @author Ralf Ratering
 * @version $Id: UserDefaults.java,v 1.2 2004/05/27 15:37:11 ratering Exp $
 */
public class UserDefaults extends Defaults {

	/**
	 * Strings for different formats
	 */
	public final static String BINARY_FORMAT = "serialized",
			XML_FORMAT = "XML", PLAIN_FORMAT = "PLAIN";
	/**
	 * prefix for default identity
	 */
	public final static String DEFAULT = "default::";
	private final static Logger logger = Logger
			.getLogger("com.pallas.unicore.client");
	// property string constants
	private final static String USITEURL = "unicore.usiteurl";
	private final static String CRLURL = "unicore.crl";
	private final static String PROXY_HOST = "unicore.proxy.host";
	private final static String PROXY_PORT = "unicore.proxy.port";
	private final static String NO_PROXY = "unicore.proxy.noproxy";
	private final static String PROXY_ENABLED = "unicore.proxy.enabled";
	private final static String SOCKS_HOST = "unicore.socks.host";
	private final static String SOCKS_PORT = "unicore.socks.port";
	private final static String SOCKS_ENABLED = "unicore.socks.enabled";
	private final static String PLUGINDIR = "unicore.plugin.directory";
	private final static String JOBDIR = "unicore.job.directory";
	private final static String RESOURCEDIR = "unicore.resource.directory";
	private final static String IMPORTDIR = "unicore.import.directory";
	private final static String EXPORTDIR = "unicore.export.directory";
	private final static String LOGDIR = "unicore.log.directory";
	private final static String KEYSTOREDIR = "unicore.keystore.path";
	private final static String TMPDIR = "unicore.keystore.tmp";
	private final static String REDIRECT = "unicore.redirect.stdout";
	private final static String USEREMAIL = "unicore.email";
	private final static String XLOGIN = "unicore.xlogin";
	private final static String PROJECT = "unicore.project";
	private final static String DEFAULTIDENTITY = "unicore.identity";
	private final static String LOOKANDFEEL = "unicore.plaf";
	private final static String JOBSTORAGEFORMAT = "unicore.jobformat";
	private final static String AUTOFETCHOUTCOME = "unicore.autofetch.outcome";
	private final static String DETAILEDJOBSTATUS = "unicore.detailed.jobstatus";
	private final static String STARTOFFLINE = "unicore.startoffline";
	private final static String SOCKET_TIMEOUT = "unicore.sockettimeout";
	private final static String THEME = "unicore.theme";
	private final static String THEME_PRIMARY1 = "unicore.theme.primary1";
	private final static String THEME_PRIMARY2 = "unicore.theme.primary2";
	private final static String THEME_PRIMARY3 = "unicore.theme.primary3";
	private final static String THEME_SECONDARY1 = "unicore.theme.secondary1";
	private final static String THEME_SECONDARY2 = "unicore.theme.secondary2";
	private final static String THEME_SECONDARY3 = "unicore.theme.secondary3";
	private final static String THEME_WHITE = "unicore.theme.white";
	private final static String THEME_BLACK = "unicore.theme.black";
	private final static String THEME_FONT_NAME = "unicore.theme.font.name";
	private final static String THEME_FONT_SIZE = "unicore.theme.font.size";
	private final static String WINDOW_SIZE_X = "unicore.window.size.x";
	private final static String WINDOW_SIZE_Y = "unicore.window.size.y";
	private final static String WINDOW_POS_X = "unicore.window.pos.x";
	private final static String WINDOW_POS_Y = "unicore.window.pos.y";
	private final static String LOG_LEVEL = "unicore.log.level";
	private final static String LOG_FORMAT = "unicore.log.format";
	private final static String PREEMPTIVE_FILESELECTOR_MODE = "unicore.preemptive.mode";
	private final static String IDENTITY = "unicore.identity.";
	// user home directory
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.ResourceStrings");

	/**
	 * Set GUI Theme
	 * 
	 * @param theme
	 *            The new Theme
	 */
	public void setTheme(DefaultMetalTheme theme) {
		put(THEME, theme.getName());
		if (theme instanceof UserTheme) {
			UserTheme userTheme = (UserTheme) theme;
			put(THEME_PRIMARY1, colorString(userTheme.getPrimary1Color()));
			put(THEME_PRIMARY2, colorString(userTheme.getPrimary2Color()));
			put(THEME_PRIMARY3, colorString(userTheme.getPrimary3Color()));
			put(THEME_SECONDARY1, colorString(userTheme.getSecondary1Color()));
			put(THEME_SECONDARY2, colorString(userTheme.getSecondary2Color()));
			put(THEME_SECONDARY3, colorString(userTheme.getSecondary3Color()));
			put(THEME_BLACK, colorString(userTheme.getBlackColor()));
			put(THEME_WHITE, colorString(userTheme.getWhiteColor()));
			put(THEME_FONT_NAME, userTheme.getControlTextFont().getName());
			put(THEME_FONT_SIZE, new Integer(userTheme.getControlTextFont()
					.getSize()).toString());
		}
	}

	/**
	 * Sets user identity for a vsite
	 * 
	 * @param vsite
	 *            destination vsite
	 * @param value
	 *            certificate alias
	 */
	public void setIdentity(Vsite vsite, String value) {
		String key = IDENTITY + vsite.getAddress() + "." + vsite.toString();
		if (value.equals(DEFAULT)) {
			String identity = (String) get(key);
			if (identity == null) {
				return;
			}
			if (!identity.equals("")) {
				this.remove(key);
			}
			return;
		}
		put(key, value);
	}

	/**
	 * Set user identity for a Usite
	 * 
	 * @param usite
	 *            destination Usite
	 * @param value
	 *            certificate alias
	 */
	public void setIdentity(Usite usite, String value) {
		String key = IDENTITY + usite.toString();
		if (value.equals(DEFAULT)) {
			String identity = (String) get(key);
			if (identity == null) {
				return;
			}
			if (!identity.equals("")) {
				this.remove(key);
			}
			return;
		}
		put(key, value);
	}

	/**
	 * Set the LookAndFeel
	 * 
	 * @param value
	 *            The new LookAndFeel value
	 */
	public void setLookAndFeel(String value) {
		put(LOOKANDFEEL, value);
	}

	/**
	 * Set URL where available usites can be requested
	 * 
	 * @param url
	 *            The new UsiteURL value
	 */
	public void setUsiteURL(String url) {
		put(USITEURL, url);
	}

	/**
	 * Set URL where the CRL can be requested
	 * 
	 * @param url
	 *            The new UsiteURL value
	 */
	public void setCrlURL(String url) {
		put(CRLURL, url);
	}

	/**
	 * Set the client log level
	 * 
	 * @param level
	 *            The new log level
	 */
	public void setLogLevel(String level) {
		put(LOG_LEVEL, level);
	}

	/**
	 * Set client file chooser to preemptive mode
	 * 
	 * @param mode
	 *            true, if file chooser uses preemptive mode
	 */
	public void setPreemptiveFileSelectorMode(boolean mode) {
		put(PREEMPTIVE_FILESELECTOR_MODE, new Boolean(mode).toString());
	}

	/**
	 * Set the client log format
	 * 
	 * @param format
	 *            The new log format
	 */
	public void setLogFormat(String format) {
		put(LOG_FORMAT, format);
	}

	/**
	 * Set default email address
	 * 
	 * @param str
	 *            email address
	 */
	public void setUserEmail(String str) {
		put(USEREMAIL, str);
		try {
			ResourceManager.updateDefaultIdentity();
		}
		catch (Exception e) {
			UserMessages.print("Setting User Email failed.", e.getMessage());
		}
	}

	/**
	 * Set default xlogin
	 * 
	 * @param str
	 *            xlogin
	 */
	public void setXlogin(String str) {
		put(XLOGIN, str);
		try {
			ResourceManager.updateDefaultIdentity();
		}
		catch (Exception e) {
			UserMessages.print("Setting Xlogin failed.", e.getMessage());
		}
	}

	/**
	 * Set default project
	 * 
	 * @param str
	 *            project
	 */
	public void setProject(String str) {
		put(PROJECT, str);
		try {
			ResourceManager.updateDefaultIdentity();
		}
		catch (Exception e) {
			UserMessages.print("Setting Project failed.", e.getMessage());
		}
	}

	/**
	 * Set default identity
	 * 
	 * @param str
	 *            identity
	 */
	public void setDefaultIdentity(String str) {
		put(DEFAULTIDENTITY, str);
	}

	/**
	 * Set directory that is scanned for plugins
	 * 
	 * @param dirname
	 *            The new PluginDirectory value
	 */
	public void setPluginDirectory(String dirname) {
		put(PLUGINDIR, dirname);
	}

	/**
	 * Set directory where stdout/stderr and logs are stored
	 * 
	 * @param dirname
	 *            The new LogDirectory value
	 */
	public void setLogDirectory(String dirname) {
		put(LOGDIR, dirname);
	}

	/**
	 * Set directory where tmp files are stored
	 * 
	 * @param dirname
	 *            The new TmpDirectory value
	 */
	public void setTmpDirectory(String dirname) {
		put(TMPDIR, dirname);
	}

	/**
	 * Set name for key store
	 * 
	 * @param s
	 *            The new KeyStoreName value
	 */
	public void setKeyStoreName(String s) {
		put(KEYSTOREDIR, s);
	}

	/**
	 * Set path to job directory
	 * 
	 * @param s
	 *            path to job directory
	 */
	public void setJobDirectory(String s) {
		put(JOBDIR, s);
	}

	/**
	 * Set path to resource directory
	 * 
	 * @param s
	 *            path to resource directory
	 */
	public void setResourceDirectory(String s) {
		put(RESOURCEDIR, s);
	}

	/**
	 * Set path to import directory
	 * 
	 * @param s
	 *            path to import directory
	 */
	public void setImportDirectory(String s) {
		put(IMPORTDIR, s);
	}

	/**
	 * Set path to export directory
	 * 
	 * @param s
	 *            path to export directory
	 */
	public void setExportDirectory(String s) {
		put(EXPORTDIR, s);
	}

	/**
	 * Redirect stdout/stderr and log to file
	 * 
	 * @param b
	 *            true, if redirected
	 */
	public void setRedirect(boolean b) {
		put(REDIRECT, new Boolean(b).toString());
	}

	/**
	 * Start client offline?
	 * 
	 * @param b
	 *            true, if client starts offline
	 */
	public void setStartOffline(boolean b) {
		put(STARTOFFLINE, new Boolean(b).toString());
	}

	/**
	 * Use a proxy?
	 * 
	 * @param enabled
	 *            true if use a proxy
	 */
	public void setProxyEnabled(boolean enabled) {
		put(PROXY_ENABLED, new Boolean(enabled).toString());
	}

	/**
	 * Proxy host name
	 * 
	 * @param host
	 *            Proxy host name
	 */
	public void setProxyHost(String host) {
		setProperty(PROXY_HOST, host);
	}

	/**
	 * Proxy host port
	 * 
	 * @param port
	 *            Proxy host port
	 */
	public void setProxyPort(int port) {
		put(PROXY_PORT, new Integer(port).toString());
	}

	/**
	 * Set hosts that should be contacted without proxy
	 * 
	 * @param noProxy
	 *            List of hosts
	 */
	public void setNoProxy(String noProxy) {
		setProperty(NO_PROXY, noProxy);
	}

	/**
	 * Use a socks proxy?
	 * 
	 * @param enabled
	 *            true if use a socks proxy
	 */
	public void setSocksEnabled(boolean enabled) {
		put(SOCKS_ENABLED, new Boolean(enabled).toString());
	}

	/**
	 * Socks Proxy host name
	 * 
	 * @param host
	 *            Socks proxy host name
	 */
	public void setSocksHost(String host) {
		setProperty(SOCKS_HOST, host);
	}

	/**
	 * Proxy host port
	 * 
	 * @param port
	 *            Proxy host port
	 */
	public void setSocksPort(int port) {
		put(SOCKS_PORT, new Integer(port).toString());
	}

	/**
	 * Set the job storage format
	 * 
	 * @param what
	 *            job storage format
	 */
	public void setJobStorageFormat(String what) {
		put(JOBSTORAGEFORMAT, what);
	}

	/**
	 * automatically fetch spooled files with outcome?
	 * 
	 * @param what
	 *            true, if autofetch on
	 */
	public void setAutofetchOutcome(boolean what) {
		put(AUTOFETCHOUTCOME, new Boolean(what).toString());
	}

	/**
	 * Sets the detailed job status display
	 * 
	 * @param what
	 *            true, if detailed display
	 */
	public void setDetailedJobStatus(boolean what) {
		put(DETAILEDJOBSTATUS, new Boolean(what).toString());
	}

	/**
	 * Set default window size
	 * 
	 * @param dim
	 *            window size
	 */
	public void setWindowSize(Dimension dim) {
		put(WINDOW_SIZE_X, new Integer(dim.width).toString());
		put(WINDOW_SIZE_Y, new Integer(dim.height).toString());
	}

	/**
	 * Set default window position
	 * 
	 * @param pos
	 *            window position
	 */
	public void setWindowPos(Point pos) {
		put(WINDOW_POS_X, new Integer(pos.x).toString());
		put(WINDOW_POS_Y, new Integer(pos.y).toString());
	}

	/**
	 * Sets the timeout value for connect and read operations
	 * 
	 * @param timeout
	 *            The new timeout value
	 */
	public void setSocketTimeout(int timeout) {
		put(SOCKET_TIMEOUT, new Integer(timeout).toString());
	}

	/**
	 * Get theme
	 * 
	 * @return current theme
	 */
	public DefaultMetalTheme getTheme() {
		String themeName = (String) get(THEME);
		if (themeName == null) {
			return new DefaultMetalTheme();
		}
		DefaultMetalTheme theme = LookAndFeelManager.getThemeByName(themeName);
		if (theme instanceof UserTheme) {
			UserTheme userTheme = (UserTheme) theme;
			userTheme
					.setPrimary1Color(colorValue((String) get(THEME_PRIMARY1)));
			userTheme
					.setPrimary2Color(colorValue((String) get(THEME_PRIMARY2)));
			userTheme
					.setPrimary3Color(colorValue((String) get(THEME_PRIMARY3)));
			userTheme
					.setSecondary1Color(colorValue((String) get(THEME_SECONDARY1)));
			userTheme
					.setSecondary2Color(colorValue((String) get(THEME_SECONDARY2)));
			userTheme
					.setSecondary3Color(colorValue((String) get(THEME_SECONDARY3)));
			userTheme.setBlackColor(colorValue((String) get(THEME_BLACK)));
			userTheme.setWhiteColor(colorValue((String) get(THEME_WHITE)));
			String fontName = (String) get(THEME_FONT_NAME);
			int fontSize = intValue(THEME_FONT_SIZE);
			userTheme.setFont(new Font(fontName, Font.PLAIN, fontSize));
		}
		return theme;
	}

	/**
	 * Gets the LookAndFeel
	 * 
	 * @return The LookAndFeel value
	 */
	public String getLookAndFeel() {
		String lookAndFeel = (String) get(LOOKANDFEEL);
		if (lookAndFeel == null) {
			lookAndFeel = "javax.swing.plaf.metal.MetalLookAndFeel";
			put(LOOKANDFEEL, lookAndFeel);
		}
		return lookAndFeel;
	}

	/**
	 * Get URL where available usites can be requested
	 * 
	 * @return String containing URL
	 */
	public String getUsiteURL() {
		return getProperty(USITEURL);
	}

	/**
	 * Get URL where CRL can be requested
	 * 
	 * @return String containing URL
	 */
	public String getCrlURL() {
		return getProperty(CRLURL);
	}

	/**
	 * Get default email address
	 * 
	 * @return default email address
	 */
	public String getUserEmail() {
		return getProperty(USEREMAIL);
	}

	/**
	 * Get default xlogin
	 * 
	 * @return default xlogin
	 */
	public String getXlogin() {
		return getProperty(XLOGIN);
	}

	/**
	 * Get default project
	 * 
	 * @return default project
	 */
	public String getProject() {
		return getProperty(PROJECT);
	}

	/**
	 * Get default identity. This one's always lower case, because the keystore
	 * will only handle lower case aliases.
	 * 
	 * @return default identity
	 */
	public String getDefaultIdentity() {
		return getProperty(DEFAULTIDENTITY).toLowerCase();
	}

	/**
	 * Get directory that is scanned for plugins
	 * 
	 * @return plugin directory
	 */
	public String getPluginDirectory() {
		return getProperty(PLUGINDIR);
	}

	/**
	 * Get directory where stdout/stderr and logs are stored
	 * 
	 * @return log directory
	 */
	public String getLogDirectory() {
		String logDir = getProperty(LOGDIR);
		if (logDir == null || logDir.equals("")) {
			logDir = ".";
		}
		return logDir;
	}

	/**
	 * Get directory where tmp files are stored
	 * 
	 * @return tmp directory
	 */
	public String getTmpDirectory() {
		return getProperty(TMPDIR);
	}

	/**
	 * Get name for key store
	 * 
	 * @return default key store name
	 */
	public String getKeyStoreName() {
		return getProperty(KEYSTOREDIR);
	}

	/**
	 * Get path to job directory
	 * 
	 * @return String containing last job path
	 */
	public String getJobDirectory() {
		return getProperty(JOBDIR);
	}

	/**
	 * Get path to resource directory
	 * 
	 * @return path to resource directory
	 */
	public String getResourceDirectory() {
		return getProperty(RESOURCEDIR);
	}

	/**
	 * Get path to import directory
	 * 
	 * @return path to import directory
	 */
	public String getImportDirectory() {
		return getProperty(IMPORTDIR);
	}

	/**
	 * Get path to export directory
	 * 
	 * @return path to export directory
	 */
	public String getExportDirectory() {
		return getProperty(EXPORTDIR);
	}

	/**
	 * Redirect console output to file?
	 * 
	 * @return true if redirected, else false
	 */
	public boolean getRedirect() {
		return booleanValue(REDIRECT);
	}

	/**
	 * Start offline?
	 * 
	 * @return true if starting offline
	 */
	public boolean getStartOffline() {
		return booleanValue(STARTOFFLINE);
	}

	/**
	 * Fetch spooled files with outcome?
	 * 
	 * @return true if yes
	 */
	public boolean getAutofetchOutcome() {
		return booleanValue(AUTOFETCHOUTCOME);
	}

	/**
	 * Get default value for job status display
	 * 
	 * @return true if detailed status is enabled
	 */
	public boolean getDetailedJobStatus() {
		return booleanValue(DETAILEDJOBSTATUS);
	}

	/**
	 * Use proxy?
	 * 
	 * @return true, if yes
	 */
	public boolean isProxyEnabled() {
		return booleanValue(PROXY_ENABLED);
	}

	/**
	 * Get proxy hostname
	 * 
	 * @return proxy hostname
	 */
	public String getProxyHost() {
		return getProperty(PROXY_HOST);
	}

	/**
	 * Get proxy port
	 * 
	 * @return proxy port
	 */
	public int getProxyPort() {
		return intValue(PROXY_PORT);
	}

	/**
	 * Use socks proxy?
	 * 
	 * @return true, if yes
	 */
	public boolean isSocksEnabled() {
		return booleanValue(SOCKS_ENABLED);
	}

	/**
	 * Get socks proxy hostname
	 * 
	 * @return socks proxy hostname
	 */
	public String getSocksHost() {
		return getProperty(SOCKS_HOST);
	}

	/**
	 * Get proxy port
	 * 
	 * @return proxy port
	 */
	public int getSocksPort() {
		return intValue(SOCKS_PORT);
	}

	/**
	 * get hosts excluded from proxy use
	 * 
	 * @return hosts excluded from proxy use
	 */
	public String getNoProxy() {
		return getProperty("unicore.proxy.noproxy");
	}

	/**
	 * Get log level
	 * 
	 * @return log level
	 */
	public String getLogLevel() {
		String level = (String) get(LOG_LEVEL);
		if (level == null) {
			level = Level.INFO.toString();
			put(LOG_LEVEL, level);
		}
		return level;
	}

	/**
	 * Get user identity for a vsite
	 * 
	 * @param vsite
	 *            destination vsite
	 * @return user cert alias
	 */
	public String getIdentity(Vsite vsite) {
		String key = IDENTITY + vsite.getAddress() + "." + vsite.toString();
		String identity = (String) get(key);
		if (identity == null || identity.equals("")) {
			// look for identity of corresponding Usite
			Usite usite = ResourceManager.getUsite(vsite);
			identity = getIdentity(usite);
		}
		return identity;
	}

	/**
	 * Get user identity for a usite
	 * 
	 * @param usite
	 *            destination usite
	 * @return user cert alias
	 */
	public String getDefaultIdentity(Usite usite) {
		String key = IDENTITY + usite.toString();
		String identity = (String) get(key);
		if (identity == null || identity.equals("")) {
			identity = "";
			String cert = getDefaultIdentity();
			try {
				if (!ResourceManager.getKeystoreManager()
						.keyStoreContainsAlias(cert)) {
					return identity;
				}
			}
			catch (Exception ex) {
				logger.log(Level.WARNING, "Could not read alias ifromkeystore",
						ex);
			}
			identity = cert + ":" + getXlogin() + ":" + getProject();
		}
		return identity;
	}

	/**
	 * Get user identity for a usite, if none is explicitly set, return default
	 * string
	 * 
	 * @param usite
	 *            destination usite
	 * @return user cert alias or default string
	 */
	public String getIdentity(Usite usite) {
		String key = IDENTITY + usite.toString();
		String identity = (String) get(key);
		if (identity == null || identity.equals("")) {
			identity = DEFAULT;
		}
		return identity;
	}

	/**
	 * Get User object for a usite
	 * 
	 * @param usite
	 *            destination usite
	 * @return User object
	 */
	public User getUser(Usite usite) {
		String identity = getIdentity(usite);
		if (DEFAULT.equals(identity)) {
			identity = getDefaultIdentity();
		}
		User userUsite = getUser(identity);
		if (userUsite == null) {
			logger.severe("user " + identity + " of the usite: " + usite
					+ " is not available");
		}
		return userUsite;
	}

	/**
	 * Get User object for a vsite
	 * 
	 * @param vsite
	 *            destination vsite
	 * @return User object
	 */
	public User getUser(Vsite vsite) {
		String identity = getIdentity(vsite);
		if (DEFAULT.equals(identity)) {
			identity = getDefaultIdentity();
		}
		User userVsite = getUser(identity);
		if (userVsite == null) {
			logger.severe("user " + identity + " of the vsite: " + vsite
					+ " is not available");
		}
		return userVsite;
	}

	/**
	 * Get log format
	 * 
	 * @return log format
	 */
	public String getLogFormat() {
		String format = (String) get(LOG_FORMAT);
		if (format == null) {
			format = PLAIN_FORMAT;
			put(LOG_FORMAT, format);
		}
		return format;
	}

	/**
	 * Get default window size
	 * 
	 * @return window size
	 */
	public Dimension getWindowSize() {
		return new Dimension(intValue(WINDOW_SIZE_X), intValue(WINDOW_SIZE_Y));
	}

	/**
	 * Get default window position
	 * 
	 * @return window position
	 */
	public Point getWindowPos() {
		return new Point(intValue(WINDOW_POS_X), intValue(WINDOW_POS_Y));
	}

	/**
	 * Get default value for job storage format
	 * 
	 * @return default storage format
	 */
	public String getJobStorageFormat() {
		String format = getProperty(JOBSTORAGEFORMAT);
		if (format == null) {
			format = BINARY_FORMAT;
			put(JOBSTORAGEFORMAT, format);
		}
		return format;
	}

	/**
	 * Run file chooser in preemptive mode?
	 * 
	 * @return true, if yes
	 */
	public boolean getPreemptiveFileSelectorMode() {
		return booleanValue(PREEMPTIVE_FILESELECTOR_MODE);
	}

	/**
	 * Gets the socket timeout value
	 * 
	 * @return The socket timeout
	 */
	public int getSocketTimeout() {
		String timeout = getProperty(SOCKET_TIMEOUT);
		if (timeout == null || timeout.equals("")) {
			timeout = "60000";
			put(SOCKET_TIMEOUT, timeout);
		}
		return Integer.parseInt(timeout);
	}

	/**
	 * Get User object for an alias
	 * 
	 * @param identity -
	 *            a string format of "identity:xLogin:project"
	 * @return User object
	 */
	private User getUser(String identity) {
		String alias = null;
		String project = "";
		String xLogin = "";
		final String delimiter = ":";
		int endIndex = identity.length();
		int beginIndex = identity.lastIndexOf(delimiter);
		if (beginIndex > -1) {
			project = identity.substring(beginIndex + 1);
			endIndex = beginIndex;
			beginIndex = identity.lastIndexOf(delimiter, endIndex - 1);
			if (beginIndex > -1) {
				xLogin = identity.substring(beginIndex + 1, endIndex);
				endIndex = beginIndex;
			}
		}
		try {
			alias = identity.substring(0, endIndex);
			User returnedUser = null;
			X509Certificate[] x509 = ResourceManager.getKeystoreManager()
					.getCertificateByAlias(alias);
			if (x509 != null) {
				returnedUser = new User(x509[0], null); //before ajo 4.1 
				if (!xLogin.equals("")) {
					returnedUser.setXlogin(xLogin);
				}
				if (!project.equals("")) {
					returnedUser.setProject(project);
				}
			}
			return returnedUser;
		}
		catch (Exception ex) {
			logger.log(Level.SEVERE, "Could not read certificate " + alias
					+ " from keystore.", ex);
			return null;
		}
	}

	/**
	 * Store UserDefaults
	 * 
	 * @return true, if success
	 */
	public boolean writeToFile() {
		// always overwrite old ${HOME}/.unicoreXX/UserDefaults.txt
		return writeToFile(unicoreDir + File.separator + "UserDefaults.txt");
	}

	/**
	 * Load UserDefaults
	 * 
	 * @return true, if success
	 */
	public boolean loadFromFile() {
		// look for ${HOME}/.unicoreXX/UserDefaults.txt
		return loadFromFile(unicoreDir + File.separator + "UserDefaults.txt");
	}

	/**
	 * Fill properties with default values
	 */
	protected void genSystemDefaults() {
		// look for home directory
		String home = "." + File.separator;
		try {
			home = System.getProperty("user.home");
		}
		catch (SecurityException se) {
			UserMessages.print(res.getString("NO_PERMISSION"));
		}
		String exampleDir = ResourceManager.getJarDirectory() + File.separator
				+ ".." + File.separator + "examples";
		put(JOBDIR, exampleDir);
		put(RESOURCEDIR, exampleDir);
		put(IMPORTDIR, home);
		put(EXPORTDIR, home);
		put(LOGDIR, home);
		put(TMPDIR, unicoreDir);
		put(USEREMAIL, "");
		put(XLOGIN, "");
		put(PROJECT, "");
		put(USITEURL, ResourceManager.getSystemDefaults().getUsiteURL());
		put(CRLURL, ResourceManager.getSystemDefaults().getCrlURL());
		put(KEYSTOREDIR, unicoreDir + File.separator + "keystore");
		put(REDIRECT, Boolean.TRUE.toString());
		put(AUTOFETCHOUTCOME, Boolean.TRUE.toString());
		put(DETAILEDJOBSTATUS, Boolean.FALSE.toString());
		put(PROXY_ENABLED, Boolean.FALSE.toString());
		put(PROXY_HOST, "");
		put(PROXY_PORT, "8080");
		put(NO_PROXY, "");
		put(SOCKS_ENABLED, Boolean.FALSE.toString());
		put(SOCKS_HOST, "");
		put(SOCKS_PORT, "0");
		put(PLUGINDIR, home + File.separator + "plugins");
		put(LOOKANDFEEL, "javax.swing.plaf.metal.MetalLookAndFeel");
		put(THEME, LookAndFeelManager.getStandardTheme());
		put(JOBSTORAGEFORMAT, BINARY_FORMAT);
		put(STARTOFFLINE, Boolean.FALSE.toString());
		put(SOCKET_TIMEOUT, "60000");
		put(WINDOW_SIZE_X, "1024");
		put(WINDOW_SIZE_Y, "768");
		put(WINDOW_POS_X, "0");
		put(WINDOW_POS_Y, "0");
		put(LOG_LEVEL, Level.INFO.toString());
		put(LOG_FORMAT, PLAIN_FORMAT);
		put(PREEMPTIVE_FILESELECTOR_MODE, "false");
	}

	/**
	 * Get default value for a property
	 * 
	 * @param prop
	 *            property
	 * @return default value
	 */
	protected int defaultIntValue(String prop) {
		if (prop.equals(WINDOW_SIZE_X)) {
			return 700;
		}
		else if (prop.equals(WINDOW_SIZE_Y)) {
			return 700;
		}
		else if (prop.equals(WINDOW_POS_X)) {
			return 0;
		}
		else if (prop.equals(WINDOW_POS_Y)) {
			return 0;
		}
		return 0;
	}

	/**
	 * update proxy
	 */
	public void update() {
		if (isProxyEnabled()) {
			ProxyManager.useProxy(getProxyHost(), getProxyPort(), getNoProxy());
		}
		else {
			ProxyManager.noProxy();
		}
	}
}