/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.io.File;
import java.util.ResourceBundle;
import java.util.Vector;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.border.TitledBorder;
import org.unicore.AAIdentifier;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.ActionGroup_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.outcome.PutPortfolio_Outcome;
import org.unicore.sets.OutcomeEnumeration;
import com.pallas.unicore.client.tables.LocalExportTable;
import com.pallas.unicore.client.trees.ContainerNode;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.extensions.LocalExportEntry;
import com.pallas.unicore.requests.GetSpooledFiles;
import com.pallas.unicore.resourcemanager.OutcomeEntry;
import com.pallas.unicore.resourcemanager.OutcomeManager;

/**
 *  Display and select files that will be exported to nspace.
 *
 *@author      Thomas Kentemich
 *@author      Ralf Ratering
 *@version    $Id: OutputFilesDialog.java,v 1.1 2004/05/25 14:58:47 rmenday Exp $
 */
public class OutputFilesDialog extends GenericDialog {

	private static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.dialogs.ResourceStrings");
	private LocalExportTable fromList;
	private JobContainer jobContainer;
	private ContainerNode jobNode;
	private JFrame parent;
	private Vector spooledFiles;
	private Vector allExports = new Vector();
	private Vector transferJobs = new Vector();

	/**
	 * Consturctor builds gui and fetches output files
	 * 
	 * @param parent
	 *            reference to main frame
	 * @param jobNode
	 *            selected node in jmc
	 * @param spooledFiles
	 *            Vector of spooled files
	 */
	public OutputFilesDialog(JFrame parent, ContainerNode jobNode,
			Vector spooledFiles) {
		super(parent, res.getString("EXPORT_TITLE"), true,
				GenericDialog.OK_CANCEL);
		this.parent = parent;
		this.jobNode = jobNode;
		this.jobContainer = (JobContainer) jobNode.getUserObject();
		this.spooledFiles = spooledFiles;
		buildComponents();
		pack();
	}

	/**
	 * Get GetSpooledFiles tasks for selected files
	 * 
	 * @return selected GetSpooledFiles tasks
	 */
	public Vector getTransferJobs() {
		return transferJobs;
	}

	/**
	 * If we cancel the dialog nothing will happen
	 * 
	 * @return true always
	 */
	protected boolean cancelValues() {
		return true;
	}

	/**
	 * If we pressed ok, the selected files will be exported to nspace.
	 * 
	 * @return true always
	 */
	protected boolean applyValues() {
		Vector selectedEntries = fromList.getSelectedEntries();
		if (selectedEntries.size() == 0) {
			return true;
		}
		jobNode.setState(ContainerNode.STATE_UPDATING);
		transferJobs.clear();
		for (int i = 0; i < selectedEntries.size(); i++) {
			FileExport fe = ((LocalExportEntry) selectedEntries.elementAt(i))
					.getExport();
			GetSpooledFiles gsf = new GetSpooledFiles(jobNode, fe);
			fe.setUnSpool(gsf);
			transferJobs.add(gsf);
		}
		return true;
	}

	private void buildComponents() {
		fromList = new LocalExportTable();
		int total = fetchSpooledFiles(jobNode, jobContainer, allExports);
		fromList.setDataVector(allExports);
		fromList.adjustWidths();
		JScrollPane fromScrollPane = new JScrollPane(fromList);
		fromScrollPane
				.setBorder(new TitledBorder(res.getString("SELECT_FILES")));
		getContentPane().add(fromScrollPane, BorderLayout.CENTER);
	}

	/**
	 * Fetch all spooled files belonging to this job. We currently do not
	 * support spooled files that persist after the job is deleted
	 *  
	 */
	private int fetchSpooledFiles(ContainerNode topNode, JobContainer top,
			Vector downloads) {
		Vector subJobs = top.getSubJobs();
		int totalDownloads = 0;
		// delete sub job spool files
		for (int j = 0; j < subJobs.size(); j++) {
			totalDownloads += fetchSpooledFiles(topNode, (JobContainer) subJobs
					.elementAt(j), downloads);
		}
		Vector newNspaceTransfers = top.getNspaceExportTasks();
		if (newNspaceTransfers.size() == 0) {
			return 0;
		}
		for (int i = 0; i < newNspaceTransfers.size(); i++) {
			TaskContainer ac = (TaskContainer) newNspaceTransfers.elementAt(i);
			Vector spooledFiles = ac.getSpooledFiles();
			if (spooledFiles != null) {
				for (int j = 0; j < spooledFiles.size(); j++) {
					FileExport export = (FileExport) spooledFiles.elementAt(j);
					if (!checkForSuccesfullSpooltask(export)) {
						continue;
					}
					// construct real absolute filename
					if (export.getDestinationName().equals("")
							&& !export.hasWildcards()) {
						export.setDestinationName(export.getSourceName());
					}
					File testFile = new File(export.getDestinationName());
					String absolutePath = testFile.getAbsolutePath();
					export.setDestinationName(absolutePath);
					LocalExportEntry entry = new LocalExportEntry(export);
					downloads.add(entry);
					totalDownloads++;
				}
			}
		}
		return totalDownloads;
	}

	/**
	 * Checks if a given FileExport actually was successful
	 *  
	 */
	private boolean checkForSuccesfullSpooltask(FileExport export) {
		ActionContainer task = export.getContainer();
		Vector outcomeEntries = OutcomeManager.getOutcomes(task);
		for (int i = 0; i < outcomeEntries.size(); i++) {
			OutcomeEntry entry = (OutcomeEntry) outcomeEntries.elementAt(i);
			Outcome taskOutcome = entry.getOutcome();
			if (taskOutcome == null) {
				logger.warning("Outcome of exporting container not yet set. "
						+ task);
				return false;
			}
			if (!(taskOutcome instanceof ActionGroup_Outcome)) {
				logger.warning("Dubious outcome for task. " + task);
				return false;
			}
			PutPortfolio_Outcome ppfOutcome = (PutPortfolio_Outcome) findOutcome(
					export.getSpoolTask().getId(),
					(ActionGroup_Outcome) taskOutcome);
			if (ppfOutcome == null) {
				logger.warning("Cannot find PutPortfolio_Outcome. " + task);
				return false;
			}
			// logger.info("Found PutPortfolio_Outcome. " + ppfOutcome);
			if (!ppfOutcome.getStatus().isEquivalent(
					AbstractActionStatus.SUCCESSFUL)) {
				return false;
			}
		}
		return true;
	}

	private Outcome findOutcome(AAIdentifier id,
			ActionGroup_Outcome groupOutcome) {
		Outcome idOutcome = groupOutcome.getOutcome(id);
		if (idOutcome != null) {
			// outcome found, return...
			return idOutcome;
		}
		// find sub ActionGroups and recurse
		OutcomeEnumeration outEnum = groupOutcome.getOutcomes();
		while (outEnum.hasMoreElements()) {
			Outcome next = (Outcome) outEnum.nextElement();
			if (next instanceof ActionGroup_Outcome) {
				idOutcome = findOutcome(id, (ActionGroup_Outcome) next);
				if (idOutcome != null) {
					return idOutcome;
				}
			}
		}
		return null;
	}
}