/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.editor;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileReader;
import java.io.Reader;
import java.util.Vector;
import java.util.logging.Level;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;
import org.unicore.Vsite;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.AbstractJob_Outcome;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.requests.GetFilesFromXspace;
import com.pallas.unicore.requests.SendByteArrayToXspace;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.threadpool.IObserver;
import com.pallas.unicore.utility.StringTools;

/**
 * A text editor that loads and saves files from remote file systems
 * 
 * @author Ralf Ratering
 * @version $Id: RemoteTextEditor.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class RemoteTextEditor extends TextEditor {

	private File defaultDir;
	private SelectorDialog fileChooser;
	private Vsite vsite;

	/**
	 * Build actions
	 */
	protected void buildActions() {
		newAction = new NewAction();
		openAction = new RemoteOpenAction();
		saveAction = new RemoteSaveAction();
		saveAsAction = new RemoteSaveAsAction();
		cutAction = new CutAction();
		copyAction = new CopyAction();
		pasteAction = new PasteAction();
		undoAction = new UndoAction();
		redoAction = new RedoAction();
	}

	/**
	 * Get default directory
	 * 
	 * @return File or RemoteFile pointing to default directory
	 */
	public File getDefaultDir() {
		return defaultDir;
	}

	/**
	 * Gets contents from the text area
	 * 
	 * @return current text
	 */
	public String getText() {
		return textArea.getText();
	}

	/**
	 * Gets the text from the text area without windows/dos style characters
	 * Take care: Using getText() will perform much better for large files.
	 * 
	 * @return The unix style text
	 * @deprecated Performance is too bad for large files
	 */
	public String getUnixStyleText() {
		return StringTools.dos2Unix(textArea.getText());
	}

	/**
	 * Get vsite to connect to
	 * 
	 * @return vsite to connect to
	 */
	public Vsite getVsite() {
		return vsite;
	}

	private void initFileChooser() {
		File dir = file;
		if (dir == null) {
			dir = defaultDir;
		}
		// always build a new file chooser to update vsite
		fileChooser = ResourceManager.getSelectorDialog(dir, vsite);
		fileChooser.removeFileFilters();
		for (int i = 0; i < fileFilters.size(); i++) {
			fileChooser.addFileFilter((FileFilter) fileFilters.elementAt(i));
		}
		if (selectedFileFilter != null) {
			fileChooser.setSelectedFileFilter(selectedFileFilter);
		}
	}

	/**
	 * Set default directory
	 * 
	 * @param defaultDir
	 *            default directory
	 */
	public void setDefaultDir(File defaultDir) {
		this.defaultDir = defaultDir;
	}

	/**
	 * Set vsite to connect to
	 * 
	 * @param vsite
	 *            vsite to connect to
	 */
	public void setVsite(Vsite vsite) {
		this.vsite = vsite;
	}

	/**
	 * Thread to load a remote file that has been spooled into local temp dir
	 */
	class RemoteFileLoaderThread extends Thread {

		private RemoteFile remoteFile;
		private File localFile;

		RemoteFileLoaderThread(RemoteFile remoteFile, File localFile) {
			this.remoteFile = remoteFile;
			this.localFile = localFile;
		}

		public void run() {
			try {
				setStatusMessage("Reading locally spooled file: "
						+ localFile.getAbsolutePath());
				// remove old document
				Document oldDoc = getEditor().getDocument();
				if (oldDoc != null) {
					oldDoc.removeUndoableEditListener(undoHandler);
				}
				getEditor().setDocument(new PlainDocument());
				Document doc = textArea.getDocument();
				// try to start reading
				Reader in = new FileReader(localFile);
				char[] buff = new char[4096];
				int nch;
				while ((nch = in.read(buff, 0, buff.length)) != -1) {
					doc.insertString(doc.getLength(), new String(buff, 0, nch),
							null);
				}
				// we are done... get rid of progressbar
				doc.addUndoableEditListener(undoHandler);
				resetUndoManager();
				// remember remote file
				file = remoteFile;
				setStatusMessage("File: " + file.getAbsolutePath());
			}
			catch (Exception e) {
				logger.log(Level.WARNING, "Could not load file", e);
				setStatusMessage("Could not load remote file: "
						+ file.getAbsolutePath());
			}
		}
	}

	class RemoteOpenAction extends OpenAction implements IObserver {

		public void actionPerformed(ActionEvent evt) {
			String oldMessage = getStatusMessage();
			initFileChooser();
			fileChooser.setTitle("UNICORE: Open file");
			int result = fileChooser.showDialog();
			if (result != JFileChooser.APPROVE_OPTION) {
				setStatusMessage(oldMessage);
				return;
			}
			// load file
			File newFile = fileChooser.getSelectedFile();
			if (newFile instanceof RemoteFile) {
				setStatusMessage("Transferring remote file: "
						+ newFile.getAbsolutePath());
				Vector remoteFiles = new Vector();
				remoteFiles.add(newFile);
				try {
					GetFilesFromXspace request = new GetFilesFromXspace(
							remoteFiles);
					request.addObserver(this);
					request.start();
				}
				catch (Exception e) {
					logger.log(Level.WARNING, "Could not get remote files: "
							+ remoteFiles, e);
					setStatusMessage(oldMessage);
					return;
				}
			}
			else {
				if (newFile.exists()) {
					Thread loader = new FileLoaderThread(newFile);
					loader.start();
				}
			}
		}

		public void observableUpdate(Object theObserved, Object changeCode) {
			GetFilesFromXspace request = (GetFilesFromXspace) theObserved;
			// assume only one file has been fetched
			Vector remoteFiles = request.getRemoteFiles();
			RemoteFile remoteFile = (RemoteFile) remoteFiles.firstElement();
			Vector files = request.getLocalFiles();
			File localFile = (File) files.firstElement();
			AbstractJob_Outcome outcome = (AbstractJob_Outcome) changeCode;
			logger.info("GetFileFromXspace result: " + outcome.getStatus());
			if (outcome.getStatus().isEquivalent(
					AbstractActionStatus.SUCCESSFUL)) {
				setStatusMessage("Remote file successfully transferred.");
				Thread loader = new RemoteFileLoaderThread(remoteFile,
						localFile);
				loader.start();
				return;
			}
		}
	}

	class RemoteSaveAction extends SaveAction implements IObserver {

		public RemoteSaveAction() {
			this("Save");
		}

		public RemoteSaveAction(String name) {
			super(name);
		}

		public void actionPerformed(ActionEvent e) {
			if (file == null || file.isDirectory()) {
				new RemoteSaveAsAction().actionPerformed(e);
				return;
			}
			save();
		}

		protected void save() {
			if (file instanceof RemoteFile) {
				RemoteFile remoteFile = (RemoteFile) file;
				// remove dos.style line breaks
				String text = StringTools.dos2Unix(getText());
				SendByteArrayToXspace request = new SendByteArrayToXspace(text
						.getBytes(), remoteFile.getVsite(), remoteFile
						.getStorage(), remoteFile.getSubspacePath());
				request.addObserver(this);
				request.start();
			}
			else {
				Thread saveThread = new FileWriterThread(file, textArea
						.getDocument());
				saveThread.start();
			}
		}

		public void observableUpdate(Object theObserved, Object changeCode) {
			AbstractJob_Outcome outcome = (AbstractJob_Outcome) changeCode;
			logger.info("GetFileFromXspace result: " + outcome.getStatus());
			if (outcome.getStatus().isEquivalent(
					AbstractActionStatus.SUCCESSFUL)) {
				setStatusMessage("Remote file successfully transferred.");
			}
			else {
				setStatusMessage("Saving remote file failed.");
			}
		}
	}

	class RemoteSaveAsAction extends RemoteSaveAction {

		RemoteSaveAsAction() {
			super("Save As...");
		}

		public void actionPerformed(ActionEvent e) {
			initFileChooser();
			fileChooser.setTitle("UNICORE: Save file as");
			int result = fileChooser.showDialog();
			if (result != JFileChooser.APPROVE_OPTION) {
				return;
			}
			file = fileChooser.getSelectedFile();
			save();
		}
	}
}