/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.menus;

import java.awt.Color;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

/**
 * A menu that manages recently select files.
 * 
 * @author Ralf Ratering
 * @version $Id: RecentFileMenu.java,v 1.1 2004/05/25 14:58:53 rmenday Exp $
 */
public class RecentFileMenu extends JMenu {

	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.menus");
	// for the moment we have constantly a maximum of 4 recent files
	private final static int RECENT_FILES = 4;
	// filename where recent files will be stored
	private String filename;
	// listener for item selections
	private ActionListener itemListener;
	// collect files in a vector
	private Vector recentFiles = new Vector(RECENT_FILES);

	/**
	 * Constructor builds GUI and loads recent filenames from file
	 * 
	 * @param title
	 *            menu title
	 * @param filename
	 *            points to file where recent filenames are stored
	 * @param itemListener
	 *            listens to menu item selections
	 */
	public RecentFileMenu(String title, String filename,
			ActionListener itemListener) {
		super(title);
		this.filename = filename;
		this.itemListener = itemListener;
		loadEntries();
		buildRecentFileMenu();
	}

	/**
	 * Add a recent file to the menu
	 * 
	 * @param fileName
	 *            filename to be added
	 */
	public void addRecentFile(String fileName) {
		int offset;
		// Check wether this file is already in the list
		if ((offset = recentFiles.indexOf(fileName)) == -1) {
			// Not present - add at beginning
			recentFiles.insertElementAt(fileName, 0);
			if (recentFiles.size() > RECENT_FILES) {
				// remove last item
				recentFiles.removeElementAt(RECENT_FILES);
			}
		}
		else {
			// Already present: move to the front
			recentFiles.removeElementAt(offset);
			recentFiles.insertElementAt(fileName, 0);
		}
		// save recent files
		saveEntries();
		buildRecentFileMenu();
	}

	/**
	 * Build the file menu entries
	 */
	private void buildRecentFileMenu() {
		if (recentFiles == null) {
			return;
		}
		int size = recentFiles.size();
		removeAll();
		for (int i = 0; i < size; i++) {
			String fileName = (String) recentFiles.elementAt(i);
			JMenuItem mi = new JMenuItem(fileName);
			mi.setForeground(getForeground());
			mi.setBackground(getBackground());
			add(mi);
			mi.addActionListener(itemListener);
		}
	}

	/**
	 * Load current entries from file
	 */
	private void loadEntries() {
		File file = new File(filename);
		try {
			FileInputStream in = new FileInputStream(file);
			BufferedReader reader = new BufferedReader(
					new InputStreamReader(in));
			while (reader.ready()) {
				String recentFile = reader.readLine();
				if (recentFile != null && !recentFile.equals("")) {
					recentFiles.add(recentFile);
				}
			}
			reader.close();
		}
		catch (Exception e) {
			logger.log(Level.INFO, "Recent files not available:"
					+ e.getMessage());
		}
	}

	/**
	 * Write current entries to file
	 */
	private void saveEntries() {
		File file = new File(filename);
		try {
			FileOutputStream out = new FileOutputStream(file);
			BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(
					out));
			for (int i = 0; i < recentFiles.size(); i++) {
				writer.write((String) recentFiles.get(i));
				writer.newLine();
			}
			writer.close();
		}
		catch (Exception e) {
			logger.info("Recent files do not exist: " + filename);
		}
	}

	/**
	 * Sets the Background attribute of the RecentFileMenu object
	 * 
	 * @param c
	 *            The new Background value
	 */
	public void setBackground(Color c) {
		super.setBackground(c);
		buildRecentFileMenu();
	}

	/**
	 * Sets the Foreground attribute of the RecentFileMenu object
	 * 
	 * @param c
	 *            The new Foreground value
	 */
	public void setForeground(Color c) {
		super.setForeground(c);
		buildRecentFileMenu();
	}
}