/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.Color;
import java.awt.Component;
import java.awt.datatransfer.DataFlavor;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.Date;
import java.util.Vector;
import javax.swing.DefaultCellEditor;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;
import org.unicore.resources.USpace;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.client.tables.AdjustTable;
import com.pallas.unicore.client.tables.ComboBoxCellEditor;
import com.pallas.unicore.client.tables.ComboBoxCellRenderer;
import com.pallas.unicore.client.tables.FilePermission;
import com.pallas.unicore.client.tables.FilePermissionCellEditor;
import com.pallas.unicore.client.tables.FilePermissionPanel;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.FileOperation;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.extensions.FileSystemOperation;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 * Panel to define a file operations in a certain file space (Uspace or Xspace)
 * 
 * @author Ralf Ratering
 * @author Klaus-Dieter Oertel
 * @version $Id: FileOperationPanel.java,v 1.1 2004/05/25 14:58:47 rmenday Exp $
 */
public class FileOperationPanel extends FileStoragePanel {

	final static int COL_OPERATION = 0;
	final static int COL_STORAGE = 1;
	final static int COL_SOURCE = 2;
	final static int COL_TARGET = 3;
	private JComboBox operationComboBox;
	private Vector operations;
	private FilePermissionCellEditor permissionsEditor;
	private FilePermissionPanel permissionsRenderer;
	private JComboBox storagesEditorComboBox;
	private ComboBoxCellRenderer storagesRendererComboBox;
	private DefaultCellEditor stringEditor;
	private FileOperationTableCellRenderer stringRenderer;

	/**
	 * Constructor
	 * 
	 * @param container
	 *            to be modified
	 */
	public FileOperationPanel(TaskContainer container) {
		this(container, null);
	}

	public FileOperationPanel(TaskContainer container, String initialDirectory) {
		super(container, initialDirectory);
		operations = new Vector();
		String[] ops = FileSystemOperation.getOperations();
		for (int i = 0; i < ops.length; i++) {
			operations.add(ops[i]);
		}
		// set combo box of file system operations
		ComboBoxListener comboBoxListener = new ComboBoxListener();
		operationComboBox = new JComboBox(operations);
		operationComboBox.addItemListener(comboBoxListener);
		table.getColumnModel().getColumn(COL_OPERATION).setCellEditor(
				new ComboBoxCellEditor(operationComboBox));
		table.getColumnModel().getColumn(COL_OPERATION).setCellRenderer(
				new ComboBoxCellRenderer(operations));
		stringRenderer = new FileOperationTableCellRenderer();
		stringEditor = new DefaultCellEditor(new JTextField());
		permissionsRenderer = new FilePermissionPanel();
		permissionsEditor = new FilePermissionCellEditor(
				new FilePermissionPanel());
		/*
		 * The documentation of javax.swing.table.TableColumn promises that a
		 * renderer/editor selection based on the classes of cell values is
		 * possible. Actually this is not true because
		 * JTable.getCellRenderer(row,col) evaluates the class only from the
		 * column by calling getColumnClass(col) and NOT by
		 * getValueAt(row,col).getClass(). Therefore
		 * JTable.getCellRenderer(row,col) has to be overwritten in
		 * FileOperationTable. In the hope of future releases of
		 * JTable.getCellRenderer(row,col) we keep the approach of mapping the
		 * class values to certain renderers in the following (which is
		 * evaluated then in FileOperationTable.getCellRenderer(row,col) where
		 * we could set the Renderer directly, however.)
		 */
		/*
		 * table.setDefaultRenderer(String.class, stringRenderer);
		 * table.setDefaultRenderer(FilePermission.class, permissionsRenderer);
		 * table.setDefaultEditor (FilePermission.class, permissionsEditor);
		 */
		buildComponents();
	}

	/**
	 * Add an empty row at the table end
	 */
	protected void addEmptyRow() {
		Vector lastRowData = null;
		int selectedRow = table.getSelectedRow();
		if (selectedRow >= 0) {
			lastRowData = (Vector) tableModel.getDataVector().elementAt(
					selectedRow);
		}
		Vector rowData = new Vector();
		// no row has been selected, usually empty table
		if (lastRowData == null) {
			// use USpace as default for new rows
			USpace uspace;
			if (container.getVsite() != null) {
				uspace = ResourceManager.getResourceSet(container.getVsite())
						.getUspace();
			}
			else {
				uspace = new USpace();
			}
			rowData.add(FileSystemOperation.COPY);
			rowData.add(uspace);
			rowData.add("");
			rowData.add("");
		}
		// add a new row with default entries based on the previously selected
		// row
		else {
			rowData.add(lastRowData.elementAt(COL_OPERATION));
			rowData.add(lastRowData.elementAt(COL_STORAGE));
			rowData.add("");
			if (FileSystemOperation.CHMOD.equals(rowData.elementAt(0))) {
				FilePermission permission = (FilePermission) lastRowData
						.elementAt(COL_TARGET);
				rowData.add(permission.clone());
			}
			else {
				rowData.add("");
			}
		}
		tableModel.addRow(rowData);
		selectedRow = table.getRowCount() - 1;
		table.setRowSelectionInterval(selectedRow, selectedRow);
	}

	/**
	 * Apply gui values to container
	 */
	public void applyValues() {
		// stop editing
		if (table.getCellEditor() != null) {
			table.getCellEditor().stopCellEditing();
		}
		// loop over rows
		int nrOfRows = table.getRowCount();
		FileSystemOperation[] fileOps = new FileSystemOperation[nrOfRows];
		for (int i = 0; i < nrOfRows; i++) {
			fileOps[i] = (FileSystemOperation) getFileOperationAt(i);
		}
		updateModifiedTime(fileOps);
		container.setFileSystemOperations(fileOps);
		container.checkContents();
	}

	private void buildComponents() {
		// minimize combobox columns
		int operationChoice = 135;
		table.getColumnModel().getColumn(COL_OPERATION).setPreferredWidth(
				operationChoice);
		// table.getColumnModel().getColumn(COL_OPERATION).setMinWidth(operationChoice);
		// table.getColumnModel().getColumn(COL_OPERATION).setMaxWidth(operationChoice);
		int storageChoice = 90;
		table.getColumnModel().getColumn(COL_STORAGE).setPreferredWidth(
				storageChoice);
		// table.getColumnModel().getColumn(COL_STORAGE).setMinWidth(storageChoice);
		// table.getColumnModel().getColumn(COL_STORAGE).setMaxWidth(storageChoice);
		// build the combobox editors and renderers in tables
		storages = new Vector();
		storages.add(FileStorage.NSPACE_STRING);
		storagesEditorComboBox = new JComboBox(storages);
		storagesRendererComboBox = new ComboBoxCellRenderer(storages);
		table.getColumnModel().getColumn(COL_STORAGE).setCellEditor(
				new DefaultCellEditor(storagesEditorComboBox));
		table.getColumnModel().getColumn(COL_STORAGE).setCellRenderer(
				storagesRendererComboBox);
	}

	protected AdjustTable createTable() {
		return new FileOperationTable(tableModel);
	}

	protected FileTableModel createTableModel() {
		columnNames = getColumnNames();
		return new FileOperationTableModel(columnNames, 0);
	}

	/**
	 * Gets the columnNames
	 */
	protected String[] getColumnNames() {
		String[] columnNames = {"Operation", "File System", "Filename",
				"Second Parameter"};
		return columnNames;
	}

	/**
	 * Gets the dataFlavor
	 */
	protected DataFlavor getDataFlavor() {
		return FileSystemOperation.getFlavor();
	}

	/**
	 * Get file operation from a certain row
	 * 
	 * @param row
	 *            index of row
	 * @return file operation at selected row
	 */
	protected FileOperation getFileOperationAt(int row) {
		String operation = (String) tableModel.getValueAt(row, COL_OPERATION);
		Object storageItem = tableModel.getValueAt(row, COL_STORAGE);
		if (storageItem.equals(res.getString("NONE"))) {
			storageItem = null;
		}
		String sourceName = (String) tableModel.getValueAt(row, COL_SOURCE);
		// replace FilePermission by String representation
		String destinationName = tableModel.getValueAt(row, COL_TARGET)
				.toString();
		return new FileSystemOperation(sourceName, operation, destinationName,
				storageItem);
	}

	/**
	 * Reset current GUI entries to last container contents.
	 */
	public void resetValues() {
		// check if container has id and name
		if (container.getIdentifier() == null) {
			return;
		}
		// stop editing
		if (table.getCellEditor() != null) {
			table.getCellEditor().stopCellEditing();
		}
		clearTable();
		FileSystemOperation[] fileOps = container.getFileSystemOperations();
		for (int i = 0; i < fileOps.length; i++) {
			addFileOperation(fileOps[i]);
		}
	}

	/**
	 * Fill an existing table row with a certain FileExport object. If row index
	 * is too high add a new row and use that one.
	 * 
	 * @param index
	 *            nr of table row
	 * @param fileOp
	 *            The new fileOperation
	 */
	protected void setFileOperationAt(FileOperation fileOp, int index) {
		FileSystemOperation fileSysOp = (FileSystemOperation) fileOp;
		String operation = fileSysOp.getOperation();
		Object target = fileSysOp.getDestinationName();
		// the table is edited
		if (index < table.getRowCount()) {
			String oldSource = (String) tableModel
					.getValueAt(index, COL_SOURCE);
			// if table entry already contains a source, keep the operation and
			// target
			if (!oldSource.equals("")) {
				operation = (String) tableModel
						.getValueAt(index, COL_OPERATION);
				target = tableModel.getValueAt(index, COL_TARGET);
			}
		}
		if (index > table.getRowCount() - 1) {
			tableModel.addRow(new Vector());
		}
		tableModel.setValueAt(operation, index, COL_OPERATION);
		tableModel.setValueAt(fileSysOp.getStorageItem(), index, COL_STORAGE);
		tableModel.setValueAt(fileSysOp.getSourceName(), index, COL_SOURCE);
		if (operation.equals(FileSystemOperation.CHMOD)
				&& !(target instanceof FilePermission)) {
			FilePermission permission = new FilePermission();
			permission.setPermission((String) target);
			tableModel.setValueAt(permission, index, COL_TARGET);
		}
		else {
			tableModel.setValueAt(target, index, COL_TARGET);
		}
	}

	protected void showBrowser() {
		if (container.getVsite() == null) {
			UserMessages.info("Please select a virtual site first.");
			return;
		}
		if (!ResourceManager.isVsiteAvailable(container.getVsite())) {
			UserMessages.info("Virtual site " + container.getVsite()
					+ " is not available.");
			return;
		}
		SelectorDialog chooser = ResourceManager.getSelectorDialog(null,
				container.getVsite());
		chooser.setTitle("UNICORE: Choose source for file operation");
		chooser.setLocalEnabled(false);
		int result = chooser.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			// for now only single selection is supported
			File[] files = {chooser.getSelectedFile()};
			for (int i = 0; i < files.length; i++) {
				FileSystemOperation fileOp = null;
				if (files[i] instanceof RemoteFile) {
					RemoteFile file = (RemoteFile) files[i];
					fileOp = new FileSystemOperation(file.getSubspacePath(),
							FileSystemOperation.COPY, file.getSubspacePath(),
							file.getStorage());
				}
				// if we have selected multiple files we want to add them
				// if we selected one table row and one file we wanted to edit
				if (table.getSelectedRowCount() == 1 && files.length == 1) {
					setFileOperationAt(fileOp, table.getSelectedRow());
				}
				else {
					addFileOperation(fileOp);
				}
			}
		}
	}

	private void updateModifiedTime(FileSystemOperation[] newOps) {
		FileSystemOperation[] oldOps = container.getFileSystemOperations();
		// first a simple check...
		if (oldOps.length != newOps.length) {
			container.setModifiedTime(new Date());
			return;
		}
		// ... now try to find matching ops
		// loop over old ops
		for (int i = 0; i < oldOps.length; i++) {
			boolean found = false;
			// loop over new ops
			for (int j = 0; j < newOps.length; j++) {
				// try to find matching new import
				if (newOps[j].equals(oldOps[i])) {
					// match was found
					found = true;
				}
			}
			if (!found) {
				// if we get here one op was new...
				container.setModifiedTime(new Date());
				return;
			}
		}
	}

	/**
	 * Get current storage resources from ResourceManager
	 */
	public void updateStorageResources() {
		//storages.update(container.getVsite());
		storages = new Vector();
		// local?
		//storages.add(FileStorage.NSPACE_STRING);
		NamedResourceSet resourceSet = ResourceManager.getResourceSet(container
				.getVsite());
		resourceSet.addStorageResources(storages);
		if (storages.size() == 0) {
			//			storages.add(res.getString("NONE"));
			//			storages.add(new USpace());
			return;
		}
		// exclude Spool and AlternativeUspace
		Vector cloned = (Vector) storages.clone();
		for (int i = 0; i < cloned.size(); i++) {
			if (cloned.elementAt(i) instanceof org.unicore.resources.Spool
					|| cloned.elementAt(i) instanceof org.unicore.resources.AlternativeUspace) {
				storages.remove(cloned.elementAt(i));
			}
		}
		cloned.clear();
	}

	/**
	 * Update storage resources
	 */
	public void updateValues() {
		updateStorageResources();
		replaceStorageResources(COL_STORAGE);
		// update combo boxes
		storagesEditorComboBox.setModel(new DefaultComboBoxModel(storages));
		storagesRendererComboBox.setModel(new DefaultComboBoxModel(storages));
	}

	private class ComboBoxListener implements ItemListener {

		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				//int rows[] = table.getSelectedRows();
				//for (int i=0; i<rows.length; i++) {
				// System.err.println("DESELECTED "+e.getItem()+" rows["+i+" =
				// "+rows[i]);}
				return;
			}
			if (e.getSource() == operationComboBox) {
				//int rows[] = table.getSelectedRows();
				//for (int i=0; i<rows.length; i++) { System.err.println("
				// SELECTED "+e.getItem()+" rows["+i+" = "+rows[i]);}
				int row = table.getSelectedRow();
				if (e.getItem().equals(FileSystemOperation.CHMOD)) {
					tableModel
							.setValueAt(new FilePermission(), row, COL_TARGET);
				}
				else if (e.getItem().equals(FileSystemOperation.DELETE)
						|| e.getItem().equals(FileSystemOperation.MAKEDIR)) {
					tableModel.setValueAt(new String(""), row, COL_TARGET);
				}
				else {
					// What was the reason for the following line?
					// Object value = tableModel.getValueAt(row, COL_TARGET);
					Object value = tableModel.getValueAt(row, COL_OPERATION);
					if (FileSystemOperation.CHMOD.equals(value)
							|| FileSystemOperation.DELETE.equals(value)
							|| FileSystemOperation.MAKEDIR.equals(value)) {
						tableModel.setValueAt(new String(""), row, COL_TARGET);
					}
				}
				tableModel.fireTableRowsUpdated(row, row);
			}
		}
	}

	/**
	 * Table with cell dependent renderers/editors
	 */
	class FileOperationTable extends AdjustTable {

		public FileOperationTable(FileTableModel tableModel) {
			super((TableModel) tableModel);
		}

		public TableCellEditor getCellEditor(int row, int col) {
			/*
			 * if (this.convertColumnIndexToModel(col) == COL_TARGET) { / the
			 * editor choice is based on the class of the cell's value return
			 * super.getDefaultEditor(table.getValueAt(row,col).getClass()); } *
			 * alternative implementation without using the value's class
			 */
			if (convertColumnIndexToModel(col) == COL_TARGET) {
				if (tableModel.getValueAt(row, COL_OPERATION).equals(
						FileSystemOperation.CHMOD)) {
					return permissionsEditor;
				}
				else {
					return stringEditor;
				}
			}
			else {
				return super.getCellEditor(row, col);
			}
		}

		public TableCellRenderer getCellRenderer(int row, int col) {
			if (convertColumnIndexToModel(col) == COL_TARGET) {
				if (tableModel.getValueAt(row, COL_OPERATION).equals(
						FileSystemOperation.CHMOD)) {
					return permissionsRenderer;
				}
				else {
					return stringRenderer;
				}
			}
			else {
				return super.getCellRenderer(row, col);
			}
		}
	}

	private class FileOperationTableCellRenderer
			extends
				DefaultTableCellRenderer {

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			Component component = super.getTableCellRendererComponent(table,
					value, isSelected, hasFocus, row, column);
			if (column == COL_TARGET) {
				String selectedItem = (String) tableModel.getValueAt(row,
						COL_OPERATION);
				if (selectedItem.equals(FileSystemOperation.DELETE)
						|| selectedItem.equals(FileSystemOperation.MAKEDIR)) {
					component.setBackground(Color.LIGHT_GRAY);
				}
				else {
					if (isSelected) {
						component.setBackground(table.getSelectionBackground());
					}
					else {
						component.setBackground(Color.WHITE);
					}
				}
			}
			return component;
		}
	}

	class FileOperationTableModel extends FileTableModel {

		public FileOperationTableModel(String[] columnNames, int numRows) {
			super(columnNames, numRows);
		}

		public boolean isCellEditable(int row, int col) {
			if (col != COL_TARGET) {
				return true;
			}
			String selectedItem = (String) tableModel.getValueAt(row,
					COL_OPERATION);
			if (selectedItem.equals(FileSystemOperation.DELETE)
					|| selectedItem.equals(FileSystemOperation.MAKEDIR)) {
				return false;
			}
			else {
				return true;
			}
		}
	}
}