/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import com.pallas.unicore.client.controls.SimpleToolBar;
import com.pallas.unicore.client.tables.AdjustTable;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.FileOperation;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.TransferableVector;

/**
 * Base class for import and export panels
 * 
 * @author Ralf Ratering
 * @version $Id: FilePanel.java,v 1.30 2001/11/20 12:40:54 rrate Exp
 */
public abstract class FilePanel extends JPanel implements ClipboardOwner {

	class AddAction extends AbstractAction {

		public AddAction() {
			super("Add", ResourceManager.getIcon(ResourceManager.ADD));
		}

		public void actionPerformed(ActionEvent e) {
			if (table.getCellEditor() != null) {
				table.getCellEditor().stopCellEditing();
			}
			addEmptyRow();
		}
	}

	class CopyAction extends AbstractAction {

		/**
		 * Constructor for the CopyAction object
		 */
		public CopyAction() {
			super("Copy", ResourceManager.getIcon(ResourceManager.COPY));
		}

		public void actionPerformed(ActionEvent e) {
			copy();
		}
	}

	class CutAction extends AbstractAction {

		public CutAction() {
			super("Cut", ResourceManager.getIcon(ResourceManager.CUT));
		}

		public void actionPerformed(ActionEvent e) {
			if (table.getCellEditor() != null) {
				table.getCellEditor().stopCellEditing();
			}
			cut();
		}
	}

	class FileTableModel extends DefaultTableModel {

		public FileTableModel(Object[] names, int rows) {
			super(names, rows);
		}

		/**
		 * JTable uses this method to determine the default renderer editor for
		 * each cell. If we didn't implement this method, then the last column
		 * would contain text ("true"/"false"), rather than a check box.
		 * 
		 * @param c
		 *            Description of Parameter
		 * @return The ColumnClass value
		 */
		public Class getColumnClass(int c) {
			Object columnObject = getValueAt(0, c);
			if (columnObject == null) {
				String dummy = new String();
				return dummy.getClass();
			}
			else {
				return columnObject.getClass();
			}
		}
	}

	private class MyKeyListener extends KeyAdapter {

		public void keyPressed(KeyEvent e) {
			int keyCode = e.getKeyCode();
			int modifiers = e.getModifiers();
			// increase id
			eventId++;
			if (keyCode == KeyEvent.VK_N && modifiers == KeyEvent.CTRL_MASK) {
				addAction.actionPerformed(new ActionEvent(this, eventId, ""));
			}
			else if (keyCode == KeyEvent.VK_X
					&& modifiers == KeyEvent.CTRL_MASK) {
				cutAction.actionPerformed(new ActionEvent(this, eventId, ""));
			}
			else if (keyCode == KeyEvent.VK_C
					&& modifiers == KeyEvent.CTRL_MASK) {
				copyAction.actionPerformed(new ActionEvent(this, eventId, ""));
			}
			else if (keyCode == KeyEvent.VK_V
					&& modifiers == KeyEvent.CTRL_MASK) {
				pasteAction.actionPerformed(new ActionEvent(this, eventId, ""));
			}
			else if (keyCode == KeyEvent.VK_DELETE && modifiers == 0) {
				removeAction
						.actionPerformed(new ActionEvent(this, eventId, ""));
			}
		}
	}

	class PasteAction extends AbstractAction {

		public PasteAction() {
			super("Paste", ResourceManager.getIcon(ResourceManager.PASTE));
		}

		public void actionPerformed(ActionEvent e) {
			paste();
		}
	}

	class PopupMouseListener extends MouseAdapter {

		public void mouseClicked(MouseEvent e) {
			testShowPopup(e);
		}

		public void mousePressed(MouseEvent e) {
			testShowPopup(e);
		}

		public void mouseReleased(MouseEvent e) {
			testShowPopup(e);
		}
	}

	class RemoveAction extends AbstractAction {

		public RemoveAction() {
			super("Remove", ResourceManager.getIcon(ResourceManager.REMOVE));
		}

		public void actionPerformed(ActionEvent e) {
			if (table.getCellEditor() != null) {
				table.getCellEditor().stopCellEditing();
			}
			remove();
		}
	}
	protected static Clipboard clipboard = new Clipboard("filePanelClipboard");
	// id for key listener actions
	private static int eventId = 0;
	protected static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.panels");
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.panels.ResourceStrings");

	/**
	 * The main program for the FilePanel class
	 * 
	 * @param args
	 *            The command line arguments
	 */
	public static void main(String[] args) {
		JPanel panel = new JPanel(new BorderLayout());
		panel.add(new JTable(0, 5), BorderLayout.CENTER);
		panel.addMouseListener(new MouseAdapter() {

			public void mousePressed(MouseEvent e) {
				System.err.println("MOUSEYYYY!");
			}
		});
		panel.setPreferredSize(new Dimension(400, 400));
		JFrame app = new JFrame();
		app.getContentPane().add(panel);
		app.pack();
		app.show();
	}
	private AddAction addAction = new AddAction();
	protected String[] columnNames;
	/**
	 * container to be modified
	 */
	protected TaskContainer container;
	private CopyAction copyAction = new CopyAction();
	private CutAction cutAction = new CutAction();
	private MyKeyListener keyListener = new MyKeyListener();
	private PasteAction pasteAction = new PasteAction();
	protected JPopupMenu popup;
	private PopupMouseListener popupMouseListener = new PopupMouseListener();
	private RemoveAction removeAction = new RemoveAction();
	/**
	 * table containing file operations
	 */
	protected AdjustTable table;
	protected FileTableModel tableModel;
	protected SimpleToolBar toolbar;

	public FilePanel(TaskContainer container) {
		this.container = container;
		buildTable();
		buildPopupMenu();
		buildToolBar();
	}

	/**
	 * Add an empty row to the table
	 */
	protected abstract void addEmptyRow();

	/**
	 * Add a new table row and fill it with a file import
	 * 
	 * @param fileOperation
	 *            The feature to be added to the FileOperation attribute
	 */
	public void addFileOperation(FileOperation fileOperation) {
		setFileOperationAt(fileOperation, tableModel.getRowCount());
	}

	/**
	 * Apply gui values to container
	 */
	public abstract void applyValues();

	protected void buildPopupMenu() {
		popup = new JPopupMenu();
		popup.setInvoker(this);
		popup.add(addAction);
		popup.add(cutAction);
		popup.add(copyAction);
		popup.add(pasteAction);
		popup.add(removeAction);
	}

	protected void buildTable() {
		tableModel = createTableModel();
		table = createTable();
		table.setRowHeight(20);
		table.addMouseListener(popupMouseListener);
		table.addKeyListener(keyListener);
		JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.addMouseListener(popupMouseListener);
		setLayout(new BorderLayout());
		add(scrollPane, BorderLayout.CENTER);
	}

	protected void buildToolBar() {
		toolbar = new SimpleToolBar();
		toolbar.add(addAction);
		toolbar.add(removeAction);
		add(toolbar, BorderLayout.NORTH);
	}

	/**
	 * Remove all table entries
	 */
	protected void clearTable() {
		int nrOfRows = table.getRowCount();
		while (nrOfRows > 0) {
			tableModel.removeRow(nrOfRows - 1);
			nrOfRows = table.getRowCount();
		}
	}

	protected void copy() {
		int[] rows = table.getSelectedRows();
		if (rows.length <= 0) {
			return;
		}
		TransferableVector vector = new TransferableVector(getDataFlavor());
		for (int i = 0; i < rows.length; i++) {
			FileOperation fileOp = getFileOperationAt(rows[i]);
			try {
				vector.add((Transferable) fileOp.clone());
			}
			catch (UnsupportedFlavorException e) {
				logger.log(Level.WARNING,
						"Wrong type of file operation copied.", e);
			}
		}
		clipboard.setContents(vector, this);
	}

	/**
	 * Allow overwrite in sub-classes
	 */
	protected AdjustTable createTable() {
		return new AdjustTable((TableModel) tableModel);
	}

	/**
	 * Allow overwrite in sub-classes
	 */
	protected FileTableModel createTableModel() {
		columnNames = getColumnNames();
		return new FileTableModel(columnNames, 0);
	}

	protected void cut() {
		copy();
		remove();
	}

	protected abstract String[] getColumnNames();

	protected abstract DataFlavor getDataFlavor();

	protected abstract FileOperation getFileOperationAt(int row);

	public void lostOwnership(Clipboard clipboard, Transferable transferable) {
		logger.warning("FilePanel Clipboard " + clipboard.getName()
				+ " lost ownership!");
	}

	protected void paste() {
		Transferable clipboardContent = clipboard.getContents(this);
		try {
			// get node from clipboard
			TransferableVector vector = (TransferableVector) clipboardContent
					.getTransferData(getDataFlavor());
			for (int i = 0; i < vector.size(); i++) {
				FileOperation fileOp = (FileOperation) vector.elementAt(i);
				addFileOperation((FileOperation) fileOp.clone());
			}
		}
		catch (Exception e) {
			logger.log(Level.WARNING, "Could not paste file operation.", e);
		}
	}

	protected void remove() {
		while (table.getSelectedRow() > -1) {
			tableModel.removeRow(table.getSelectedRow());
		}
		int selectedRow = table.getRowCount() - 1;
		if (selectedRow >= 0) {
			table.setRowSelectionInterval(selectedRow, selectedRow);
		}
	}

	public void removeFileOperation(FileOperation fileOperation) {
		for (int i = 0; i < tableModel.getRowCount(); i++) {
			if (getFileOperationAt(i).equals(fileOperation)) {
				tableModel.removeRow(i);
				// decrease i, because of deleted row
				i--;
			}
		}
	}

	/**
	 * Apply container entries to gui elements
	 */
	public abstract void resetValues();

	protected abstract void setFileOperationAt(FileOperation fileOp, int row);

	private void testShowPopup(MouseEvent evt) {
		if (evt.isPopupTrigger()) {
			Transferable vector = clipboard.getContents(this);
			pasteAction.setEnabled(vector != null
					&& vector.isDataFlavorSupported(getDataFlavor()));
			popup.show(this, evt.getX(), evt.getY());
		}
	}

	/**
	 * Update storage resource entries and build dependent GUI elements like
	 * combobox editors and renderers
	 */
	public abstract void updateValues();
}