/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.plugins.proxycertificate;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.security.KeyPair;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.logging.Logger;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.DERInputStream;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DEROutputStream;
import org.bouncycastle.asn1.x509.X509Name;
import org.bouncycastle.jce.X509V3CertificateGenerator;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.security.CertificateUtility;
import com.pallas.unicore.utility.Base64;
import com.pallas.unicore.utility.UserMessages;

/**
 * @author Thomas Kentemich
 * @version $Id: ProxyCertificate.java,v 1.1 2004/05/25 14:58:52 rmenday Exp $
 */
public class ProxyCertificate implements IProxyCertificate {

	static X509V3CertificateGenerator v3CertGen = new X509V3CertificateGenerator();
	private static RSAPrivateKey privateKey;
	private static RSAPublicKey publicKey;
	private static boolean initialized = false;
	private boolean limited = false;
	private long lifeTime = 0;
	private X509Certificate proxyCertificate;
	private X509Certificate userCertificate;
	private long validFrom;
	private long validTo;
	private int keyLength;
	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.plugins.proxycertificate");

	/**
	 * Constructor for the ProxyCertificate object
	 */
	private ProxyCertificate() {
	}

	private ProxyCertificate(X509Certificate userCert, boolean limited,
			long validFrom, long validTo, int keyLength) {
		this.userCertificate = userCert;
		this.limited = limited;
		this.validFrom = validFrom;
		this.validTo = validTo;
		this.keyLength = keyLength;
		try {
			if (!initialized || !this.userCertificate.equals(userCert)) {
				KeyPair keys = CertificateUtility.generateNewKeys("RSA",
						keyLength);
				privateKey = (RSAPrivateKey) keys.getPrivate();
				publicKey = (RSAPublicKey) keys.getPublic();
				initialized = true;
			}
			this.proxyCertificate = newproxyCertificate();
		}
		catch (Exception e) {
			UserMessages.error("Problems generating Proxy certificate: "
					+ e.getLocalizedMessage());
			this.proxyCertificate = null;
		}
	}

	/**
	 * Returns the proxy certifcate signed with the users private key
	 * 
	 * @return the proxy certifcate
	 * @exception Exception
	 *                Description of the Exception
	 */
	public X509Certificate getProxyCertificate() throws Exception {
		return proxyCertificate;
	}

	/**
	 * Returns the private unprotected key of the proxy certificate
	 * 
	 * @return The private key
	 */
	public PrivateKey getPrivateKey() {
		// key is rubbish anyway, we can give it away
		return (privateKey == null) ? null : privateKey;
	}

	/**
	 * Returns start time of the proxy certificate
	 * 
	 * @return Start time
	 */
	public long getValidFrom() {
		return this.validFrom;
	}

	/**
	 * Returns end time of the proxy certificate
	 * 
	 * @return End Time
	 */
	public long getValidTo() {
		return this.validTo;
	}

	/**
	 * Returns the user certificate
	 * 
	 * @return The user certificate
	 */
	public X509Certificate getUserCertificate() {
		return this.userCertificate;
	}

	/**
	 * Is this a limited proxy?
	 * 
	 * @return Limited or not
	 */
	public boolean isLimited() {
		return this.limited;
	}

	/**
	 * Return the length of the public/private key pair
	 * 
	 * @return Length
	 */
	public int getKeyLength() {
		return this.keyLength;
	}

	/**
	 * Generate a new proxy certificate for each call. Subject and issuer is the
	 * current default identity.
	 * 
	 * @param limited
	 *            Generate a limited proxy ?
	 * @param lifeTime
	 *            Description of the Parameter
	 * @param keyLength
	 *            Description of the Parameter
	 * @exception Exception
	 *                Description of the Exception
	 */
	public static ProxyCertificate getInstance(X509Certificate userCert,
			boolean limited, long lifeTime, int keyLength) throws Exception {
		logger.finest("Limited: " + limited + " valid: " + lifeTime
				+ " key size: " + keyLength);
		if (userCert == null) {
			throw new IllegalArgumentException("Invalid user certifcate");
		}
		// give 5 minutes grace period here to avoid difficulties with not
		// synchronized clocks
		long validFrom = System.currentTimeMillis() - 300000;
		long validTo = System.currentTimeMillis() + lifeTime;
		if (validTo <= validFrom) {
			throw new IllegalArgumentException(
					"Invalid lifetime for proxy certifcate");
		}
		if ((keyLength != 512) && (keyLength != 1024) && (keyLength != 2048)
				&& (keyLength != 4096)) {
			throw new IllegalArgumentException(
					"Invalid key length for proxy certifcate");
		}
		return new ProxyCertificate(userCert, limited, validFrom, validTo,
				keyLength);
	}

	public void writePEM(OutputStream fos) throws Exception {
		String certStart = "-----BEGIN CERTIFICATE-----\n";
		String certEnd = "\n-----END CERTIFICATE-----\n";
		String keyStart = "-----BEGIN RSA PRIVATE KEY-----\n";
		String keyEnd = "\n-----END RSA PRIVATE KEY-----\n";
		// proxy certificate
		fos.write(certStart.getBytes());
		fos.write(Base64.encode(proxyCertificate.getEncoded(), true));
		fos.write(certEnd.getBytes());
		// proxy certificate private key
		fos.write(keyStart.getBytes());
		fos.write(Base64.encode(getEncoded(privateKey), true));
		fos.write(keyEnd.getBytes());
		// users public certificate
		fos.write(certStart.getBytes());
		fos.write(Base64.encode(userCertificate.getEncoded(), true));
		fos.write(certEnd.getBytes());
	}

	/**
	 * Returns the encoded byte array representing the PrivateKey
	 * 
	 * @return the private key byte array
	 * @exception <code>IOException</code> whenever an error occurs during
	 *                decoding.
	 */
	private byte[] getEncoded(PrivateKey privateKey) throws IOException {
		org.bouncycastle.asn1.pkcs.PrivateKeyInfo pkey = null;
		ByteArrayInputStream bs = new ByteArrayInputStream(privateKey
				.getEncoded());
		DERInputStream derin = new DERInputStream(bs);
		DERObject keyInfo = derin.readObject();
		pkey = new org.bouncycastle.asn1.pkcs.PrivateKeyInfo(
				(ASN1Sequence) keyInfo);
		DERObject derKey = pkey.getPrivateKey();
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		DEROutputStream derout = new DEROutputStream(bout);
		derout.writeObject(derKey);
		return bout.toByteArray();
	}

	private X509Certificate newproxyCertificate() throws Exception {
		String newissuer = "";
		// The Sun JCE is buggy!!!
		// The ordering of the SubjectDN fields for key entries is reversed
		// We have to create a new certificate in order to match the issuerDN
		// in the proxy with the original user cert !!
		// Simply create a new certificate from the DER encoding
		// Alternative: set issuer DN == subjectDN (see below)
		String issuer = userCertificate.getSubjectDN().getName();
		int start = -1;
		if ((start = issuer.indexOf("EMAILADDRESS")) >= 0) {
			issuer = issuer.substring(0, start) + "EmailAddress"
					+ issuer.substring(start + "EmailAddress".length());
		}
		String mark = "";
		StringTokenizer st = new StringTokenizer(issuer, ",");
		while (st.hasMoreTokens()) {
			String next = st.nextToken();
			if (st.hasMoreTokens()) {
				mark = "," + next;
			}
			else {
				mark = next;
			}
			newissuer = mark + newissuer;
		}
		// expand the users CN with the CN=proxy entry
		String proxyExpand = "CN=proxy";
		if (limited) {
			// special tag for limited proxy certificates
			proxyExpand = "limited proxy";
		}
		/**
		 * trick: set the issuer and subject DN to equal values (except the
		 * "proxy" ) entry this circumvents the re-ordering of DN entries of the
		 * SUn JCE for key entries!!
		 */
		X509Name issuerDN = new X509Name(newissuer);
		X509Name subjectDN = new X509Name(newissuer + ", " + proxyExpand);
		//
		// create the certificate - version 3 without extensions
		//
		v3CertGen.reset();
		v3CertGen.setIssuerDN(issuerDN);
		v3CertGen.setSubjectDN(subjectDN);
		v3CertGen.setSerialNumber(userCertificate.getSerialNumber());
		v3CertGen.setNotBefore(new Date(validFrom));
		v3CertGen.setNotAfter(new Date(validTo));
		// this is the *new* public key !!
		v3CertGen.setPublicKey(publicKey);
		v3CertGen.setSignatureAlgorithm(userCertificate.getSigAlgName());
		// add extensions
		boolean[] keyUsage = userCertificate.getKeyUsage();
		// generate cert
		X509Certificate proxyCert = ResourceManager.getKeystoreManager()
				.signCertificate(v3CertGen, userCertificate);
		// recheck
		proxyCert.checkValidity(new Date());
		proxyCert.verify(userCertificate.getPublicKey());
		logger.info("Generated proxyCert for: \n" + proxyCert.getSubjectDN());
		return proxyCert;
	}
}