/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.plugins.script;

import java.awt.Font;
import java.io.File;
import java.util.Enumeration;
import org.unicore.ajo.ScriptType;
import com.pallas.unicore.client.util.Defaults;

/**
 * Define, load and save script plugin default values
 * 
 * @author Ralf Ratering
 * @version $Id: ScriptDefaults.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class ScriptDefaults extends Defaults {

	/**
	 * String constants defining property keys
	 */
	public final static String SCRIPTDIRECTORY = "plugin.script.scriptdirectory",
			TYPE = "plugin.script.type",
			EDITORFONTNAME = "plugin.script.editor.fontname",
			EDITORFONTSIZE = "plugin.script.editor.fontsize",
			EDITORLINEWRAP = "plugin.script.editor.linewrap";

	/**
	 * Fill properties with default values
	 */
	protected void genSystemDefaults() {
		put(SCRIPTDIRECTORY, userHome);
		put(TYPE, ScriptType.SH.toString());
		put(EDITORFONTNAME, "Default");
		put(EDITORFONTSIZE, "12");
		put(EDITORLINEWRAP, "true");
	}

	/**
	 * Get the editor font
	 * 
	 * @return The editor font
	 */
	public Font getEditorFont() {
		String fontName = (String) get(EDITORFONTNAME);
		int fontSize = intValue(EDITORFONTSIZE);
		return new Font(fontName, Font.PLAIN, fontSize);
	}

	/**
	 * Get the line wrap flag for editor
	 * 
	 * @return boolean line wrap value
	 */
	public boolean getLineWrap() {
		return booleanValue(EDITORLINEWRAP);
	}

	/**
	 * Get the script directory
	 * 
	 * @return script directory value
	 */
	public String getScriptDirectory() {
		return getProperty(SCRIPTDIRECTORY);
	}

	/**
	 * Get the script type
	 * 
	 * @return script type
	 */
	public ScriptType getType() {
		String typeString = getProperty(TYPE);
		Enumeration typeEnum = ScriptType.elements();
		while (typeEnum.hasMoreElements()) {
			ScriptType type = (ScriptType) typeEnum.nextElement();
			if (type.toString().equals(typeString)) {
				return type;
			}
		}
		return null;
	}

	/**
	 * Get script type as String
	 * 
	 * @return script type as String
	 */
	public String getTypeString() {
		return getProperty(TYPE);
	}

	/**
	 * Load script default values from file ${HOME}/.unicore/ScriptDefaults.txt
	 * 
	 * @return true if loading was successful, else false
	 */
	public boolean loadFromFile() {
		// look for ${HOME}/.unicoreXX/ScriptDefaults.txt
		return loadFromFile(unicoreDir + File.separator + "ScriptDefaults.txt");
	}

	/**
	 * Set the editor font
	 * 
	 * @param font
	 *            The new editor font
	 */
	public void setEditorFont(Font font) {
		put(EDITORFONTNAME, font.getFontName());
		put(EDITORFONTSIZE, new Integer(font.getSize()).toString());
	}

	/**
	 * Set the line wrap flag for editor
	 * 
	 * @param b
	 *            boolean line wrap flag
	 */
	public void setLineWrap(boolean b) {
		put(EDITORLINEWRAP, new Boolean(b).toString());
	}

	/**
	 * Set the script directory
	 * 
	 * @param s
	 *            The new script directory
	 */
	public void setScriptDirectory(String s) {
		put(SCRIPTDIRECTORY, s);
	}

	/**
	 * Set the script type as string
	 * 
	 * @param type
	 *            script type as string
	 */
	public void setTypeString(String type) {
		put(TYPE, type);
	}

	/**
	 * Try to write script default values to file
	 * ${HOME}/.unicore/ScriptDefaults.txt
	 * 
	 * @return true if saving was successful, else false
	 */
	public boolean writeToFile() {
		// always overwrite old ${HOME}/.unicore/ScriptDefaults.txt
		return writeToFile(unicoreDir + File.separator + "ScriptDefaults.txt");
	}
}