/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.tables;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.EventObject;
import java.util.Vector;
import javax.swing.DefaultCellEditor;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.TableModelEvent;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.extensions.LocalExportEntry;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.VectorSorter;

/**
 * @author Ralf Ratering
 * @author Thomas Kentemich
 * @version $Id: LocalExportTable.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class LocalExportTable extends AdjustTable {

	private static TableCellRenderer cellRenderer = new CellRenderer();
	private final static String[] columnIdentifiers = {"Source", "Destination",
			"Overwrite", ""};
	private boolean ascending;
	private boolean initialized = false;
	private Vector lastData;
	private int lastSortColumn = -1;
	private DetailsTableModel model;
	private Vector selectedEntries = new Vector();

	/**
	 * Constructor
	 */
	public LocalExportTable() {
		super();
		setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		buildComponents();
		getTableHeader().addMouseListener(new TableHeaderListener());
	}

	private void adjustWidthsAndLookForIterations(Vector rows) {
		adjustWidths();
	}

	private void buildComponents() {
		model = new DetailsTableModel(columnIdentifiers, 0);
		model.setColumnIdentifiers(columnIdentifiers);
		setModel(model);
		setDefaultRenderer(JTextField.class, cellRenderer);
		ExportTableEditor editor = new ExportTableEditor(new JTextField());
		setDefaultEditor(ImageIcon.class, editor);
		setFont(new Font(getFont().getName(), getFont().getStyle(), 11));
	}

	public void clear() {
		int rowCount = getRowCount();
		for (int i = 0; i < rowCount; i++) {
			model.removeRow(0);
		}
	}

	public Object getEntryAt(int row, int col) {
		return model.getValueAt(row, col);
	}

	public Vector getSelectedEntries() {
		selectedEntries.clear();
		int[] indices = getSelectedRows();
		for (int i = 0; i < indices.length; i++) {
			LocalExportEntry entry = (LocalExportEntry) model.getDataVector()
					.elementAt(indices[i]);
			FileExport fe = entry.getExport();
			fe.setDestinationName((String) model.getValueAt(indices[i], 1));
			fe.setOverwritePolicy(((Boolean) model.getValueAt(indices[i], 2))
					.booleanValue());
			selectedEntries.add(entry);
			//         System.out.println("New selection ["+i+"]
			// "+model.getDataVector().elementAt(indices[i]));
		}
		return selectedEntries;
	}

	public void reloadData() {
		setDataVector(lastData);
		model.fireTableDataChanged();
	}

	public void resort() {
		ascending = !ascending;
		sortByColumn(lastSortColumn);
	}

	public void setDataVector(Vector rows) {
		this.clear();
		lastData = rows;
		model.setRowCount(0);
		for (int i = 0; i < rows.size(); i++) {
			model.addRow((Vector) rows.elementAt(i));
		}
		resort();
		if (selectedEntries.size() == 0) {
			selectedEntries = rows;
			setRowSelectionInterval(0, rows.size() - 1);
		}
		if (rows.size() > 0 && !initialized) {
			// adjustWidthsAndLookForIterations(rows);
			initialized = true;
		}
	}

	public void sortByColumn(int column) {
		if (column < 0) {
			return;
		}
		if (lastSortColumn == column) {
			ascending = !ascending;
		}
		else {
			ascending = true;
		}
		LocalExportEntry.setCompareIndex(column, 0);
		Vector dataVector = model.getDataVector();
		VectorSorter.sort(dataVector, ascending);
		lastSortColumn = column;
	}

	public void tableChanged(TableModelEvent evt) {
		super.tableChanged(evt);
	}

	protected static class CellRenderer implements TableCellRenderer {

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			String valueString = value.toString();
			JLabel label = new JLabel(valueString);
			label.setOpaque(true);
			label.setFont(table.getFont());
			Vector data = ((DefaultTableModel) table.getModel())
					.getDataVector();
			LocalExportEntry entry = (LocalExportEntry) data.elementAt(row);
			Color whichColor = table.getBackground();
			label.setBackground(whichColor);
			return label;
		}
	}

	public class DetailsTableModel extends DefaultTableModel {

		public DetailsTableModel(Object[] names, int rows) {
			super(names, rows);
		}

		/**
		 * JTable uses this method to determine the default renderer editor for
		 * each cell. If we didn't implement this method, then the last column
		 * would contain text ("true"/"false"), rather than a check box.
		 */
		public Class getColumnClass(int c) {
			Object columnObject = getValueAt(0, c);
			if (columnObject == null) {
				String dummy = new String();
				return dummy.getClass();
			}
			else {
				return columnObject.getClass();
			}
		}

		public boolean isCellEditable(int row, int col) {
			if (col == 0) {
				return false;
			}
			else {
				return true;
			}
		}
	}

	public class ExportTableEditor extends DefaultCellEditor {

		private SelectorDialog chooser;
		private String currentFile = System.getProperty("user.dir");
		private Object inValue;
		private int lastCol;
		private int lastRow;

		public ExportTableEditor(JTextField textField) {
			super(textField);
			textField.setEditable(false);
			chooser = ResourceManager.getSelectorDialog(new File(currentFile),
					null);
			chooser.setTitle("UNICORE: Choose destination for export");
			chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		}

		public void cancelCellEditing() {
			fireEditingCanceled();
		}

		public Object getCellEditorValue() {
			return inValue;
		}

		public Component getTableCellEditorComponent(JTable table,
				Object value, boolean isSelected, int row, int col) {
			((JTextField) editorComponent).setText("Browse...");
			inValue = value;
			lastRow = row;
			lastCol = col;
			return editorComponent;
		}

		public boolean isCellEditable(EventObject evt) {
			if (evt instanceof MouseEvent) {
				if (((MouseEvent) evt).getClickCount() >= getClickCountToStart()) {
					return true;
				}
			}
			return false;
		}

		public boolean shouldSelectCell(EventObject evt) {
			if (isCellEditable(evt)) {
				int result = chooser.showDialog();
				if (result == JFileChooser.APPROVE_OPTION) {
					currentFile = chooser.getSelectedFile().getAbsolutePath();
					boolean ret = stopCellEditing();
				}
				else {
					cancelCellEditing();
				}
			}
			return true;
		}

		public boolean stopCellEditing() {
			LocalExportEntry entry = (LocalExportEntry) model.getDataVector()
					.elementAt(lastRow);
			if (entry == null) {
				return false;
			}
			FileExport fe = entry.getExport();
			fe.setDestinationName(currentFile);
			model.setValueAt(currentFile, lastRow, 1);
			reloadData();
			((DefaultTableModel) getModel()).fireTableDataChanged();
			fireEditingStopped();
			return true;
		}
	}

	protected class TableHeaderListener extends MouseAdapter {

		public TableHeaderListener() {
		}

		public void mouseClicked(MouseEvent e) {
			TableColumnModel columnModel = getColumnModel();
			int viewColumn = columnModel.getColumnIndexAtX(e.getX());
			int column = convertColumnIndexToModel(viewColumn);
			sortByColumn(column);
		}
	}
}