/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.themes;

import java.util.Enumeration;
import java.util.Hashtable;
import javax.swing.LookAndFeel;
import javax.swing.UIManager;
import javax.swing.plaf.metal.DefaultMetalTheme;

/**
 * A class that manages different look and feels and metal themes
 * 
 * @author Ralf Ratering
 * @version $Id: LookAndFeelManager.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class LookAndFeelManager {

	// look and feel
	private final static String mac = "com.sun.java.swing.plaf.mac.MacLookAndFeel";
	private final static String metal = "javax.swing.plaf.metal.MetalLookAndFeel";
	private final static String motif = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
	// table containing plaf name/class pairs
	private static Hashtable plafTable = getAvailableLookAndFeels(),
			themeTable = getAvailableThemes();
	private final static String windows = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";

	/**
	 * Build a Hashtable that contains the Plaf name as key and the
	 * corresponding class name as entry. This is needed to display the plafs
	 * with their short names in a GUI element.
	 * 
	 * @return Hashtable containing name/class pairs
	 */
	public static Hashtable getAvailableLookAndFeels() {
		Hashtable table = new Hashtable();
		if (isAvailableLookAndFeel(mac)) {
			table.put("Mac", mac);
		}
		if (isAvailableLookAndFeel(metal)) {
			table.put("Metal", metal);
		}
		if (isAvailableLookAndFeel(motif)) {
			table.put("Motif", motif);
		}
		if (isAvailableLookAndFeel(windows)) {
			table.put("Windows", windows);
		}
		return table;
	}

	public static Hashtable getAvailableThemes() {
		Hashtable table = new Hashtable();
		DefaultMetalTheme theme = new DefaultMetalTheme();
		table.put(theme.getName(), theme);
		theme = new SystemTheme();
		table.put(theme.getName(), theme);
		theme = new AquaTheme();
		table.put(theme.getName(), theme);
		theme = new BigContrastTheme();
		table.put(theme.getName(), theme);
		theme = new CharcoalTheme();
		table.put(theme.getName(), theme);
		theme = new ContrastTheme();
		table.put(theme.getName(), theme);
		theme = new DemoTheme();
		table.put(theme.getName(), theme);
		theme = new EmeraldTheme();
		table.put(theme.getName(), theme);
		theme = new KhakiTheme();
		table.put(theme.getName(), theme);
		theme = new RubyTheme();
		table.put(theme.getName(), theme);
		theme = new UserTheme();
		table.put(theme.getName(), theme);
		return table;
	}

	public static String getCrossPlatformPlaf() {
		return UIManager.getCrossPlatformLookAndFeelClassName();
	}

	public static String getCrossPlatformPlafName() {
		return getPlafByFullName(getCrossPlatformPlaf());
	}

	public static String getPlafByFullName(String name) {
		Enumeration keys = plafTable.keys();
		while (keys.hasMoreElements()) {
			String plafName = (String) keys.nextElement();
			if (plafTable.get(plafName).equals(name)) {
				return plafName;
			}
		}
		return null;
	}

	public static String getPlafByName(String name) {
		return (String) plafTable.get(name);
	}

	public static Object[] getPlafNames() {
		return plafTable.keySet().toArray();
	}

	public static String getStandardTheme() {
		return new DefaultMetalTheme().getName();
	}

	public static DefaultMetalTheme getThemeByName(String name) {
		return (DefaultMetalTheme) themeTable.get(name);
	}

	public static Object[] getThemeNames() {
		return themeTable.keySet().toArray();
	}

	public static boolean isAvailableLookAndFeel(String plaf) {
		try {
			Class lnfClass = Class.forName(plaf);
			LookAndFeel newLAF = (LookAndFeel) (lnfClass.newInstance());
			return newLAF.isSupportedLookAndFeel();
		}
		catch (Exception e) {
			// If ANYTHING weird happens, return false
			return false;
		}
	}
}