/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.trees;

import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.logging.Logger;

import javax.swing.AbstractAction;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import com.pallas.unicore.client.Client;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Abstract base class for a panel that shows jobs for jpa/jmc as a tree
 *
 *@author     Ralf Ratering
 *@version    $Id: ClientTree.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */
public abstract class ClientTree extends JTree implements ClipboardOwner {

	protected static Clipboard clipboard = new Clipboard("treeclipboard");

	// id for key listener actions
	private static int eventId = 0;

	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.trees");

	/**
	 *  reference to UNICORE client
	 */
	protected Client client;

	protected CopyAction copyAction = new CopyAction();
	protected ContainerNode currentGroupNode, currentTopLevelJobNode;
	protected CutAction cutAction = new CutAction();
	protected PasteAction pasteAction = new PasteAction();
	protected RemoveAction removeAction = new RemoveAction();

	/**
	 *  selected node that has focus
	 */
	protected PanelNode focussedNode;

	/**
	 *  object corresponding to focussed node
	 */
	protected Object focussedObject;

	/**
	 *  panel corresponding to focussed node
	 */
	protected JPanel focussedPanel;

	/**
	 *  tree model
	 */
	protected DefaultTreeModel model;

	protected JPopupMenu popup = new JPopupMenu();

	/**
	 *  root node
	 */
	protected PanelNode root;

	/**
	 *  Constructor inits tree
	 *
	 *@param  client  reference to UNICORE client
	 */
	public ClientTree(Client client) {
		this.client = client;

		root = new PanelNode("");

		model = new DefaultTreeModel(root);
		model.setAsksAllowsChildren(true);

		setModel(model);

		// some cosmetics
		setCellRenderer(new ClientCellRenderer());
		setEditable(false);
		//setBackground(Color.white);
		putClientProperty("JTree.lineStyle", "Angled");

		// listen to mouse clicks...
		addMouseListener(new TreeMouseListener());

		// ...and selections
		addTreeSelectionListener(new MyTreeSelectionListener());

		// ...and keyboard events
		addKeyListener(new MyKeyListener());

		// allow only selection of one node at a time
		getSelectionModel().setSelectionMode(
			TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);

	}

	/**
	 * Abstract copy action
	 */
	protected abstract void copy();

	/**
	 * Abstract cut action
	 */
	protected abstract void cut();

	/**
	 * define default actions
	 *
	 */
	protected abstract void enableDefaultActions();

	private TreeNode findJobNode(TreeNode node) {
		if (!(node instanceof ContainerNode)) {
			return null;
		}
		while (node.getParent() instanceof ContainerNode) {
			node = node.getParent();
			if (node == null) {
				return null;
			}
		}
		return node;
	}

	public AbstractAction getCutAction() {
		return cutAction;
	}

	public AbstractAction getCopyAction() {
		return copyAction;
	}

	public AbstractAction getPasteAction() {
		return pasteAction;
	}

	public AbstractAction getRemoveAction() {
		return removeAction;
	}

	/**
	 *  Gets the current GroupContainer
	 */
	public GroupContainer getCurrentGroup() {
		if (currentGroupNode == null) {
			return null;
		}
		return (GroupContainer) currentGroupNode.getActionContainer();
	}

	/**
	 *  Gets the current Job Node 
	 */
	public ContainerNode getCurrentGroupNode() {
		return currentGroupNode;
	}

	/**
	 *  Gets the current Job Container
	 */
	public JobContainer getCurrentJob() {
		return (JobContainer) getCurrentJobNode().getActionContainer();
	}

	/**
	 *  Gets the current Job Node
	 */
	public ContainerNode getCurrentJobNode() {
		ContainerNode node = getCurrentGroupNode();
		if (node == null) {
			return null;
		}
		while (!node.isJobGroup()) {
			node = (ContainerNode) node.getParent();
		}
		return node;
	}

	/**
	 *  Gets the current Top Level JobContainer
	 */
	public JobContainer getCurrentTopLevelJob() {
		if (currentTopLevelJobNode == null) {
			return null;
		}
		return (JobContainer) currentTopLevelJobNode.getActionContainer();
	}

	/**
	 *  Gets the current Top Level Job Node
	 */
	public ContainerNode getCurrentTopLevelJobNode() {
		return currentTopLevelJobNode;
	}

	/**
	 * Returns the UNICORE client instance
	 */
	public Client getClient() {
		return client;
	}

	/**
	 *  Gets the focussed node
	 */
	public PanelNode getFocussedNode() {
		return focussedNode;
	}

	/**
	 *  Gets the object belonging to the focussed node
	 */
	public Object getFocussedObject() {
		return focussedObject;
	}

	/**
	 * Check if root node is selected
	 */
	public boolean isRootSelected() {
		return root.equals(focussedNode);
	}

	/**
	 * Check if top level node is selected
	 */
	public boolean isTopLevelJobNodeSelected() {
		return focussedNode == currentTopLevelJobNode;
	}

	/**
	 *  ClipboardOwner interface
	 *
	 */
	public void lostOwnership(Clipboard clipboard, Transferable transferable) {
		logger.warning("Clipboard " + clipboard.getName() + " lost ownership!");
	}

	protected abstract void paste();
	protected abstract void remove();

	public boolean selectionContainsContainerNode() {
		TreePath[] paths = getSelectionPaths();
		if (paths == null) {
			return false;
		}
		for (int i = 0; i < paths.length; i++) {
			if (paths[i].getLastPathComponent() instanceof ContainerNode) {
				return true;
			}
		}
		return false;
	}

	public void selectNode(PanelNode node) {
		synchronized (root) {
			setSelectionPath(new TreePath(node.getPath()));
		}
	}

	public void selectRootNode() {
		selectNode(root);
	}
	public void setFocussedNode(PanelNode node) {
		this.focussedNode = node;
	}

	private void testShowPopup(MouseEvent evt) {
		if (evt.isPopupTrigger()) {
			enableDefaultActions();
			popup.show(this, evt.getX(), evt.getY());
		}
	}

	public void updateNode(TreeNode node) {
		synchronized (root) {
			model.nodeChanged(node);
		}
	}
	protected void updateSelection() {
		synchronized (root) {
			// if no node has been selected, automatically select last selected nodes
			// and if it's the first time, select root node
			if (isSelectionEmpty()) {
				focussedNode = root;
				setSelectionPath(new TreePath(focussedNode.getPath()));
			}

			// set focus to last selected node
			TreePath path = getLeadSelectionPath();
			focussedNode = (PanelNode) path.getLastPathComponent();
			focussedObject = focussedNode.getUserObject();
			focussedPanel = focussedNode.getPanel();

			currentTopLevelJobNode = (ContainerNode) findJobNode(focussedNode);

			if (focussedNode.isGroup()) {
				currentGroupNode = (ContainerNode) focussedNode;
			}
			else if (focussedNode.isTask()) {
				currentGroupNode = (ContainerNode) focussedNode.getParent();
			}
			// root selected
			else {
				currentGroupNode = null;
			}
		}
	}

	public void updateSubtree(TreeNode node) {
		model.nodeChanged(node);
		for (int i = 0; i < node.getChildCount(); i++) {
			updateSubtree(node.getChildAt(i));
		}
	}

	class CopyAction extends AbstractAction {
		public CopyAction() {
			super("Copy", ResourceManager.getIcon(ResourceManager.COPY));
			setEnabled(false);
		}
		public void actionPerformed(ActionEvent e) {
			copy();
		}
	}

	class CutAction extends AbstractAction {
		public CutAction() {
			super("Cut", ResourceManager.getIcon(ResourceManager.CUT));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			cut();
		}
	}

	private class MyKeyListener extends KeyAdapter {
		public void keyPressed(KeyEvent e) {
			int keyCode = e.getKeyCode();
			int modifiers = e.getModifiers();
			// increase id
			eventId++;
			if (keyCode == KeyEvent.VK_X && modifiers == KeyEvent.CTRL_MASK) {
				cutAction.actionPerformed(new ActionEvent(this, eventId, ""));
			}
			else if (
				keyCode == KeyEvent.VK_C && modifiers == KeyEvent.CTRL_MASK) {
				copyAction.actionPerformed(new ActionEvent(this, eventId, ""));
			}
			else if (
				keyCode == KeyEvent.VK_V && modifiers == KeyEvent.CTRL_MASK) {
				pasteAction.actionPerformed(new ActionEvent(this, eventId, ""));
			}
			else if (keyCode == KeyEvent.VK_DELETE && modifiers == 0) {
				removeAction.actionPerformed(
					new ActionEvent(this, eventId, ""));
			}
		}
	}

	/**
	 *  Listen to tree selections and call updateSelection method
	 */
	private class MyTreeSelectionListener implements TreeSelectionListener {
		public void valueChanged(TreeSelectionEvent event) {
			updateSelection();
			enableDefaultActions();
		}
	}

	class PasteAction extends AbstractAction {
		public PasteAction() {
			super("Paste", ResourceManager.getIcon(ResourceManager.PASTE));
			setEnabled(false);
		}
		public void actionPerformed(ActionEvent e) {
			paste();
		}
	}

	class RemoveAction extends AbstractAction {
		public RemoveAction() {
			super("Remove", ResourceManager.getIcon(ResourceManager.REMOVE));
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			remove();
		}
	}

	class TreeMouseListener extends MouseAdapter {
		public void mouseClicked(MouseEvent e) {
			enableDefaultActions();
			testShowPopup(e);
		}

		public void mousePressed(MouseEvent e) {
			if (!hasFocus()) {
				requestFocus();
				updateSelection();
			}
			testShowPopup(e);
		}

		public void mouseReleased(MouseEvent e) {
			testShowPopup(e);
		}

	}

}
