/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.trees;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.tree.DefaultMutableTreeNode;
import org.unicore.Vsite;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.ElseContainer;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.IfContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.container.ThenContainer;
import com.pallas.unicore.extensions.Usite;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Node that contains a JPanel a user object and an icon, keeps track of a state
 * and allows transfer form and to the clipboard.
 * 
 * @author Ralf Ratering
 * @version $Id: PanelNode.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */
public class PanelNode extends DefaultMutableTreeNode implements Transferable {

	private static DataFlavor flavor;
	protected static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.trees");
	/**
	 * constants defining different states
	 */
	public final static int STATE_UNKNOWN = ActionContainer.STATE_UNKNOWN,
			STATE_UPDATING = ActionContainer.STATE_UPDATING,
			STATE_UP_TO_DATE = ActionContainer.STATE_UP_TO_DATE;

	public static DataFlavor getFlavor() {
		return flavor;
	}
	/**
	 * icon belonging to this node
	 */
	protected ImageIcon icon, currentIcon;
	private Object lock = new Object();
	// panel belonging to this node
	private JPanel panel;
	// current node state
	private int state = STATE_UNKNOWN;

	/**
	 * Constructor passes user object through to super class
	 * 
	 * @param object
	 *            user object stored with node
	 */
	public PanelNode(Object object) {
		super(object);
		try {
			flavor = new DataFlavor(Class
					.forName("com.pallas.unicore.client.trees.PanelNode"),
					"PanelNode");
		}
		catch (ClassNotFoundException e) {
			logger.log(Level.WARNING, "Could find PanelNode class.", e);
		}
	}

	/**
	 * Constructor gets user object and panel as input
	 * 
	 * @param object
	 *            user object stored with node
	 * @param panel
	 *            JPanel stored with node
	 */
	public PanelNode(Object object, JPanel panel) {
		this(object);
		this.panel = panel;
	}

	public ImageIcon getCurrentIcon() {
		return currentIcon;
	}

	/**
	 * Get icon that is stored with this node
	 * 
	 * @return JPanel
	 */
	public ImageIcon getIcon() {
		return icon;
	}

	/**
	 * Get panel that is stored with this node
	 * 
	 * @return JPanel
	 */
	public JPanel getPanel() {
		return panel;
	}

	/**
	 * Get the current state
	 * 
	 * @return current state
	 */
	public int getState() {
		synchronized (lock) {
			return state;
		}
	}

	public synchronized Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException {
		if (flavor.equals(flavor)) {
			return this;
		}
		else {
			throw new UnsupportedFlavorException(flavor);
		}
	}

	/**
	 * Transferable interface.
	 *  
	 */
	public synchronized DataFlavor[] getTransferDataFlavors() {
		DataFlavor[] flavors = {flavor};
		return flavors;
	}

	/**
	 * Transferable interface.
	 */
	public boolean isDataFlavorSupported(DataFlavor flavor) {
		return (flavor.equals(flavor));
	}

	/**
	 * Test type of user object
	 * 
	 * @return true if it's a group
	 */
	public boolean isGroup() {
		return (userObject instanceof GroupContainer);
	}

	public boolean isIfGroup() {
		return (userObject instanceof IfContainer);
	}

	/**
	 * Test type of user object
	 * 
	 * @return true if it's a job group
	 */
	public boolean isJobGroup() {
		return (userObject instanceof JobContainer);
	}

	/**
	 * Overwrite method, because tasks have no children
	 * 
	 * @return The leaf value
	 */
	public boolean isLeaf() {
		return isTask();
	}

	/**
	 * Test type of user object
	 * 
	 * @return true if it's a task
	 */
	public boolean isTask() {
		return (userObject instanceof TaskContainer);
	}

	public boolean isThenElseGroup() {
		return (userObject instanceof ThenContainer)
				|| (userObject instanceof ElseContainer);
	}

	public boolean isTopLevelJobGroup() {
		if (userObject instanceof JobContainer) {
			JobContainer job = (JobContainer) userObject;
			return job.getParentContainer() == null;
		}
		return false;
	}

	/**
	 * Ask, if this node is currently updating
	 * 
	 * @return true if it's updating
	 */
	public boolean isUpdating() {
		return (getState() == STATE_UPDATING);
	}

	/**
	 * Ask, if this node is up to date
	 * 
	 * @return true if it's up to date
	 */
	public boolean isUpToDate() {
		return (getState() == STATE_UP_TO_DATE);
	}

	/**
	 * Test type of user object
	 * 
	 * @return true if it's a usite
	 */
	public boolean isUsite() {
		return (userObject instanceof Usite);
	}

	/**
	 * Test type of user object
	 * 
	 * @return true if it's a vsite
	 */
	public boolean isVsite() {
		return (userObject instanceof Vsite);
	}

	public void setCurrentIcon(ImageIcon currentIcon) {
		this.currentIcon = currentIcon;
	}

	public void setIcon(ImageIcon icon) {
		this.icon = icon;
		this.currentIcon = icon;
	}

	/**
	 * Set state
	 * 
	 * @param state
	 *            should be one of the defined state constants
	 */
	public void setState(int state) {
		synchronized (lock) {
			this.state = state;
		}
		switch (state) {
			case STATE_UNKNOWN :
			case STATE_UP_TO_DATE :
				currentIcon = icon;
				break;
			case STATE_UPDATING :
				currentIcon = ResourceManager.getIcon(ResourceManager.WAIT);
				break;
		}
	}

	/**
	 * Set this node's state to "up to date"
	 */
	public void setUpToDate() {
		setState(STATE_UP_TO_DATE);
	}
}