/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.util;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import javax.swing.ImageIcon;
import com.pallas.unicore.client.Client;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 * Class that scans directories for plugins and starts them
 * 
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @version $Id: PluginManager.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */
public class PluginManager {

	protected static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.util");
	protected Hashtable iconTable = new Hashtable();
	protected Vector knownPlugins = new Vector();
	protected Vector pluginClasses = new Vector();
	protected Vector taskPluginClasses = new Vector();

	public ImageIcon getIcon(ActionContainer container) {
		ImageIcon icon = (ImageIcon) iconTable.get(container.getClass());
		if (icon == null) {
			icon = ResourceManager.getIcon(ResourceManager.UNKNOWNTASK);
		}
		return icon;
	}

	/**
	 * Return the initializer classes of all plugins
	 * 
	 *  
	 */
	public Vector getPluginClasses() {
		return pluginClasses;
	}

	/**
	 * Return all known plugins
	 * 
	 */
	public Vector getPlugins() {
		return knownPlugins;
	}

	public Vector getTaskPlugins() {
		return taskPluginClasses;
	}

	private void scanDirectory(String dirname) {
		logger.info("Scanning directory: " + dirname);
		File dir = new File(dirname);
		if (!dir.exists() || !dir.isDirectory()) {
			return;
		}
		String[] plugins = dir.list();
		if (plugins == null) {
			return;
		}
		for (int i = 0; i < plugins.length; i++) {
			String plugin = plugins[i];
			// log.logWrite("Scanning file: "+plugin);
			if (!plugin.toLowerCase().endsWith("plugin.jar")) {
				continue;
			}
			ZipFile zip;
			try {
				zip = new ZipFile(dirname + File.separator + plugin);
			}
			catch (IOException e) {
				logger
						.log(Level.WARNING, "Could not open plugin: " + plugin,
								e);
				continue;
			}
			logger.info("Found plugin jar: " + plugin);
			URL urls[] = new URL[1];
			URLClassLoader urlLoader = null;
			try {
				urls[0] = new URL("file:" + dirname + File.separator + plugin);
				Class self = PluginManager.class;
				urlLoader = new URLClassLoader(urls, self.getClassLoader());
			}
			catch (MalformedURLException mfue) {
				UserMessages
						.error("Cannot convert file name to URL: " + plugin);
				continue;
			}
			Enumeration entries = zip.entries();
			while (entries.hasMoreElements()) {
				ZipEntry zipEntry = (ZipEntry) entries.nextElement();
				String name = zipEntry.getName();
				if (!name.toLowerCase().endsWith("plugin.class")) {
					continue;
				}
				boolean duplicate = false;
				for (int j = 0; j < knownPlugins.size(); j++) {
					UnicorePlugin pl = (UnicorePlugin) knownPlugins
							.elementAt(j);
					// assume that packages are separated by '/'s
					String currentClassName = name.replace('/', '.');
					String loadedClassName = pl.getPluginClass().getName()
							+ ".class";
					if (loadedClassName.equals(currentClassName)) {
						logger.warning("Plugin class: " + currentClassName
								+ " already loaded");
						duplicate = true;
						continue;
					}
				}
				if (!duplicate) {
					logger.info("Found plugin class: " + name);
					UnicorePlugin unicorePlugin = new UnicorePlugin(dirname
							+ File.separator + plugin, name, urls, urlLoader);
					if (unicorePlugin.isTrusted()) {
						knownPlugins.add(unicorePlugin);
					}
				}
			}
		}
	}

	public void scanPlugins(String pluginPath) {
		String jarDir = ResourceManager.getJarDirectory();
		// first look for plugins in current directory
		scanDirectory(jarDir);
		// first look for plugins in system default directory
		//        scanDirectory(jarDir + "/../plugin");
		//.. now take those from user default plugin dir
		scanDirectory(pluginPath);
	}

	public void startPlugins(final Client parent) {
		for (int i = 0; i < knownPlugins.size(); i++) {
			final UnicorePlugin plugin = (UnicorePlugin) knownPlugins
					.elementAt(i);
			if (!(plugin.isTrusted())) {
				continue;
			}
			// problem: instantiation of plugins that e.g. overwrite
			// final methods will cause a VerifyError instead of an exception...
			try {
				logger.info("Loading plugin: " + plugin.getPluginName());
				Class pluginClass = plugin.getPluginClass();
				UnicorePlugable plugable = (UnicorePlugable) pluginClass
						.newInstance();
				plugable.startPlugin();
				pluginClasses.add(plugable);
				if (plugable instanceof TaskPlugable) {
					taskPluginClasses.add(plugable);
					TaskPlugable taskPlugable = (TaskPlugable) plugable;
					iconTable.put(taskPlugable.getContainerClass(),
							new ImageIcon(taskPlugable.getIconImage()));
				}
			}
			catch (Error e) {
				UserMessages.error("Cannot start plugin <"
						+ plugin.getPluginName() + ">.", e.getMessage());
				logger.log(Level.SEVERE, "", e);
			}
			catch (Exception e) {
				UserMessages.error("Cannot start plugin <"
						+ plugin.getPluginName() + ">.", e.getMessage());
				logger.log(Level.SEVERE, "", e);
			}
		}
	}

	public void stopPlugins() {
		for (int i = 0; i < pluginClasses.size(); i++) {
			UnicorePlugable plugable = (UnicorePlugable) pluginClasses
					.elementAt(i);
			plugable.stopPlugin();
		}
		knownPlugins.clear();
		pluginClasses.clear();
		taskPluginClasses.clear();
		iconTable.clear();
	}
}