/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.util;

import java.awt.Image;
import java.awt.Toolkit;
import java.net.URL;
import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Abstract base class for Client task plugins.
 * 
 * @author Ralf Ratering
 * @version $Id: TaskPlugable.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */
public abstract class TaskPlugable extends UnicorePlugable {

	/**
	 * Count container instances
	 */
	protected int counter = 0;
	/**
	 * Image for icon representing the plugin task in the client GUI.
	 */
	protected Image iconImage;

	/**
	 * Gets the container class from the plugin
	 * 
	 * @return The containerClass value
	 */
	public final Class getContainerClass() {
		counter--;
		if (counter < 0) {
			counter = 0;
		}
		return getContainerInstance(new JobContainer()).getClass();
	}

	/**
	 * Build a new container for this plugin
	 * 
	 * @param parentContainer
	 *            JobContainer that plugin task belongs to or null if the
	 *            container was attached to the root of the tree.
	 * @return a new instance of the plugin specific task container
	 */
	public abstract ActionContainer getContainerInstance(
			GroupContainer parentContainer);

	/**
	 * Gets the Image representing the plugin task in the client GUI.
	 * 
	 * @return plugin icon image
	 */
	public Image getIconImage() {
		if (iconImage == null) {
			iconImage = loadImage(getIconPath());
		}
		return iconImage;
	}

	/**
	 * Get path to image icon representaing the task in the GUI
	 * 
	 * @return path to icon image as String
	 */
	protected String getIconPath() {
		return null;
	}

	/**
	 * Get name for plugin task task will be presented to the user e.g. in the
	 * JPA menu.
	 * 
	 * @return String displaying task name to the user
	 */
	public abstract String getName();

	/**
	 * Return a new JPA panel inatnace to set the parameter for the plugin task.
	 * This panel will be added to the clients JPA area, when the task is
	 * seleceted in the job preparation tree. Non-Task Plugins should return
	 * null here, because they will not provide a panel for the job preparation
	 * area.
	 * 
	 * @param container
	 *            Description of Parameter
	 * @return a new instance of the plugin specific JPAPanel.
	 */
	public abstract JPAPanel getPanelInstance(ActionContainer container);

	/**
	 * Check, if plugin will provide a certain container class
	 * 
	 * @param cls
	 *            container class to ask for
	 * @return true if container class matches plugin container class
	 */
	public final boolean hasContainerClass(Class cls) {
		counter--;
		if (counter < 0) {
			counter = 0;
		}
		return getContainerInstance(new JobContainer()).getClass().equals(cls);
	}

	/**
	 * Load an image from a given path
	 * 
	 * @param filename
	 *            path to Image as String
	 * @return loaded Image
	 * @deprecated use getIconPath() instead
	 */
	protected Image loadImage(String filename) {
		if (filename == null) {
			return ResourceManager.getImage(ResourceManager.UNKNOWNTASK);
		}
		ClassLoader loader = getClass().getClassLoader();
		URL url = loader.getResource(filename);
		return Toolkit.getDefaultToolkit().createImage(url);
	}
}