/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.container;

import java.io.Serializable;
import java.util.Date;
import java.util.logging.Level;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.AbstractService;
import org.unicore.ajo.ActionGroup;
import org.unicore.upl.ConsignJob;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.upl.UnicoreResponse;
import com.pallas.unicore.connection.Connection;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.security.JobConverter;
import com.pallas.unicore.threadpool.GenericObservable;
import com.pallas.unicore.threadpool.IObservable;
import com.pallas.unicore.threadpool.IObserver;
import com.pallas.unicore.utility.SerialLock;

/**
 * Container used for service tasks in requests.
 * 
 * @author Thomas Kentemich
 * @version $Id: ServiceContainer.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */
public class ServiceContainer extends ActionContainer
		implements
			Serializable,
			IObservable,
			Runnable {

	final static long serialVersionUID = 1130011431582336137L;
	// internal state information e.g. for coloring symbols
	private int containerState = STATE_UNKNOWN;
	// serializable version of a lock
	private SerialLock lock = new SerialLock();
	private Reply reply;
	private AbstractService service = null;
	// task id (for future use)
	private transient int taskId;
	protected GenericObservable theGenericObservable = new GenericObservable();
	private User user = null;
	private Vsite vsite = null;
	private AbstractJob wrapperJob = null;

	public ServiceContainer(AbstractService service, User user, Vsite vsite) {
		super();
		this.service = service;
		this.user = user;
		this.vsite = vsite;
	}

	public void addObserver(IObserver anObserver) {
		this.theGenericObservable.addObserver(anObserver);
	}

	/**
	 * abstract methods
	 */
	public void buildActionGroup() {
		actionGroup = new ActionGroup(getName());
		actionGroup.add(service);
	}

	public void deleteAllObservers() {
		this.theGenericObservable.deleteAllObservers();
	}

	public void deleteObserver(IObserver anObserver) {
		this.theGenericObservable.deleteObserver(anObserver);
	}

	public AbstractService getService() {
		return service;
	}

	/**
	 * Query the state of the container. This gets updated by the threads
	 * actually submitting the job
	 * 
	 * @return The State value
	 */
	public final int getState() {
		synchronized (lock) {
			return containerState;
		}
	}

	public Reply getSubmissionResult() {
		return reply;
	}

	public int getTaskId() {
		return taskId;
	}

	public User getUser() {
		return user;
	}

	public Vsite getVsite() {
		return vsite;
	}

	public AbstractJob getWrapperJob() {
		return wrapperJob;
	}

	public void notifyObservers(Object theObserved, Object changeCode) {
		theGenericObservable.notifyObservers(theObserved, changeCode);
	}

	public void run() {
		try {
			Connection connection = ResourceManager.getConnectionManager()
					.lookupConnection(vsite, ResourceManager.getUser(vsite));
			//now the wrapping AbstractJob
			AbstractJob job = new AbstractJob(ResourceManager
					.getServicePrefix()
					+ "ServiceJob" + ResourceManager.getNextObjectIdentifier(),
					null, null, getVsite(), ResourceManager.getUser(vsite));
			job.add(this.getService());
			// Limit the lifetime of Service AJO's default 30 s
			job.setTerminationTime(new Date(System.currentTimeMillis()
					+ connection.getAjoLifetime()));
			setWrapperJob(job);
			ConsignJob cj = JobConverter.convert(job, false, true);
			reply = null;
			int returnCode = 0;
			UnicoreResponse resp;
			reply = connection.sendMessage(cj, null);
			if (reply instanceof RetrieveOutcomeReply && (returnCode == 0)) {
			}
			else {
				resp = reply.getLastEntry();
				logger.log(Level.WARNING, "Error in ServiceContainer: "
						+ resp.getReturnCode(), resp.getComment());
			}
		}
		catch (Exception e) {
			logger.warning("Could not connect to host: "
					+ getVsite().getAddress() + ".\nReason: " + e.getMessage());
		}
		setState(ActionContainer.STATE_SUBMITTED);
		notifyObservers(this, service);
	}

	public void setService(AbstractService service) {
		this.service = service;
	}

	/**
	 * Set the internal state of the container. This is used to control job
	 * submission
	 *  
	 */
	public final void setState(int state) {
		synchronized (lock) {
			containerState = state;
		}
	}

	public void setTaskId(int taskId) {
		this.taskId = taskId;
	}

	public void setUser(User user) {
		this.user = user;
	}

	public void setVsite(Vsite vsite) {
		this.vsite = vsite;
	}

	public void setWrapperJob(AbstractJob job) {
		this.wrapperJob = job;
	}
}