/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.container;

import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectStreamField;
import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.Enumeration;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;

import org.unicore.AAIdentifier;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractAction;
import org.unicore.ajo.AbstractTask;
import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.Dependency;
import org.unicore.ajo.ExecuteTask;
import org.unicore.resources.Resource;
import org.unicore.sets.ResourceSet;

import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.extensions.ExportFactory;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.extensions.FileImport;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.extensions.FileSystemOperation;
import com.pallas.unicore.extensions.FileTransfer;
import com.pallas.unicore.extensions.ImportFactory;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.resourcemanager.ResourceTray;

/**
 * This is the superclass of all executable tasks. It knows about resources and
 * priorities that should be used for execution.
 * 
 * @author ThomasKentemich
 * @author Ralf Ratering
 * @version $Id: TaskContainer.java,v 1.2 2004/05/27 15:44:53 ratering Exp $
 */
public abstract class TaskContainer extends ActionContainer implements Serializable, Cloneable {

	// field definitions for storing the container in XML format
	private static String[][] persistentXMLFields = {{"SPOOLEDFILES", "spooledFiles"},
			{"IMPORTS", "fileImports"}, {"EXPORTS", "fileExports"}, {"TRANSFERS", "fileTransfers"},
			{"FILEOPERATIONS", "fileSystemOperations"},};
	private static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.container.ResourceStrings");
	final static long serialVersionUID = -6015318006938114109L;
	/**
	 * ActionGroupd for file imports, axecuted before executeGroup
	 */
	protected ActionGroup importGroup;
	/**
	 * Main execution ActionGroup
	 */
	protected ActionGroup executeGroup;
	/**
	 * ActionGroup for file exports, executed after executeGroup
	 */
	protected ActionGroup exportGroup;
	/**
	 * array with filenames to be exported, initially empty
	 */
	protected FileExport[] fileExports = new FileExport[0];
	/**
	 * array with filenames to be imported, initially empty
	 */
	protected FileImport[] fileImports = new FileImport[0];
	/**
	 * array with file operations in uspace, initially empty
	 */
	protected FileSystemOperation[] fileSystemOperations = new FileSystemOperation[0];
	/**
	 * array with filenames to be transferred, initially empty
	 */
	protected FileTransfer fileTransfers[] = new FileTransfer[0];
	/**
	 * vector containing filenames for file exported to nspace
	 */
	protected Vector spooledFiles;

	/**
	 * Constructor
	 * 
	 * @param parentContainer
	 *            parent JobContainer
	 */
	public TaskContainer(GroupContainer parentContainer) {
		super(parentContainer);
		//        parentContainer.addTask(this);
	}

	/**
	 * Add a file export
	 * 
	 * @param fileExport
	 */
	public void addFileExport(FileExport fileExport) {
		FileExport[] newExports = new FileExport[fileExports.length + 1];
		for (int i = 0; i < fileExports.length; i++) {
			newExports[i] = fileExports[i];
			// null out old imports for garbage collector
			fileExports[i] = null;
		}
		newExports[newExports.length - 1] = fileExport;
		fileExports = newExports;
	}

	/**
	 * Add a file import
	 * 
	 * @param fileImport
	 */
	public void addFileImport(FileImport fileImport) {
		FileImport[] newImports = new FileImport[fileImports.length + 1];
		for (int i = 0; i < fileImports.length; i++) {
			newImports[i] = fileImports[i];
			// null out old imports for garbage collector
			fileImports[i] = null;
		}
		newImports[newImports.length - 1] = fileImport;
		fileImports = newImports;
	}

	/**
	 * Build the AJO ActionGroup
	 */
	public void buildActionGroup() {
		actionGroup = new ActionGroup(getName());
		importGroup = new ActionGroup(getName() + "_IMPORT");
		exportGroup = new ActionGroup(getName() + "_EXPORT");
		// needed for dependency later
		AbstractTask transferTask = null;
		ImportFactory importFactory = new ImportFactory(this.getFileImports(), importGroup, this);
		importFactory.build();
		ExportFactory exportFactory = new ExportFactory(getFileExports(), exportGroup, this);
		exportFactory.build();
		buildExecuteGroup();
		// if we have an executeGroup and it has not been added somewhere else
		// (e.g. in UserContainer buildExecuteGroup), add it now.
		if (executeGroup != null && !actionGroup.isIn(executeGroup)) {
			actionGroup.add(executeGroup);
		}
		// only add import group if not empty
		if (importGroup.getActions().hasMoreElements()) {
			// check if importGroup has already been added somewhere else
			// (e.g. in UserContainer buildExecuteGroup)
			if (!actionGroup.isIn(importGroup)) {
				actionGroup.add(importGroup);
			}
			if (executeGroup != null) {
				try {
					actionGroup.addDependency(new Dependency(importGroup, executeGroup));
				}
				catch (ActionGroup.InvalidDependencyException ex) {
					logger.log(Level.SEVERE, "Could not add dependency.", ex);
				}
			}
		}
		// only add import group if not empty
		if (exportGroup.getActions().hasMoreElements()) {
			actionGroup.add(exportGroup);
			if (executeGroup != null) {
				try {
					actionGroup.addDependency(new Dependency(executeGroup, exportGroup));
				}
				catch (ActionGroup.InvalidDependencyException ex) {
					logger.log(Level.SEVERE, "Could not add dependency.", ex);
				}
			}
			else if (importGroup.getActions().hasMoreElements()) {
				try {
					actionGroup.addDependency(new Dependency(importGroup, exportGroup));
				}
				catch (ActionGroup.InvalidDependencyException ex) {
					logger.log(Level.SEVERE, "Could not add dependency.", ex);
				}
			}
		}
		// copy identifier from action group to container to to have a unique id
		// for the njs.
		syncIdentifiers();
	}

	/**
	 * Build an ActionGroup that is put between import and export tasks in the
	 * main ActionGroup. This method should be implemented by sub classes.
	 */
	protected void buildExecuteGroup() {
		// do nothing, executeGroup remains null
	}

	/**
	 * Self check the correctness of the values relevant for this container and
	 * add it to the ErrorSet of the super classes
	 * 
	 * @return complete ErrorSet
	 */
	public ErrorSet checkContents() {
		ErrorSet errorSet = super.checkContents();
		NamedResourceSet siteResources = ResourceManager.getResourceSet(getVsite());
		getResourceSet().addErrorsToSet(errorSet, id, getVsite(), siteResources);
		// check file exports
		checkExports(errorSet);
		// check file imports
		checkImports(errorSet);
		// check file operations
		checkFileOperations(errorSet);
		// check file operations
		checkTransfers(errorSet);
		setErrors(errorSet);
		return errorSet;
	}

	/**
	 * @param errorSet
	 *            Any errors will be added to this ErrorSet
	 */
	protected void checkExports(ErrorSet errorSet) {
		for (int i = 0; i < fileExports.length; i++) {
			if (!fileExports[i].isComplete()) {
				errorSet.add(new UError(id, res.getString("INCOMPLETE_FILE_EXPORT")
						+ fileExports[i] + res.getString("AT_ROW") + (i + 1)));
				// skip other checks if incomplete...
				continue;
			}
			// check for export of a directory to an existing file
			if (fileExports[i].isLocal()
					&& fileExports[i].getSourceName().endsWith(
							fileExports[i].getSourceFileSeparator())) {
				String filename = fileExports[i].getDestinationName();
				File file = new File(filename);
				if (file.isFile()) {
					errorSet.add(new UError(id, res.getString("EXPORT_FROM_DIR_TO_FILE")
							+ fileExports[i].getSourceName()));
				}
			}
			// check, if file storage is available
			checkStorage(errorSet, fileExports[i]);
		}
	}

	/**
	 * Check file operations for errors
	 * 
	 * @param errorSet
	 *            set collecting all errors
	 */
	protected void checkFileOperations(ErrorSet errorSet) {
		for (int i = 0; i < fileSystemOperations.length; i++) {
			if (!fileSystemOperations[i].isComplete()) {
				errorSet.add(new UError(id, res.getString("INCOMPLETE_FILE_OPERATION")
						+ fileSystemOperations[i] + res.getString("AT_ROW") + (i + 1)));
				// skip other checks if incomplete...
				continue;
			}
			// check, if file storage is available
			checkStorage(errorSet, fileSystemOperations[i]);
		}
	}

	/**
	 * Check file imports for errors
	 * 
	 * @param errorSet
	 *            set collecting all errors
	 */
	protected void checkImports(ErrorSet errorSet) {
		Vector uspaceNames = new Vector();
		for (int i = 0; i < fileImports.length; i++) {
			if (!fileImports[i].isComplete()) {
				errorSet.add(new UError(id, res.getString("INCOMPLETE_FILE_IMPORT")
						+ fileImports[i] + res.getString("AT_ROW") + (i + 1)));
				// skip other checks if incomplete...
				continue;
			}
			// test, if local files for Nspace fileImports are available, but
			// ignore wildcards
			if (fileImports[i].isLocal() && !fileImports[i].hasWildcards()) {
				String filename = fileImports[i].getSourceName();
				File file = new File(filename);
				if (!file.exists()) {
					errorSet.add(new UError(id, res.getString("LOCAL_FILE") + filename
							+ res.getString("FOR_IMPORT")));
				}
			}
			// check for duplicate uspace names in imports
			String uspaceName = fileImports[i].getDestinationName();
			if (uspaceNames.contains(uspaceName)) {
				errorSet.add(new UError(id, res.getString("USPACE_NAME") + uspaceName
						+ res.getString("FOR_IMPORT_OF") + fileImports[i].getSourceName()
						+ res.getString("ALREADY_EXISTS")));
			}
			else {
				uspaceNames.add(fileImports[i].getDestinationName());
			}
			// check, if file storage is available
			checkStorage(errorSet, fileImports[i]);
		}
	}

	/**
	 * Check file transfers for errors
	 * 
	 * @param errorSet
	 *            set collecting all errors
	 */
	protected void checkTransfers(ErrorSet errorSet) {
		Vector uspaceNames = new Vector();
		for (int i = 0; i < fileTransfers.length; i++) {
			if (!fileTransfers[i].isComplete()) {
				errorSet.add(new UError(id, res.getString("INCOMPLETE_FILE_TRANSFER")
						+ fileTransfers[i] + res.getString("AT_ROW") + (i + 1)));
				// skip other checks if incomplete...
				continue;
			}
			// check, if file storage is available
			checkStorage(errorSet, fileTransfers[i]);
			//			checkTargetStorage(errorSet, fileTransfers[i]);
		}
	}

	/**
	 * Check file storage objects for errors
	 * 
	 * @param errorSet
	 *            set collecting all errors
	 * @param fileStorage
	 *            object to be checked
	 */
	protected void checkStorage(ErrorSet errorSet, FileStorage fileStorage) {
		if (getVsite() == null) {
			return;
		}
		if (fileStorage.getStorageItem().equals(FileStorage.NSPACE_STRING)) {
			return;
		}
		if (fileStorage.getStorageItem().equals("")) {
			errorSet.add(new UError(id, res.getString("STORAGE") + fileStorage.getStorageItem()
					+ res.getString("NOT_AVAILABLE") + getVsite() + "."));
		}
		NamedResourceSet siteResources = ResourceManager.getResourceSet(getVsite());
		if (!siteResources.contains((Resource) fileStorage.getStorageItem())) {
			errorSet.add(new UError(id, res.getString("STORAGE") + fileStorage.getStorageItem()
					+ res.getString("NOT_AVAILABLE") + getVsite() + "."));
		}
	}

	/**
	 * Clonable interface.
	 * 
	 * @return TaskContainer clone
	 */
	public Object clone() {
		TaskContainer clone = (TaskContainer) super.clone();
		clone.setFileExports((FileExport[]) cloneFileExports());
		clone.setFileImports((FileImport[]) cloneFileImports());
		clone.setFileSystemOperations((FileSystemOperation[]) cloneFileSystemOperations());
		clone.setFileTransfers((FileTransfer[]) cloneFileTransfers());
		return clone;
	}

	/**
	 * @return file export clones
	 */
	private FileExport[] cloneFileExports() {
		FileExport[] clones = new FileExport[fileExports.length];
		for (int i = 0; i < clones.length; i++) {
			clones[i] = (FileExport) fileExports[i].clone();
		}
		return clones;
	}

	/**
	 * @return file import clones
	 */
	private FileImport[] cloneFileImports() {
		FileImport[] clones = new FileImport[fileImports.length];
		for (int i = 0; i < clones.length; i++) {
			clones[i] = (FileImport) fileImports[i].clone();
		}
		return clones;
	}

	/**
	 * @return file system ops clones
	 */
	private FileSystemOperation[] cloneFileSystemOperations() {
		FileSystemOperation[] clones = new FileSystemOperation[fileSystemOperations.length];
		for (int i = 0; i < clones.length; i++) {
			clones[i] = (FileSystemOperation) fileSystemOperations[i].clone();
		}
		return clones;
	}

	/**
	 * @return file transfer clones
	 */
	private FileTransfer[] cloneFileTransfers() {
		FileTransfer[] clones = new FileTransfer[fileTransfers.length];
		for (int i = 0; i < clones.length; i++) {
			clones[i] = (FileTransfer) fileTransfers[i].clone();
		}
		return clones;
	}

	/**
	 * Description of the Method
	 * 
	 * @param group
	 *            Description of the Parameter
	 * @return Description of the Return Value
	 */
	public ExecuteTask findExecuteTask(ActionGroup group) {
		// try to find execute task in current group
		Enumeration actions = group.getActions();
		while (actions.hasMoreElements()) {
			AbstractAction action = (AbstractAction) actions.nextElement();
			if (action instanceof ExecuteTask) {
				return (ExecuteTask) action;
			}
		}
		// couldn't find execute task, search sub groups
		actions = group.getActions();
		while (actions.hasMoreElements()) {
			AbstractAction action = (AbstractAction) actions.nextElement();
			if (action instanceof ActionGroup) {
				return findExecuteTask((ActionGroup) action);
			}
		}
		// no success
		return null;
	}

	/**
	 * method for returning exports that will not appear in the gui can be
	 * overridden in sub-classes
	 * 
	 * @return The additionalExports value
	 */
	public FileExport[] getAdditionalExports() {
		return new FileExport[0];
	}

	/**
	 * method for returning imports not related to data files e.g. executables
	 * in the UserTask; can be overridden in sub-classes
	 * 
	 * @return The AdditionalImports value
	 */
	public FileImport[] getAdditionalImports() {
		return new FileImport[0];
	}

	/**
	 * Gets the executeGroup attribute of the TaskContainer object
	 * 
	 * @return The executeGroup value
	 */
	public ActionGroup getExecuteGroup() {
		return executeGroup;
	}

	/**
	 * Gets the AJO ExecuteTask that is responsible for return code of this
	 * Container. This method will return the first ExcuteTask from the Action
	 * graph. If different ExecuteTasks are required from sub classes, this
	 * method should be overwritten.
	 * 
	 * @return The executeTask value
	 */
	public ExecuteTask getExecuteTask() {
		// if execute group has not been built yet, we do not have an execute
		// task
		// cannot build exeute group here, because it has to be re-build
		// with different ids at submission time
		if (executeGroup == null) {
			return null;
		}
		return findExecuteTask(executeGroup);
	}

	/**
	 * Gets the FileExports
	 * 
	 * @return The FileExports
	 */
	public FileExport[] getFileExports() {
		return fileExports;
	}

	/**
	 * Gets the FileImports
	 * 
	 * @return The FileImports
	 */
	public FileImport[] getFileImports() {
		return fileImports;
	}

	/**
	 * Gets all local FileImports
	 * 
	 * @return local FileImports
	 */
	public FileImport[] getFileImportsFromLocal() {
		// loop over file operations in container and identify imports from
		// nspace
		FileImport[] additionalImports = getAdditionalImports();
		FileImport[] normalImports = getFileImports();
		int normalLength = 0;
		if (normalImports != null) {
			for (int i = 0; i < normalImports.length; i++) {
				if (normalImports[i].isLocal()) {
					normalLength++;
				}
			}
		}
		int additionalLength = 0;
		if (additionalImports != null) {
			additionalLength = additionalImports.length;
		}
		FileImport[] localImports = new FileImport[normalLength + additionalLength];
		if (normalLength > 0) {
			int j = 0;
			for (int i = 0; i < normalImports.length; i++) {
				if (normalImports[i].isLocal()) {
					localImports[j] = normalImports[i];
					j++;
				}
			}
		}
		if (additionalLength > 0) {
			System.arraycopy(additionalImports, 0, localImports, normalLength, additionalLength);
		}
		return localImports;
	}

	/**
	 * Gets the FileSystemOperations
	 * 
	 * @return The FileSystemOperations
	 */
	public FileSystemOperation[] getFileSystemOperations() {
		return fileSystemOperations;
	}

	/**
	 * Gets the FileTransfers
	 * 
	 * @return The FileTransfers
	 */
	public FileTransfer[] getFileTransfers() {
		return fileTransfers;
	}

	/**
	 * Gets the ResourceSet
	 * 
	 * @return The ResourceSet
	 */
	public NamedResourceSet getResourceSet() {
		NamedResourceSet resourceSet = (NamedResourceSet) getParentJobContainer().getResourceTray()
				.get(getIdentifier());
		return resourceSet;
	}

	/**
	 * Get all spooled files
	 * 
	 * @return spooled files
	 */
	public Vector getSpooledFiles() {
		return this.spooledFiles;
	}

	/**
	 * Get the Vsite we are running on from the parent container
	 * 
	 * @return The Vsite
	 */
	public Vsite getVsite() {
		if (getParentJobContainer() == null) {
			return null;
		}
		else {
			return getParentJobContainer().getVsite();
		}
	}

	/**
	 * Do we have an ExecuteTask in our AJO?
	 * 
	 * @return true, if yes
	 */
	public boolean hasExecuteTask() {
		if (executeGroup == null) {
			buildExecuteGroup();
		}
		if (executeGroup == null) {
			return false;
		}
		return true;
	}

	/**
	 * Do we have local exports
	 * 
	 * @return true, if yes
	 */
	public boolean hasNspaceExports() {
		for (int i = 0; i < fileExports.length; i++) {
			FileExport fileExport = (FileExport) fileExports[i];
			if (fileExport.isLocal()) {
				return true;
			}
		}
		// check additional hidden exports as well
		FileExport[] additionalExports = getAdditionalExports();
		for (int i = 0; i < additionalExports.length; i++) {
			FileExport fileExport = (FileExport) additionalExports[i];
			if (fileExport.isLocal()) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Check if container contains any import tasks from nspace. Method is
	 * needed by JobContainer, where import files from nspace are collected and
	 * streamed after the AJO.
	 * 
	 * @return true, if container has nspace imports
	 */
	public boolean hasNspaceImports() {
		if (getAdditionalImports().length > 0) {
			return true;
		}
		FileImport[] fileImports = getFileImports();
		for (int i = 0; i < fileImports.length; i++) {
			if (fileImports[i].isLocal()) {
				return true;
			}
		}
		return false;
	}

	private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
		boolean DEBUG = false;
		// all containers from versions >=4 or UNICOREpro 1.0 should be
		// compatible
		if (getContainerVersion().startsWith("4") || getContainerVersion().startsWith("5")
				|| getContainerVersion().startsWith("1")) {
			stream.defaultReadObject();
			return;
		}
		logger.info("Reading TaskContainer " + id + " from stream (potentially old format)");
		ObjectInputStream.GetField getField = null;
		try {
			getField = stream.readFields();
			ObjectStreamField streamFields[] = getField.getObjectStreamClass().getFields();
			for (int i = 0; i < streamFields.length; i++) {
				String name = streamFields[i].getName();
				Object value = getField.get(name, null);
				Field field = null;
				try {
					field = TaskContainer.class.getDeclaredField(name);
					field.set(this, value);
					logger.fine("Setting field " + field + " to value " + value);
				}
				catch (NoSuchFieldException nsfe) {
					// Class clazz = (streamFields[i]).getType(); // returns
					// useless java.lang.Object class (JVM error?)
					if (name.equals("resourceSet") && (value instanceof ResourceSet)) {
						logger.warning("Field " + name
								+ " not found in TaskContainer.class (OLD JOB FORMAT)");
						AAIdentifier id = getIdentifier();
						String taskName = getName();
						if (taskName == null) {
							taskName = id.getName();
						}
						ResourceSet clone = ((ResourceSet) value).cloneSet();
						NamedResourceSet resources = new NamedResourceSet("RS_" + taskName + "_"
								+ id.getValue() + "[generated]", clone);
						JobContainer job = getParentJobContainer();
						//resources.setVsite(job.getVsite());
						job.getResourceTray().put(id, resources);
					}
					else {
						logger.log(Level.SEVERE, "Field " + name
								+ " not found in TaskContainer.class (OLD JOB FORMAT)", nsfe);
					}
				}
				catch (Exception e) {
					logger.log(Level.SEVERE, "Field " + field.getDeclaringClass() + "."
							+ field.getName(), e);
				}
			}
			logger.info("Finished reading TaskContainer " + id + " from stream");
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "Could not read TaskContainer " + id + " from stream", e);
		}
	}

	/**
	 * Remove a file export
	 * 
	 * @param fileExport
	 */
	public void removeFileExport(FileExport fileExport) {
		Vector newExports = new Vector();
		for (int i = 0; i < fileExports.length; i++) {
			if (!(fileExports[i].equals(fileExport))) {
				newExports.add(fileExports[i]);
				fileExports[i] = null;
			}
		}
		fileExports = new FileExport[newExports.size()];
		for (int i = 0; i < fileExports.length; i++) {
			fileExports[i] = (FileExport) newExports.elementAt(i);
		}
	}

	/**
	 * Remove a file import
	 * 
	 * @param fileImport
	 */
	public void removeFileImport(FileImport fileImport) {
		Vector newImports = new Vector();
		for (int i = 0; i < fileImports.length; i++) {
			if (!(fileImports[i].equals(fileImport))) {
				newImports.add(fileImports[i]);
				fileImports[i] = null;
			}
		}
		fileImports = new FileImport[newImports.size()];
		for (int i = 0; i < fileImports.length; i++) {
			fileImports[i] = (FileImport) newImports.elementAt(i);
		}
	}

	/**
	 * set all file exports
	 * 
	 * @param fileExports
	 */
	public void setFileExports(FileExport[] fileExports) {
		this.fileExports = fileExports;
	}

	/**
	 * set all file imports
	 * 
	 * @param fileImports
	 */
	public void setFileImports(FileImport[] fileImports) {
		this.fileImports = fileImports;
	}

	/**
	 * Setall file operations
	 * 
	 * @param fileSystemOperations
	 */
	public void setFileSystemOperations(FileSystemOperation[] fileSystemOperations) {
		this.fileSystemOperations = fileSystemOperations;
	}

	/**
	 * Sets all file transfers
	 * 
	 * @param fileTransfers
	 */
	public void setFileTransfers(FileTransfer[] fileTransfers) {
		this.fileTransfers = fileTransfers;
	}

	/**
	 * Sets the SpooledFiles
	 * 
	 * @param spooledFiles
	 */
	public void setSpooledFiles(Vector spooledFiles) {
		this.spooledFiles = spooledFiles;
	}

	/**
	 * Copy identifier from action group to container to have a unique id for
	 * the njs. Here also the resource tray of the job container is updated,
	 * because it uses the id as hash key.
	 */
	protected void syncIdentifiers() {
		AAIdentifier newId = actionGroup.getId();
		AAIdentifier oldId = getIdentifier();
		// update resource tray
		ResourceTray tray = getParentJobContainer().getResourceTray();
		NamedResourceSet resource = (NamedResourceSet) tray.get(oldId);
		// resource tray will return site default resource instead of null, so
		// check this!
		if (resource != null && !resource.isSiteDefault()) {
			tray.put(newId, resource);
			tray.remove(oldId);
		}
		// update id
		setIdentifier(newId);
	}
}