/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.container.errorspec;

import java.util.Vector;

/**
 * A type-safe container class for <tt>UError</tt>s.
 * 
 * @author Michel Drescher
 * @version $Id: ErrorSet.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class ErrorSet {

	protected Vector set;

	/**
	 * Creates a new ErrorSet
	 */
	public ErrorSet() {
		set = new Vector();
	}

	/**
	 * Creates a new ErrorSet
	 * 
	 * @param first
	 *            an initial <code>UError</code> entry
	 */
	public ErrorSet(UError first) {
		set = new Vector();
		add(first);
	}

	public void add(ErrorSet errorSet) {
		ErrorSetEnumeration enum = errorSet.elements();
		while (enum.hasMoreElements()) {
			add(enum.nextElement());
		}
	}

	/**
	 * Add a <tt>UError</tt> to the set.
	 * 
	 * @param thing
	 *            The <tt>UError</tt> to be added.
	 */
	public void add(UError thing) {
		set.addElement(thing);
	}

	public ErrorSet cloneSet() {
		// Do not use clone as we do not return an Object
		// so compiler will complain (Object sits poorly with rest)
		ErrorSet result = new ErrorSet();
		result.set = (Vector) set.clone();
		return result;
	}

	public boolean containsError() {
		return set.size() > 0;
	}

	/**
	 * Create and return an enumeration over the elements in the
	 * <tt>ErrorSet</tt>
	 * 
	 * @return The enumeration.
	 */
	public ErrorSetEnumeration elements() {
		return new ErrorSetEnumeration(this);
	}

	/**
	 * Returns the element at position <code>pos</code>.
	 * 
	 * @param pos
	 *            the position (0 to this.size()-1) in this set
	 * @return the <code>UError</code> at the specified position.
	 */
	public UError getAt(int pos) {
		return (UError) set.elementAt(pos);
	}

	/**
	 * Returns the amount of <code>UErrors</code> contained in this set
	 * 
	 * @return the amount of elements as <code>int</code>
	 */
	public int getSize() {
		return set.size();
	}

	/**
	 * Test if a <tt>UError</tt> is in the set
	 * 
	 * @param thing
	 *            the UError instance to be tested
	 * @return Return true iff the <tt>UError</tt> is in the set
	 */
	public boolean isIn(UError thing) {
		return set.contains(thing);
	}

	/**
	 * Remove the <tt>UError</tt> from the set.
	 * 
	 * @param thing
	 *            The <tt>UError</tt> to be removed.
	 */
	public void remove(UError thing) {
		set.removeElement(thing);
	}

	/**
	 * Replaces <code>oldError</code> with <code>newError</code>.
	 * 
	 * @param oldError
	 *            The UError to be replaced.
	 * @param newError
	 *            The UError instance to be inserted.
	 * @return True, If the operation succeedes, false, if oldError is not in
	 *         this set, or if oldError or newError are <code>null</code>.
	 */
	public boolean replace(UError oldError, UError newError) {
		if ((oldError == null) || (newError == null)) {
			return false;
		}
		int i = set.indexOf(oldError);
		if (i < 0) {
			return false;
		}
		try {
			set.removeElementAt(i);
			set.insertElementAt(newError, i);
		}
		catch (ArrayIndexOutOfBoundsException aioob) {
			return false;
		}
		return true;
	}

	/**
	 * Build String out of set elements separated by newlines
	 * 
	 * @return new Error String
	 */
	public String toString() {
		String message = "";
		for (int i = 0; i < getSize(); i++) {
			message += getAt(i) + "\n";
		}
		return message;
	}
}