/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.extensions;

import java.awt.datatransfer.DataFlavor;
import java.io.File;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.unicore.ajo.Portfolio;
import org.unicore.ajo.PutPortfolio;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.requests.GetSpooledFiles;

/**
 * Class representing a file export to a specified location from uspace.
 * 
 * @author Ralf Ratering
 * @version $Id: FileExport.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class FileExport extends FileStorage {

	/**
	 * In case the overwrite flag of the FileExport is set, the user can choose
	 * the target files which he wants to be overwritten.
	 */
	public final static int CHOOSE_OVERWRITTEN_FILES = 1;
	private static DataFlavor flavor;
	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.extensions");
	public final static int NOT_CHOOSE_OVERWRITTEN_FILES = 2;
	final static long serialVersionUID = -8215834295484608115L;

	public static DataFlavor getFlavor() {
		if (flavor == null) {
			try {
				flavor = new DataFlavor(Class
						.forName("com.pallas.unicore.extensions.FileExport"),
						"FileExport");
			}
			catch (ClassNotFoundException e) {
				logger
						.log(
								Level.SEVERE,
								"Error getting DataFlavore for com.pallas.unicore.extensions.FileExport",
								e);
			}
		}
		return flavor;
	}
	/**
	 * Reference to the originating container
	 */
	private ActionContainer container;
	private transient String resolvedPath;
	/**
	 * declare a portfolio containing all files exported to nspace
	 */
	private Portfolio spoolPortfolio;
	/**
	 * the reference to the SpoolTask for Nspace exports
	 */
	private PutPortfolio spoolTask;
	private Vector targetFiles = null;
	private int targetFilesFlag = NOT_CHOOSE_OVERWRITTEN_FILES;
	private transient GetSpooledFiles unspoolTask;

	/**
	 * Constructor
	 * 
	 * @param storageItem
	 *            object describing destination location
	 * @param sourceName
	 *            name of source file
	 * @param destinationName
	 *            name of destination file
	 * @param container
	 */
	public FileExport(ActionContainer container, Object storageItem,
			String sourceName, String destinationName) {
		super(storageItem, sourceName, destinationName);
		this.container = container;
	}

	/**
	 * Constructor
	 * 
	 * @param storageItem
	 *            object describing destination location
	 * @param sourceName
	 *            name of source file
	 * @param destinationName
	 *            name of destination file
	 * @param overwrite
	 *            overwrite policy
	 * @param container
	 */
	public FileExport(ActionContainer container, Object storageItem,
			String sourceName, String destinationName, boolean overwrite) {
		this(container, storageItem, sourceName, destinationName);
		this.overwrite = overwrite;
	}

	/**
	 * Constructor
	 * 
	 * @param storageItem
	 *            object describing destination location
	 * @param sourceName
	 *            name of source file
	 * @param destinationName
	 *            name of destination file
	 * @param overwrite
	 *            overwrite policy
	 * @param isAscii
	 *            binary or ASCII format
	 * @param container
	 */
	public FileExport(ActionContainer container, Object storageItem,
			String sourceName, String destinationName, boolean overwrite,
			boolean isAscii) {
		this(container, storageItem, sourceName, destinationName);
		this.overwrite = overwrite;
		this.isAscii = isAscii;
	}

	/**
	 * Gets the ActionContainer originating this Export
	 * 
	 * @return The ActionContainer
	 */
	public ActionContainer getContainer() {
		return this.container;
	}

	/**
	 * Get separator of destination file system (Local)
	 * 
	 * @return separator string
	 */
	public String getDestinationFileSeparator() {
		if (this.isLocal()) {
			return File.separator;
		}
		// remote file systems are assumed to be UNIX
		else {
			return "/";
		}
	}

	/**
	 * Get separator of source file system (always UNIX)
	 * 
	 * @return separator string
	 */
	public String getSourceFileSeparator() {
		return "/";
	}

	public Portfolio getSpoolPortfolio() {
		return this.spoolPortfolio;
	}

	public PutPortfolio getSpoolTask() {
		return this.spoolTask;
	}

	public DataFlavor getSubClassFlavor() {
		return getFlavor();
	}

	public Vector getTargetFiles() {
		if (targetFiles == null) {
			return null;
		}
		return this.targetFiles;
	}

	public int getTargetFilesFlag() {
		return this.targetFilesFlag;
	}

	public GetSpooledFiles getUnSpool() {
		return unspoolTask;
	}

	public void resetTargetFilesFlag() {
		this.targetFilesFlag = NOT_CHOOSE_OVERWRITTEN_FILES;
	}

	/**
	 * Sets the ActionContainer originating this Export
	 * 
	 * @param container
	 *            The new container value
	 */
	public void setContainer(ActionContainer container) {
		this.container = container;
	}

	public void setSpoolPortfolio(Portfolio spoolPortfolio) {
		this.spoolPortfolio = spoolPortfolio;
	}

	public void setSpoolTask(PutPortfolio spoolTask) {
		this.spoolTask = spoolTask;
	}

	public void setTargetFiles(Vector targetFiles) {
		this.targetFiles = targetFiles;
	}

	public void setTargetFilesFlag(int flag) {
		this.targetFilesFlag = flag;
	}

	public void setUnSpool(GetSpooledFiles gsf) {
		unspoolTask = gsf;
	}

	/**
	 * @return String describing operation
	 */
	public String toString() {
		return sourceName + " -> " + storageItem + ": " + destinationName;
	}
}