/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.extensions;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.File;
import java.io.Serializable;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.unicore.ajo.AbstractAction;

/**
 * Abstract class representing a file operation that takes a source file as
 * input and may produce a destination file as output
 * 
 * @author Ralf Ratering
 * @version $Id: FileOperation.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public abstract class FileOperation
		implements
			Serializable,
			Cloneable,
			Transferable {

	protected static Logger logger = Logger
			.getLogger("com.pallas.unicore.extensions");
	final static long serialVersionUID = -6307530181526319154L;
	/**
	 * Keep track of ajo tasks that are represented by this object
	 */
	protected AbstractAction abstractAction;
	/**
	 * each operation has a source and eventually a destination
	 */
	protected String sourceName, destinationName;
	/**
	 * byte size of source file
	 */
	protected long totalBytes, transferredBytes;
	/**
	 * flag if file operation is currently executed
	 */
	protected boolean transferring = false;

	/**
	 * Constructor for operations with source and destination
	 * 
	 * @param sourceName
	 *            name of source file
	 * @param destinationName
	 *            name of destination file
	 */
	public FileOperation(String sourceName, String destinationName) {
		this.sourceName = sourceName;
		this.destinationName = destinationName;
	}

	public Object clone() {
		try {
			FileOperation clone = (FileOperation) super.clone();
			return clone;
		}
		catch (CloneNotSupportedException e) {
			logger.log(Level.SEVERE, "Cloning " + this + " failed.", e);
			return null;
		}
	}

	public boolean equals(Object object) {
		if (object == null) {
			return false;
		}
		FileOperation op = (FileOperation) object;
		// this should catch the null-case
		if (sourceName == op.getSourceName()
				&& destinationName == op.getDestinationName()) {
			return true;
		}
		return sourceName.equals(op.getSourceName())
				&& destinationName.equals(op.getDestinationName());
	}

	public AbstractAction getAbstractAction() {
		return abstractAction;
	}

	/**
	 * Gets the base name in the destination name
	 * 
	 * @return The base name value of the destination name
	 */
	public String getDestinationBasename() {
		String basename;
		// Use File class for local files which handles Windows and UNIX file
		// separators transparently (e.g. in case of job defined on UNIX Client
		// which is now submitted on Windows Client)
		if (this.isLocal()) {
			File file = new File(destinationName);
			basename = file.getName();
		}
		else {
			if (destinationName.endsWith(getDestinationFileSeparator())) {
				basename = destinationName.substring(0, destinationName
						.length() - 1);
			}
			else {
				basename = destinationName;
			}
			basename = basename.substring(basename
					.lastIndexOf(getDestinationFileSeparator()) + 1);
		}
		return basename;
	}

	/**
	 * Get separator of destination file system
	 * 
	 * @return separator string
	 */
	public abstract String getDestinationFileSeparator();

	/**
	 * @return name of destination file
	 */
	public String getDestinationName() {
		return destinationName;
	}

	/**
	 * Gets the base name in the source name
	 * 
	 * @return The base name value of the source name
	 */
	public String getSourceBasename() {
		String basename;
		// Use File class for local files which handles Windows and UNIX file
		// separators transparently (e.g. in case of job defined on UNIX Client
		// which is now submitted on Windows Client)
		if (this.isLocal()) {
			File file = new File(sourceName);
			basename = file.getName();
		}
		else {
			if (sourceName.endsWith(getSourceFileSeparator())) {
				basename = sourceName.substring(0, sourceName.length() - 1);
			}
			else {
				basename = sourceName;
			}
			basename = basename.substring(basename
					.lastIndexOf(getSourceFileSeparator()) + 1);
		}
		return basename;
	}

	/**
	 * Get separator of source file system
	 * 
	 * @return separator string
	 */
	public abstract String getSourceFileSeparator();

	/**
	 * @return name of source file
	 */
	public String getSourceName() {
		return sourceName;
	}

	public abstract DataFlavor getSubClassFlavor();

	public long getTotalBytes() {
		return this.totalBytes;
	}

	/**
	 * Transferable interface.
	 *  
	 */
	public synchronized Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException {
		if (getSubClassFlavor().equals(flavor)) {
			return this;
		}
		else {
			throw new UnsupportedFlavorException(flavor);
		}
	}

	/**
	 * Transferable interface.
	 * 
	 *  
	 */
	public synchronized DataFlavor[] getTransferDataFlavors() {
		DataFlavor[] flavors = {getSubClassFlavor()};
		return flavors;
	}

	public long getTransferredBytes() {
		return this.transferredBytes;
	}

	/**
	 * Alwayas override hashCode, when overriding equals.
	 * 
	 * @return Description of the Return Value
	 */
	public int hashCode() {
		int result = 17;
		result = 37 * result + sourceName.hashCode();
		result = 37 * result + destinationName.hashCode();
		return result;
	}

	public boolean hasWildcards() {
		return ((sourceName.indexOf("*") >= 0)
				|| (sourceName.indexOf("?") >= 0) || (sourceName.indexOf("[") >= 0));
	}

	/**
	 * Check if source and destination names have been set
	 * 
	 * @return true if operation is complete
	 */
	public boolean isComplete() {
		// removed [ && !destinationName.equals("")) ];
		// destinationName.equals("") specifies the Storage, i.e. Uspace, Home
		// which defines a path already
		return (sourceName != null && !sourceName.equals("") && destinationName != null);
	}

	/**
	 * Test if storageItem is a local one
	 * 
	 * @return true if storage item is local
	 */
	public abstract boolean isLocal();

	/**
	 * Transferable interface.
	 * 
	 *  
	 */
	public boolean isDataFlavorSupported(DataFlavor flavor) {
		return (getSubClassFlavor().equals(flavor));
	}

	public boolean isTransferring() {
		return transferring;
	}

	public void setAbstractAction(AbstractAction abstractAction) {
		this.abstractAction = abstractAction;
	}

	public void setDestinationName(String destinationName) {
		this.destinationName = destinationName;
	}

	public void setSourceName(String sourceName) {
		this.sourceName = sourceName;
	}

	public void setTotalBytes(long totalBytes) {
		this.totalBytes = totalBytes;
	}

	public void setTransferredBytes(long transferredBytes) {
		this.transferredBytes = transferredBytes;
	}

	public void setTransferring(boolean transferring) {
		this.transferring = transferring;
	}

	/**
	 * Abstract method describing this operation as String e.g. for gui elements
	 * 
	 * @return description as String
	 */
	public abstract String toString();
}