/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.extensions;

import org.unicore.resources.Storage;

/**
 * Abstract class representing a file transfer containing a specified storage
 * location
 * 
 * @author Ralf Ratering
 * @version $Id: FileStorage.java,v 1.1 2004/05/25 14:58:44 rmenday Exp $
 */
public abstract class FileStorage extends FileOperation {

	/**
	 * Local constant
	 */
	public final static String NSPACE_STRING = "Local";
	final static long serialVersionUID = 23669528063295652L;
	/**
	 * ASCII or binary mode
	 */
	protected boolean isAscii = false;
	/**
	 * overwrite policy
	 */
	protected boolean overwrite = true;
	/**
	 * the storage item may be a String or a Storage resource
	 */
	protected Object storageItem;
	/**
	 * the other storage item will be used for - Transfer targets - Import
	 * targets - Export sources
	 */
	protected Storage otherStorage = null;

	/**
	 * Constructor including storage item
	 * 
	 * @param storageItem
	 *            defining storage location
	 * @param sourceName
	 *            name of source file
	 * @param destinationName
	 *            name of destination file
	 */
	public FileStorage(Object storageItem, String sourceName,
			String destinationName) {
		super(sourceName, destinationName);
		this.storageItem = storageItem;
	}

	/**
	 * Constructor including storage item
	 * 
	 * @param storageItem
	 *            defining storage location
	 * @param sourceName
	 *            name of source file
	 * @param otherStorage
	 *            other storage location
	 * @param destinationName
	 *            name of destination file
	 */
	public FileStorage(Object storageItem, String sourceName,
			Storage otherStorage, String destinationName) {
		super(sourceName, destinationName);
		this.storageItem = storageItem;
		this.otherStorage = otherStorage;
	}

	public boolean equals(Object object) {
		FileStorage storage = (FileStorage) object;
		boolean result = super.equals(storage);
		if (storageItem == null) {
			result = result && (storageItem == storage.getStorageItem());
		}
		else {
			result = result && storageItem.equals(storage.getStorageItem());
		}
		if (otherStorage == null) {
			result = result && (otherStorage == storage.getOtherStorage());
		}
		else {
			result = result && otherStorage.equals(storage.getOtherStorage());
		}
		return result;
	}

	public boolean getFileMode() {
		return isAscii;
	}

	public boolean getOverwritePolicy() {
		return overwrite;
	}

	public Object getStorageItem() {
		return storageItem;
	}

	public Storage getOtherStorage() {
		return otherStorage;
	}

	public int hashCode() {
		int result = 17;
		result = 37 * result + super.hashCode();
		result = 37 * result + storageItem.hashCode();
		if (otherStorage != null) {
			result = 37 * result + otherStorage.hashCode();
		}
		return result;
	}

	/**
	 * Check if storage has been set and super class is complete
	 * 
	 * @return true if operation is complete
	 */
	public boolean isComplete() {
		boolean result = super.isComplete() && storageItem != null;
		return result;
	}

	/**
	 * Test if storageItem is a local one
	 * 
	 * @return true if storage item is local
	 */
	public boolean isLocal() {
		boolean ret = storageItem.equals(NSPACE_STRING);
		return ret;
	}

	public void setFileMode(boolean isAscii) {
		this.isAscii = isAscii;
	}

	public void setOverwritePolicy(boolean overwrite) {
		this.overwrite = overwrite;
	}

	public void setStorageItem(Object storageItem) {
		this.storageItem = storageItem;
	}

	public void setOtherStorage(Storage otherStorage) {
		this.otherStorage = otherStorage;
	}
}