/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.extensions;

import java.awt.datatransfer.DataFlavor;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *  Class representing a file operation (copy, rename, delete, make dir)
 *
 *@author     Ralf Ratering
 *@version    $Id: FileSystemOperation.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class FileSystemOperation extends FileStorage {
	
	public final static String COPY = "copy to",
		RENAME = "rename to",
		MAKEDIR = "make directory",
		DELETE = "delete",
		CHMOD = "change permissions";
	private static DataFlavor flavor;
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.extensions");
	final static long serialVersionUID = 4202108145757606227L;

	
	public static DataFlavor getFlavor() {
		if (flavor == null) {
			try {
				flavor =
					new DataFlavor(
						Class.forName(
							"com.pallas.unicore.extensions.FileSystemOperation"),
						"FileSystemOperation");
			} catch (ClassNotFoundException e) {
				logger.log(
					Level.SEVERE,
					"Error getting DataFlavor for com.pallas.unicore.extensions.FileSystemOperatins",
					e);
			}

		}
		return flavor;
	}

	
	public static String[] getOperations() {
		String[] operations = { COPY, DELETE, RENAME, MAKEDIR, CHMOD };
		return operations;
	}

	/**
	 *  String describing uspace operation
	 */
	protected String operation;

	/**
	 *  Constructor including uspace operation
	 *
	 *@param  sourceName       name of source file
	 *@param  operation        String describing uspace operation
	 *@param  destinationName  name of destination file, ignored for delete or
	 *      make dirs
	 *@param  storageItem     
	 */
	public FileSystemOperation(
		String sourceName,
		String operation,
		String destinationName,
		Object storageItem) {
		super(storageItem, sourceName, destinationName);
		this.operation = operation;
	}

	/**
	 *  Get separator of destination file system (always UNIX)
	 *
	 *@return    separator string
	 */
	public String getDestinationFileSeparator() {
		return "/";
	}

	/**
	 *@return    operation as String
	 */
	public String getOperation() {
		return operation;
	}

	/**
	 *  Get separator of source file system (always UNIX)
	 *
	 *@return    separator string
	 */
	public String getSourceFileSeparator() {
		return "/";
	}

	
	public DataFlavor getSubClassFlavor() {
		return getFlavor();
	}

	
	public boolean isComplete() {
		// super checks already for empty source, null destination and storage item
		boolean complete = super.isComplete();

		if (operation.equals(RENAME)) {
			// no empty destination allowed
			complete &= !destinationName.equals("");
		} else if (operation.equals(COPY)) {
			complete &= !destinationName.equals("");
		} else if (operation.equals(DELETE)) {
			complete &= destinationName.equals("");
		} else if (operation.equals(MAKEDIR)) {
			complete &= destinationName.equals("");
		} else if (!operation.equals(CHMOD)) {
			complete = false;
		}

		return complete;
	}

	/**
	 * @param operation
	 *            as String
	 */
	public void setOperation(String operation) {
		this.operation = operation;
	}

	/**
	 *@return    String describing operation
	 */
	public String toString() {
		if (operation.equals(DELETE) || operation.equals(MAKEDIR)) {
			return storageItem + ": " + sourceName + " " + operation;
		} else {
			return storageItem
				+ ": "
				+ sourceName
				+ " "
				+ operation
				+ " "
				+ destinationName;
		}
	}

}
