/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.extensions;

import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;
import org.unicore.AJOIdentifier;
import org.unicore.ajo.AbstractAction;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.CopyFile;
import org.unicore.ajo.CreateDirectory;
import org.unicore.ajo.DeletePortfolio;
import org.unicore.ajo.Dependency;
import org.unicore.ajo.FileCheck;
import org.unicore.ajo.If;
import org.unicore.ajo.MakePortfolio;
import org.unicore.ajo.PutPortfolio;
import org.unicore.idiomatic.Transfer;
import org.unicore.idiomatic.TransferFile;
import org.unicore.outcome.FileCheck_Outcome;
import org.unicore.resources.AlternativeUspace;
import org.unicore.resources.PathedStorage;
import org.unicore.resources.USpace;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Class builds AJO ActionGroup from an input Array of FileTransfers
 * 
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @author Klaus-Dieter Oertel
 * @version $Id: TransferFactory.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class TransferFactory extends FileOperationFactory {

	private final static String createTransferDirectories = System
			.getProperty("TransferFactory.createTransferDirectories");

	public TransferFactory(FileTransfer[] fileOpsArray,
			ActionGroup actionGroup, TaskContainer container) {
		super(fileOpsArray, actionGroup, container);
	}

	public void build() {
		FileTransfer[] transfers = container.getFileTransfers();
		if (transfers.length == 0) {
			// no transfers specified
			return;
		}
		// collect different Destinations and store them with
		// corresponding transfers in Hashtable
		Hashtable transferTable = new Hashtable();
		/*
		 * for (int i = 0; i < transfers.length; i++) { JobContainer
		 * destinationJob = transfers[i].getDestinationJob(); if
		 * (!transferTable.containsKey(destinationJob)) { Vector jobTransfers =
		 * new Vector(); jobTransfers.add(transfers[i]);
		 * transferTable.put(destinationJob, jobTransfers); } else { Vector
		 * jobTransfers = (Vector)transferTable.get(destinationJob);
		 * jobTransfers.add(transfers[i]); } } Enumeration jobEnum =
		 * transferTable.keys(); while (jobEnum.hasMoreElements()) {
		 * JobContainer job = (JobContainer)jobEnum.nextElement();
		 * buildTransferTask((Vector)transferTable.get(job), job); }
		 */
		for (int i = 0; i < transfers.length; i++) {
			JobContainer destinationJob = transfers[i].getDestinationJob();
			Vector jobTransfers = new Vector();
			jobTransfers.add(transfers[i]);
			FileTransfer fileTransfer = transfers[i];
			/*
			 * Requirements for transfers: - Support all combinations of
			 * storages: U-U, X-U, U-X, X-X - Enable transfers of directories
			 * and use of wildcards - Renaming should be possible
			 * 
			 * The easiest solution would be to use TransferFile to meet all
			 * requirements, however, there are some drawbacks: - For identical
			 * Vsites, the current TransferFile implementation by the NJS is a
			 * performance loss, because internally a Transfer is performed,
			 * i.e. streaming via UPL. Instead, PutPortfolio should be used for
			 * identical Vsites - The current TransferFile implementation (AJO
			 * 4.0.0 plus NJS 4.0.3) has to be manipulated to support the
			 * transfer of directories (cf. buildTransferFile) - The current
			 * TransferFile implementation cannot be used for directories if the
			 * target is a Uspace (the NJS does not execute the
			 * CopyFile-StreamFile{CopyFile} sequence!) - The current
			 * TransferFile implementation for AFT has a bug if the target is a
			 * Uspace - The current TransferFile implementation for AFT does not
			 * support directories
			 * 
			 * Implemented solution: For identical Vsites always a
			 * CopyFile-PutPortfolio-CopyFile sequence is used for performance
			 * reasons, where the CopyFiles are only executed if they are needed
			 * (e.g. in case of a target Xspace or renaming). On different
			 * Vsites, TransferFile is used if the target storage is an Xspace.
			 * TransferFile is also used if the target is an Uspace and renaming
			 * is done. If the target is the Uspace and no renaming is
			 * performed, a CopyFile-Transfer-CopyFile sequence is executed. As
			 * for identical Vsites, the CopyFiles are only executed if they are
			 * needed (e.g. in case of a target Xspace). The implementation will
			 * be changed for an improved version of TransferFile in the future.
			 * Some details: - Renaming is assumed if the target is neither the
			 * emtpy string "", nor the current directory "." or "./". Therefore
			 * the user should not repeat the source name in the target field of
			 * the panel. - buildCopyTransferCopy: * Performs a CopyFile from
			 * the source storage into a hidden directory in the Uspace,
			 * followed by a Transfer or PutPortfolio (depending on whether
			 * Vsites are different or identical) of this hidden directory or
			 * its included files, followed by a CopyFile of the files from the
			 * hidden directory (now in the target Uspace) to the target storage
			 * (this CopyFile only works for identical Vsites!) * The CopyFiles
			 * are only executed if required: The source and target CopyFile is
			 * needed if either the target storage is an Xspace or renaming
			 * occurs. The source CopyFile is needed if the source storage is an
			 * Xspace * If the CopyFiles are not needed buildCopyTransferCopy
			 * produces the same AJO as the previous buildTransfer method -
			 * buildTransferFile: * Performs a TransferFile in case of a source
			 * file * If the source is a directory
			 */
			boolean sourceXspace = !(fileTransfer.getStorageItem() instanceof USpace);
			boolean targetXspace = !(fileTransfer.getOtherStorage() instanceof USpace);
			boolean renaming = !fileTransfer.getDestinationName().equals("")
					&& !fileTransfer.getDestinationName().equals(".")
					&& !fileTransfer.getDestinationName().equals("./");
			// identical Vsites
			if (destinationJob.getVsite().equals(
					container.getParentJobContainer().getVsite())) {
				if (fileTransfer.getStorageItem() instanceof USpace
						&& fileTransfer.getOtherStorage() instanceof USpace
						&& !renaming) {
					//					buildTransfer(jobTransfers, destinationJob);
					//					equivalent to buildTransfer
					buildCopyTransferCopy(jobTransfers, destinationJob);
				}
				else {
					buildCopyTransferCopy(jobTransfers, destinationJob);
				}
			}
			else {
				if (fileTransfer.getStorageItem() instanceof USpace
						&& fileTransfer.getOtherStorage() instanceof USpace
						&& !renaming) {
					//					buildTransfer(jobTransfers, destinationJob);
					//					equivalent to buildTransfer
					buildCopyTransferCopy(jobTransfers, destinationJob);
				}
				else if (fileTransfer.getOtherStorage() instanceof USpace) {
					if (!renaming) {
						buildCopyTransferCopy(jobTransfers, destinationJob);
					}
					else {
						buildTransferFile(jobTransfers, destinationJob);
					}
				}
				else {
					buildTransferFile(jobTransfers, destinationJob);
				}
			}
		}
	}

	/**
	 * Build an import task for one specific storage location in Xspace
	 */
	private void buildTransfer(Vector transfers, JobContainer destination) {
		AbstractAction theTransfer = null;
		// build the source portfolio
		MakePortfolio sourcePortfolio = new MakePortfolio(container.getName()
				+ "_MAKEPORTFOLIO_" + ResourceManager.getNextObjectIdentifier());
		if (destination.getVsite().equals(
				container.getParentJobContainer().getVsite())) {
			theTransfer = new PutPortfolio(container.getName() + "_PP_TRANS_"
					+ ResourceManager.getNextObjectIdentifier());
		}
		else {
			theTransfer = new Transfer(container.getName() + "_TT_TRANS_"
					+ ResourceManager.getNextObjectIdentifier());
			((AbstractJob) theTransfer).setEndorser(destination.getUser());
		}
		// build filename array as input for portfolio
		String[] filenames = new String[transfers.size()];
		ActionGroup dummyGroup = new ActionGroup(
				createActionName("TRANSFER_DUMMY_GROUP"));
		for (int i = 0; i < transfers.size(); i++) {
			FileTransfer fileTransfer = (FileTransfer) transfers.elementAt(i);
			filenames[i] = fileTransfer.getSourceName();
			// remember complete action group in transfer objects
			fileTransfer.setAbstractAction(dummyGroup);
		}
		sourcePortfolio.setFiles(filenames);
		// allow only the same name currently
		MakePortfolio targetPortfolio = sourcePortfolio;
		if (destination.getVsite().equals(
				container.getParentJobContainer().getVsite())) {
			PutPortfolio put = (PutPortfolio) theTransfer;
			put.setSource(sourcePortfolio.getPortfolio());
			put.setTarget(targetPortfolio.getPortfolio().getId());
			AlternativeUspace uspace = new AlternativeUspace();
			uspace.setUspace((AJOIdentifier) destination.getIdentifier());
			put.addResource(uspace);
		}
		else {
			Transfer transferTask = (Transfer) theTransfer;
			transferTask.setSourcePortfolio(sourcePortfolio.getPortfolio());
			transferTask.setTargetPortfolio(targetPortfolio.getPortfolio()
					.getId());
			transferTask.setTargetUspace((AJOIdentifier) destination
					.getIdentifier());
			transferTask.setTargetVsite(destination.getVsite());
		}
		// try to delete previous portfolio in case of loops,
		// because NJS will complain about already existing
		// portfolio else
		DeletePortfolio deletePortfolio = new DeletePortfolio(
				"_DELETEPORTFOLIO_" + ResourceManager.getNextObjectIdentifier());
		deletePortfolio.setPortfolio(sourcePortfolio.getPortfolio().getId());
		// set ignore failure, because portfolio will not exist yet,
		// if task is not in a loop and also in the first iteration
		deletePortfolio.setPropagateSuccessfulIfNotSuccessful(true);
		dummyGroup.add(sourcePortfolio);
		dummyGroup.add(theTransfer);
		dummyGroup.add(deletePortfolio);
		try {
			dummyGroup.addDependency(new Dependency(deletePortfolio,
					sourcePortfolio));
			dummyGroup.addDependency(new Dependency(sourcePortfolio,
					theTransfer));
		}
		catch (ActionGroup.InvalidDependencyException ex) {
			logger.log(Level.SEVERE, "Could not add dependency.", ex);
		}
		actionGroup.add(dummyGroup);
	}

	/**
	 * Build an import task for one specific storage location in Xspace
	 */
	private void buildCopyTransferCopy(Vector transfers,
			JobContainer destination) {
		AbstractAction theTransfer = null;
		for (int i = 0; i < transfers.size(); i++) {
			ActionGroup dummyGroup = new ActionGroup(
					createActionName("COPY.TRANSFER.COPY_DUMMY_GROUP"));
			FileTransfer fileTransfer = (FileTransfer) transfers.elementAt(i);
			// remember complete action group in transfer objects
			fileTransfer.setAbstractAction(dummyGroup);
			// build the source portfolio
			MakePortfolio sourcePortfolio = new MakePortfolio(container
					.getName()
					+ "_MAKEPORTFOLIO_"
					+ ResourceManager.getNextObjectIdentifier());
			// allow only the same name currently
			MakePortfolio targetPortfolio = sourcePortfolio;
			if (destination.getVsite().equals(
					container.getParentJobContainer().getVsite())) {
				theTransfer = new PutPortfolio(
						createActionName("PP_COPY.TRANS.COPY"));
				PutPortfolio put = (PutPortfolio) theTransfer;
				put.setSource(sourcePortfolio.getPortfolio());
				put.setTarget(targetPortfolio.getPortfolio().getId());
				AlternativeUspace uspace = new AlternativeUspace();
				uspace.setUspace((AJOIdentifier) destination.getIdentifier());
				put.addResource(uspace);
			}
			else {
				theTransfer = new Transfer(
						createActionName("TT_COPY.TRANS.COPY"));
				((AbstractJob) theTransfer).setEndorser(destination.getUser());
				Transfer transferTask = (Transfer) theTransfer;
				transferTask.setSourcePortfolio(sourcePortfolio.getPortfolio());
				transferTask.setTargetPortfolio(targetPortfolio.getPortfolio()
						.getId());
				transferTask.setTargetUspace((AJOIdentifier) destination
						.getIdentifier());
				transferTask.setTargetVsite(destination.getVsite());
			}
			// try to delete previous portfolio in case of loops,
			// because NJS will complain about already existing
			// portfolio else
			DeletePortfolio deletePortfolio = new DeletePortfolio(
					"_DELETEPORTFOLIO_"
							+ ResourceManager.getNextObjectIdentifier());
			deletePortfolio
					.setPortfolio(sourcePortfolio.getPortfolio().getId());
			// set ignore failure, because portfolio will not exist yet,
			// if task is not in a loop and also in the first iteration
			deletePortfolio.setPropagateSuccessfulIfNotSuccessful(true);
			dummyGroup.add(deletePortfolio);
			dummyGroup.add(sourcePortfolio);
			dummyGroup.add(theTransfer);
			try {
				dummyGroup.addDependency(new Dependency(deletePortfolio,
						sourcePortfolio));
				dummyGroup.addDependency(new Dependency(sourcePortfolio,
						theTransfer));
			}
			catch (ActionGroup.InvalidDependencyException ex) {
				logger.log(Level.SEVERE, "Could not add dependency.", ex);
			}
			boolean sourceXspace = !(fileTransfer.getStorageItem() instanceof USpace);
			boolean targetXspace = !(fileTransfer.getOtherStorage() instanceof USpace);
			boolean renaming = !fileTransfer.getDestinationName().equals("")
					&& !fileTransfer.getDestinationName().equals(".")
					&& !fileTransfer.getDestinationName().equals("./");
			String copyDir = null;
			CreateDirectory createDir = null;
			CopyFile copySource = null;
			CopyFile copyTarget = null;
			// Use a hidden directory, which contains copies of the transfer
			// data,
			// if either the source or target storage is an Xspace
			// or if the transferred data has to be renamed
			if (sourceXspace || targetXspace || renaming) {
				// Create a temporary hidden directory
				copyDir = ".copyDir_"
						+ fileTransfer.getStorageItem().toString().replace(' ',
								'_') + "_"
						+ ResourceManager.getNextObjectIdentifier();
				createDir = new CreateDirectory();
				createDir.setName(createActionName("CREATE_IMPORT_DIR_"
						+ copyDir));
				createDir.setTarget(copyDir);
				logger.finest("New CreateDirectory: "
						+ createDir.getId().getName() + " ID: "
						+ createDir.getId());
				// source (including wildcards) and destination are taken from
				// transfer
				String sourceName = fileTransfer.getSourceName();
				copySource = new CopyFile();
				copySource
						.setName(createActionName("_TT_TRANS_IMPORT_BY_COPYFILE_"
								+ sourceName));
				copySource.addResource((PathedStorage) fileTransfer
						.getStorageItem());
				copySource.setFrom(sourceName);
				copySource.setTo(copyDir);
				// copy to new hidden directory in Uspace
				copySource.setOverwrite(true);
				// copy from X|Uspace to hidden directory in Uspace
				copySource.setCopyFromUspace(false);
				logger.finest("New source CopyFile: "
						+ copySource.getId().getName() + " ID: "
						+ copySource.getId());
				dummyGroup.add(createDir);
				dummyGroup.add(copySource);
				try {
					dummyGroup.addDependency(new Dependency(createDir,
							copySource));
					dummyGroup.addDependency(new Dependency(copySource,
							deletePortfolio));
				}
				catch (ActionGroup.InvalidDependencyException ex) {
					logger.log(Level.SEVERE, "Could not add dependency.", ex);
				}
				if (targetXspace || renaming) {
					// transfer the complete hidden directory
					sourcePortfolio.addFile(copyDir);
				}
				else {
					// transfer the files in the hidden directory in case
					// of Uspace target and no renamimg
					sourcePortfolio.addFile(copyDir + "/*");
				}
			}
			else {
				// source and target are Uspaces and no renaming is required
				sourcePortfolio.addFile(fileTransfer.getSourceName());
			}
			if (targetXspace || renaming) {
				// copy from internal hidden directory to destination
				String targetName = fileTransfer.getDestinationName();
				copyTarget = new CopyFile();
				copyTarget
						.setName(createActionName("_TT_TRANS_EXPORT_BY_COPYFILE_"
								+ targetName));
				// copy all files from hidden directory
				copyTarget.setFrom(copyDir + "/*");
				copyTarget.setTo(targetName);
				// set overwrite flag
				boolean overwrite = fileTransfer.getOverwritePolicy();
				copyTarget.setOverwrite(overwrite);
				if (destination.getVsite().equals(
						container.getParentJobContainer().getVsite())) {
					// copy from Uspace to X|Uspace
					copyTarget.setCopyFromUspace(true);
					PathedStorage targetStorage = (PathedStorage) fileTransfer
							.getOtherStorage();
					if (targetStorage instanceof USpace) {
						AlternativeUspace uspace = new AlternativeUspace();
						uspace.setUspace((AJOIdentifier) fileTransfer
								.getDestinationJob().getIdentifier());
						targetStorage = uspace;
					}
					copyTarget.addResource(targetStorage);
					dummyGroup.add(copyTarget);
					try {
						dummyGroup.addDependency(new Dependency(theTransfer,
								copyTarget));
					}
					catch (ActionGroup.InvalidDependencyException ex) {
						logger.log(Level.SEVERE, "Could not add dependency.",
								ex);
					}
				}
				else {
					Transfer transferTask = (Transfer) theTransfer;
					/*
					 * transferTask.add(copyTarget); try {
					 * transferTask.addDependency( new
					 * Dependency(transferTask.getPutPortfolio(), copyTarget)); }
					 * catch (ActionGroup.InvalidDependencyException ex) {
					 * logger.log(Level.SEVERE, "Could not add dependency.",
					 * ex); }
					 */
					AbstractJob copyJob = new AbstractJob(
							createActionName("COPYTARGETJOB"));
					copyJob.setEndorser(transferTask.getEndorser());
					copyJob.setVsite(transferTask.getTargetVsite());
					AlternativeUspace uspace = new AlternativeUspace();
					uspace.setUspace((AJOIdentifier) fileTransfer
							.getDestinationJob().getIdentifier());
					PathedStorage targetStorage = (PathedStorage) fileTransfer
							.getOtherStorage();
					if (targetStorage instanceof USpace) {
						targetStorage = uspace;
					}
					copyTarget.addResource(uspace);
					copyTarget.addResource(targetStorage);
					// copy from AlternativeUspace to X|AlternativeUspace
					copyTarget.setCopyFromUspace(true);
					copyJob.add(copyTarget);
					dummyGroup.add(copyJob);
					try {
						dummyGroup.addDependency(new Dependency(theTransfer,
								copyJob));
					}
					catch (ActionGroup.InvalidDependencyException ex) {
						logger.log(Level.SEVERE, "Could not add dependency.",
								ex);
					}
				}
				logger.finest("New target CopyFile: "
						+ copyTarget.getId().getName() + " ID: "
						+ copyTarget.getId());
			}
			actionGroup.add(dummyGroup);
		}
	}

	/**
	 * Build an import task for one specific storage location in Xspace
	 */
	private void buildTransferFile(Vector transfers, JobContainer destination) {
		for (int i = 0; i < transfers.size(); i++) {
			ActionGroup dummyGroup = new ActionGroup(
					createActionName("TRANSFERFILE_DUMMY_GROUP"));
			FileTransfer fileTransfer = (FileTransfer) transfers.elementAt(i);
			// remember complete action group in transfer objects
			fileTransfer.setAbstractAction(dummyGroup);
			boolean renaming = !fileTransfer.getDestinationName().equals("")
					&& !fileTransfer.getDestinationName().equals(".")
					&& !fileTransfer.getDestinationName().equals("./");
			boolean forDir = true;
			ActionGroup groupForDir = createTransferFileGroup(fileTransfer,
					forDir);
			if (renaming) {
				// use the directory variant
				dummyGroup.add(groupForDir);
			}
			else {
				// check for file or directory, this will enable the use of
				// simple AFT
				ActionGroup groupForFile = createTransferFileGroup(
						fileTransfer, !forDir);
				FileCheck fileCheck = new FileCheck(
						createActionName("FILECHECK_"
								+ fileTransfer.getSourceName()));
				fileCheck.setFileToCheck(fileTransfer.getSourceName());
				fileCheck.addResource((PathedStorage) fileTransfer
						.getStorageItem());
				fileCheck.setIsDirectory(Boolean.TRUE);
				If ifTask = new If(createActionName("IF_FILECHECK_"
						+ fileTransfer.getSourceName()));
				ifTask.setTestTask(fileCheck);
				ifTask.setTest(FileCheck_Outcome.SATISFIED);
				ifTask.setTrueBranch(groupForDir);
				ifTask.setFalseBranch(groupForFile);
				dummyGroup.add(fileCheck);
				dummyGroup.add(ifTask);
				try {
					dummyGroup.addDependency(new Dependency(fileCheck, ifTask));
				}
				catch (ActionGroup.InvalidDependencyException ex) {
					logger.log(Level.SEVERE, "Could not add dependency.", ex);
				}
			}
			actionGroup.add(dummyGroup);
		}
	}

	/**
	 * Build an import task for one specific storage location in Xspace
	 */
	private ActionGroup createTransferFileGroup(FileTransfer fileTransfer,
			boolean forDirectories) {
		ActionGroup dummyGroup = new ActionGroup(
				createActionName("TRANSFERFILE_DUMMY_GROUP" + "-DIR:"
						+ forDirectories));
		TransferFile theTransfer = new TransferFile();
		theTransfer.setName(createActionName("TT_TRANSFERFILE" + "-DIR:"
				+ forDirectories));
		PathedStorage sourceStorage = (PathedStorage) fileTransfer
				.getStorageItem();
		if (sourceStorage instanceof USpace) {
			AlternativeUspace uspace = new AlternativeUspace();
			uspace.setUspace((AJOIdentifier) container.getParentJobContainer()
					.getIdentifier());
			sourceStorage = uspace;
		}
		PathedStorage targetStorage = (PathedStorage) fileTransfer
				.getOtherStorage();
		if (targetStorage instanceof USpace) {
			AlternativeUspace uspace = new AlternativeUspace();
			uspace.setUspace((AJOIdentifier) fileTransfer.getDestinationJob()
					.getIdentifier());
			targetStorage = uspace;
		}
		theTransfer.setUser(fileTransfer.getDestinationJob().getUser());
		theTransfer.setSourceStorage(sourceStorage);
		theTransfer.setSourceFile(fileTransfer.getSourceName());
		theTransfer.setTargetVsite(fileTransfer.getDestinationJob().getVsite());
		theTransfer.setTargetStorage(targetStorage);
		theTransfer.setTargetFile(fileTransfer.getDestinationName());
		theTransfer.setOverwrite(fileTransfer.getOverwritePolicy());
		dummyGroup.add(theTransfer);
		String copyDir = null;
		CreateDirectory createDir = null;
		CopyFile copySource = null;
		boolean renaming = !fileTransfer.getDestinationName().equals("")
				&& !fileTransfer.getDestinationName().equals(".")
				&& !fileTransfer.getDestinationName().equals("./");
		// Use a hidden directory, which contains copies of the transfer data,
		// if either the source or target storage is an Xspace
		// or if the transferred data has to be renamed
		if (false && renaming) {
			// Create a temporary hidden directory
			copyDir = ".copyDir_"
					+ fileTransfer.getStorageItem().toString()
							.replace(' ', '_') + "_"
					+ ResourceManager.getNextObjectIdentifier();
			createDir = new CreateDirectory();
			createDir.setName(createActionName("CREATE_IMPORT_DIR_" + copyDir));
			createDir.setTarget(copyDir);
			logger.finest("New CreateDirectory: " + createDir.getId().getName()
					+ " ID: " + createDir.getId());
			// source (including wildcards) and destination are taken from
			// transfer
			String sourceName = fileTransfer.getSourceName();
			copySource = new CopyFile();
			copySource.setName(createActionName("_TT_TRANS_IMPORT_BY_COPYFILE_"
					+ sourceName));
			copySource.addResource((PathedStorage) fileTransfer
					.getStorageItem());
			copySource.setFrom(sourceName);
			copySource.setTo(copyDir);
			// copy to new hidden directory in Uspace
			copySource.setOverwrite(true);
			// copy from X|Uspace to hidden directory in Uspace
			copySource.setCopyFromUspace(false);
			logger.finest("New source CopyFile: "
					+ copySource.getId().getName() + " ID: "
					+ copySource.getId());
			dummyGroup.add(createDir);
			dummyGroup.add(copySource);
			try {
				dummyGroup.addDependency(new Dependency(createDir, copySource));
				dummyGroup
						.addDependency(new Dependency(copySource, theTransfer));
			}
			catch (ActionGroup.InvalidDependencyException ex) {
				logger.log(Level.SEVERE, "Could not add dependency.", ex);
			}
			AlternativeUspace uspace = new AlternativeUspace();
			uspace.setUspace((AJOIdentifier) container.getParentJobContainer()
					.getIdentifier());
			sourceStorage = uspace;
			sourceStorage.setPath(copyDir);
			theTransfer.setSourceStorage(sourceStorage);
			theTransfer.setSourceFile("*");
		}
		// Correct problem in default implementation of TransferFile
		//			if (createTransferDirectories != null ||
		//				!(targetStorage instanceof AlternativeUspace)) {
		if (forDirectories) {
			// First: create target of CopyFile as internal diretcory
			CopyFile importIt = theTransfer.getImport();
			CreateDirectory importDir = new CreateDirectory(
					createActionName("CREATE_IMPORT_DIR"));
			importDir.setTarget(importIt.getTo());
			dummyGroup.add(importDir);
			try {
				dummyGroup
						.addDependency(new Dependency(importDir, theTransfer));
			}
			catch (ActionGroup.InvalidDependencyException ex) {
				logger.log(Level.SEVERE, "Could not add dependency.", ex);
			}
			// Second: copy all files from internal directory
			// (which has been transferred) to the final target
			CopyFile exportIt = theTransfer.getRemoteJob().getExport();
			exportIt.setFrom(exportIt.getFrom() + "/*");
		}
		// just for debugging: do not delete the internal temporary
		//		theTransfer.getDelete().setTarget("ttt");
		//		theTransfer.getDelete().setPropagateSuccessfulIfNotSuccessful(true);
		return dummyGroup;
	}
}