/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.Vector;
import java.util.logging.Level;
import org.unicore.AJOIdentifier;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.DeletePortfolio;
import org.unicore.ajo.Portfolio;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.DeletePortfolio_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.outcome.PutPortfolio_Outcome;
import org.unicore.sets.OutcomeEnumeration;
import org.unicore.sets.ResourceSet;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.OutcomeEntry;
import com.pallas.unicore.resourcemanager.OutcomeManager;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Delete files from spool area that belong to a certain job
 * 
 * @author Thomas Kentemich
 * @version $Id: DeleteSpoolFiles.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class DeleteSpoolFiles extends ObservableRequestThread {

	private AbstractJob_Outcome ajOut;
	private Vector fileExports;
	private JobContainer parent;

	/**
	 * Constructor
	 * 
	 * @param parent
	 *            JobContainer that exports belong to
	 * @param fileExports
	 *            List of exports to be removed from spool area
	 */
	public DeleteSpoolFiles(JobContainer parent, Vector fileExports) {
		this.parent = parent;
		this.fileExports = fileExports;
	}

	/**
	 * Method will be called from start() in Thread
	 */
	public void run() {
		try {
			Vsite vsite = parent.getVsite();
			PutPortfolio_Outcome ppo = null;
			User user = parent.getUser();
			AJOIdentifier id = (AJOIdentifier) parent.getIdentifier();
			AbstractJob job = new AbstractJob(ResourceManager
					.getServicePrefix()
					+ "_DeleteSpool_"
					+ ResourceManager.getNextObjectIdentifier(), null, null,
					vsite, user);
			for (int i = 0; i < fileExports.size(); i++) {
				FileExport export = (FileExport) (fileExports.elementAt(i));
				Vsite matchingVsite = ResourceManager.getMatchingVsite(vsite);
				if (matchingVsite == null) {
					logger.severe("Cannot obtain matching Vsite from cache"
							+ vsite.getName());
					return;
				}
				NamedResourceSet siteResources = ResourceManager
						.getResourceSet(matchingVsite);
				if (siteResources.isSiteDefault()) {
					logger
							.severe("Cannot obtain site specific ResourceSet from Vsite= "
									+ vsite.getName());
					return;
				}
				else {
					logger.info("Found site specific ResourceSet for Vsite= "
							+ vsite.getName());
				}
				org.unicore.resources.Spool spoolResource = (org.unicore.resources.Spool) siteResources
						.findResource(new org.unicore.resources.Spool());
				if (spoolResource == null) {
					logger
							.severe("Cannot obtain spool space resource from Vsite= "
									+ vsite.getName());
					return;
				}
				else {
					logger.info("Found spool space resource for Vsite= "
							+ vsite.getName());
				}
				Vector spoolOutcomes = OutcomeManager.getOutcomeEntries(export
						.getSpoolTask());
				for (int j = 0; j < spoolOutcomes.size(); j++) {
					ppo = (PutPortfolio_Outcome) ((OutcomeEntry) spoolOutcomes
							.elementAt(j)).getOutcome();
					//uplName = ppo.getPortfolio().getName();
					Portfolio pf = new Portfolio("", ppo.getPortfolio()
							.getValue());
					DeletePortfolio deleteSpooledPortfolio = new DeletePortfolio(
							"DeletePF_(" + id.getName() + ")"
									+ ResourceManager.getNextObjectIdentifier());
					logger.info("Deleting Portfolio " + pf.getId());
					deleteSpooledPortfolio.setPortfolio(pf.getId());
					deleteSpooledPortfolio.setResources(new ResourceSet(
							spoolResource));
					job.add(deleteSpooledPortfolio);
				}
			}
			Reply roReply = nonPolling(job, vsite, user);
			// now analyze outcome
			if (!(roReply instanceof RetrieveOutcomeReply)) {
				throw new Exception("Cannot contact virtual site " + vsite
						+ "\nReason: " + roReply.getLastEntry().getComment());
			}
			else {
				// deserialize the Outcome Object
				Outcome outcome = (Outcome) new ObjectInputStream(
						new ByteArrayInputStream(
								((RetrieveOutcomeReply) roReply).getOutcome()))
						.readObject();
				if (outcome instanceof AbstractJob_Outcome) {
					AbstractJob_Outcome ajOut = ConsignForm
							.convertFrom((AbstractJob_Outcome) outcome);
					OutcomeEnumeration oen = ajOut.getOutcomes();
					while (oen.hasMoreElements()) {
						Outcome next = oen.nextElement();
						if (next instanceof DeletePortfolio_Outcome) {
							AbstractActionStatus status = next.getStatus();
							if (status
									.isEquivalent(AbstractActionStatus.NOT_SUCCESSFUL)) {
								logger.severe("DeletePortfolio failed: "
										+ next.getReason());
							}
						}
					}
				}
				else {
					throw new Exception("Unexpected type of outcome: "
							+ outcome);
				}
			}
		}
		catch (Exception e) {
			//UserMessages.warning("Could not delete spooled files for job " +
			// parent);
			logger.log(Level.SEVERE, "Could not delete spooled files for job "
					+ parent, e);
		}
		notifyObservers(this, ajOut);
	}
}