/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.ObjectInputStream;
import java.util.Vector;
import java.util.logging.Level;
import org.unicore.PortfolioIdentifier;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.CopyPortfolioToOutcome;
import org.unicore.ajo.ImportTask;
import org.unicore.ajo.ActionGroup.InvalidDependencyException;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.resources.Storage;
import org.unicore.sets.ResourceSet;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;
import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.connection.Connection;
import com.pallas.unicore.resourcemanager.OutcomeManager;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Get a list of files from a storage resource at xspace of a given vsite
 * 
 * @author Ralf Ratering
 * @version $Id: GetFilesFromXspace.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class GetFilesFromXspace extends ObservableRequestThread {

	private Vector remoteFiles = new Vector();
	private Vector localFiles = new Vector();
	private Vsite vsite;
	private Storage storage;
	private String[] filenames;

	/**
	 * Set parameters of request in constructor
	 * 
	 * @param remoteFiles
	 *            Vector of RemoteFile objects
	 * @throws Exception
	 *             if remoteFile objects are inconsistent in Vsite or Storage
	 */
	public GetFilesFromXspace(Vector remoteFiles) throws Exception {
		this.remoteFiles = remoteFiles;
		checkRemoteFiles();
	}

	private void checkRemoteFiles() throws Exception {
		vsite = null;
		storage = null;
		filenames = new String[remoteFiles.size()];
		for (int i = 0; i < remoteFiles.size(); i++) {
			// might cause a class cast exception
			RemoteFile file = (RemoteFile) remoteFiles.elementAt(i);
			// check vsites
			if (vsite == null) {
				vsite = file.getVsite();
			}
			else if (!vsite.equals(file.getVsite())) {
				throw new Exception(
						"Inconsistent virtual sites in remote file list.");
			}
			// check storage
			if (storage == null) {
				storage = file.getStorage();
			}
			else if (!storage.equals(file.getStorage())) {
				throw new Exception(
						"Inconsistent storage resources in remote file list.");
			}
			// get filenames
			filenames[i] = file.getSubspacePath();
		}
	}

	/**
	 * Get the list of filenames
	 * 
	 * @return Array containing filenames
	 */
	public Vector getRemoteFiles() {
		return remoteFiles;
	}

	/**
	 * Start the request
	 */
	public void run() {
		ImportTask importTask = new ImportTask("ImportTask");
		importTask.setResources(new ResourceSet(storage));
		importTask.setFiles(filenames);
		PortfolioIdentifier portfolioId = importTask.getPortfolio().getId();
		CopyPortfolioToOutcome copyPortfolio = new CopyPortfolioToOutcome(
				"CopyPortfolio");
		copyPortfolio.setTarget(portfolioId);
		String servicePrefix = ResourceManager.getServicePrefix();
		User user = ResourceManager.getUser(vsite);
		AbstractJob job = new AbstractJob(servicePrefix + "GetFileFromXSpace"
				+ ResourceManager.getNextObjectIdentifier(), null, null, vsite,
				user);
		job.add(importTask);
		job.add(copyPortfolio);
		try {
			job.addDependency(importTask, copyPortfolio);
		}
		catch (InvalidDependencyException e) {
			logger.log(Level.SEVERE, "Could not add dependency.", e);
		}
		Reply reply = nonPolling(job, vsite, user);
		AbstractJob_Outcome ajOut = null;
		// now analyze outcome
		if (!(reply instanceof RetrieveOutcomeReply)) {
			logger.warning("Virtual site not available: " + vsite
					+ "\nReason: " + reply.getLastEntry().getComment());
			return;
		}
		else {
			try {
				RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) reply;
				// deserialize the Outcome Object
				byte[] outcomeBytes = roReply.getOutcome();
				Outcome outcome = (Outcome) new ObjectInputStream(
						new ByteArrayInputStream(outcomeBytes)).readObject();
				ajOut = ConsignForm.convertFrom((AbstractJob_Outcome) outcome);
			}
			catch (Exception e) {
				logger.log(Level.SEVERE, "Could not get files from uspace.", e);
			}
		}
		// figure out and remember local filenames
		String pfdir = importTask.getPortfolio().getUPLDirectoryName();
		String sessionDir = Connection.getSessionDir();
		Vector streamedFiles = OutcomeManager.getStreamedFiles();
		localFiles.clear();
		for (int i = 0; i < filenames.length; i++) {
			String name = filenames[i];
			int index = filenames[i].lastIndexOf("/");
			if (index > 0) {
				name = filenames[i].substring(index);
			}
			String ending = pfdir + name;
			for (int j = 0; j < streamedFiles.size(); j++) {
				String streamedFile = (String) streamedFiles.elementAt(j);
				if (streamedFile.endsWith(ending)) {
					localFiles.add(new File(sessionDir + File.separator
							+ streamedFile));
				}
			}
		}
		notifyObservers(this, ajOut);
	}

	/**
	 * Get Vector of resulting Files in local scratch directory
	 * 
	 * @return resulting Files in local scratch directory
	 */
	public Vector getLocalFiles() {
		return localFiles;
	}
}