/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.Vector;
import java.util.logging.Level;
import org.unicore.AJOIdentifier;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.CopyPortfolioToOutcome;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.CopyPortfolioToOutcome_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.outcome.PutPortfolio_Outcome;
import org.unicore.sets.OutcomeEnumeration;
import org.unicore.sets.ResourceSet;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;
import com.pallas.unicore.client.trees.ContainerNode;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.OutcomeEntry;
import com.pallas.unicore.resourcemanager.OutcomeManager;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 * Get list of spooled files for a job container
 * 
 * @author Thomas Kentemich
 * @version $Id: GetSpooledFiles.java,v 1.1 2004/05/25 14:58:53 rmenday Exp $
 */
public class GetSpooledFiles extends ObservableRequestThread {

	private AbstractJob_Outcome ajOut;
	private FileExport export;
	private JobContainer parent;
	private ContainerNode parentNode;

	/**
	 * Constructor
	 * 
	 * @param parentNode
	 *            Job node in job monitor
	 * @param export
	 */
	public GetSpooledFiles(ContainerNode parentNode, FileExport export) {
		this.parentNode = parentNode;
		this.parent = export.getContainer().getParentJobContainer();
		this.export = export;
	}

	/**
	 * Constructor
	 * 
	 * @param parentNode
	 *            Job node in job monitor
	 * @param parent
	 *            job container
	 * @param export
	 */
	public GetSpooledFiles(ContainerNode parentNode, JobContainer parent,
			FileExport export) {
		this.parent = parent;
		this.export = export;
		this.parentNode = parentNode;
	}

	/**
	 * Return the FileExort object associated with this task
	 * 
	 * @return file export
	 */
	public FileExport getFileExport() {
		return this.export;
	}

	/**
	 * Return the Container node for the files to be retrieved
	 * 
	 * @return container node
	 */
	public ContainerNode getNode() {
		return this.parentNode;
	}

	/**
	 * Start the Thread
	 */
	public void run() {
		try {
			//generating the job
			if ((export == null)) {
				throw new Exception("No file export found in job " + parent);
			}
			parentNode.setState(ContainerNode.STATE_UPDATING);
			Vsite vsite = ResourceManager.getMatchingVsite(parent.getVsite());
			if (vsite == null) {
				logger.severe("Cannot obtain matching Vsite from cache"
						+ vsite.getName());
				return;
			}
			User user = parent.getUser();
			AJOIdentifier id = (AJOIdentifier) parent.getIdentifier();
			AbstractJob job = new AbstractJob(ResourceManager
					.getServicePrefix()
					+ "_UnSpool_" + ResourceManager.getNextObjectIdentifier(),
					null, null, vsite, user);
			NamedResourceSet siteResources = ResourceManager
					.getResourceSet(vsite);
			if (siteResources.isSiteDefault()) {
				logger
						.severe("Cannot obtain site specific ResourceSet from Vsite= "
								+ vsite.getName());
				return;
			}
			else {
				logger.info("Found site specific ResourceSet for Vsite= "
						+ vsite.getName());
			}
			org.unicore.resources.Spool spoolResource = (org.unicore.resources.Spool) siteResources
					.findResource(new org.unicore.resources.Spool());
			if (spoolResource == null) {
				logger.severe("Cannot obtain spool space resource from Vsite= "
						+ vsite.getName());
				return;
			}
			else {
				logger.fine("Found spool space resource for Vsite= "
						+ vsite.getName());
			}
			Vector spoolOutcomes = OutcomeManager.getOutcomeEntries(export
					.getSpoolTask());
			if (spoolOutcomes.size() <= 0) {
				// no Outcome
				logger.severe("Cannot find outcome of spool task for file: "
						+ export.getSourceName());
				return;
			}
			else {
				logger.info("Unspooling file: " + export.getSourceName());
			}
			for (int i = 0; i < spoolOutcomes.size(); i++) {
				CopyPortfolioToOutcome unspool = new CopyPortfolioToOutcome(
						"CopyPF");
				PutPortfolio_Outcome ppo = (PutPortfolio_Outcome) ((OutcomeEntry) spoolOutcomes
						.elementAt(i)).getOutcome();
				unspool.setTarget(ppo.getPortfolio());
				unspool.setResources(new ResourceSet(spoolResource));
				job.add(unspool);
			}
			Reply roReply = nonPolling(job, vsite, user, export, export
					.getFileMode());
			// now analyze outcome
			if (!(roReply instanceof RetrieveOutcomeReply)) {
				throw new Exception("Could not contact virtual site " + vsite);
			}
			else {
				// deserialize the Outcome Object
				Outcome outcome = (Outcome) new ObjectInputStream(
						new ByteArrayInputStream(
								((RetrieveOutcomeReply) roReply).getOutcome()))
						.readObject();
				if (outcome instanceof AbstractJob_Outcome) {
					ajOut = ConsignForm
							.convertFrom((AbstractJob_Outcome) outcome);
					OutcomeEnumeration oen = ajOut.getOutcomes();
					while (oen.hasMoreElements()) {
						Outcome next = oen.nextElement();
						if (next instanceof CopyPortfolioToOutcome_Outcome) {
							AbstractActionStatus status = next.getStatus();
							if (status
									.isEquivalent(AbstractActionStatus.NOT_SUCCESSFUL)) {
								logger.severe("CopyPortfolio failed: "
										+ next.getReason());
							}
						}
					}
				}
				else {
					throw new Exception("Unexpected type of outcome " + outcome);
				}
			}
		}
		catch (Exception e) {
			UserMessages.warning("Could not get spooled files for job "
					+ parent + "\n" + e.getLocalizedMessage());
			logger.log(Level.SEVERE, "Could not get spooled files for job "
					+ parent, e);
		}
		// notify all my observers on task end
		notifyObservers(this, ajOut);
	}
}