/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.StreamCorruptedException;
import java.security.KeyStoreException;
import java.security.SignatureException;
import java.util.Date;
import java.util.Vector;
import java.util.logging.Level;
import javax.net.ssl.SSLPeerUnverifiedException;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.Outcome;
import org.unicore.upl.ConsignJob;
import org.unicore.upl.ConsignJobReply;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcome;
import org.unicore.upl.RetrieveOutcomeAck;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.upl.UnicoreResponse;
import com.pallas.unicore.connection.Connection;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.security.JobConverter;

/**
 * A request started as a new thread that will notify its observers when
 * finished.
 * 
 * @author Ralf Ratering
 * @version $Id: ObservableRequestThread.java,v 1.18 2004/03/03 14:50:02
 *          tkentemi Exp $
 */
public abstract class ObservableRequestThread extends ObservableThread {

	// set flag to true when thread should stop
	private boolean interrupted;

	/**
	 * set flag to true when thread should stop
	 * 
	 * @param interrupted
	 *            true causes thread to stop
	 */
	public void setInterrupted(boolean interrupted) {
		this.interrupted = interrupted;
	}

	/**
	 * Submit a job in non-polling mode. Caution: If non-polling connection
	 * times out the request will automatically switch into polling mode.
	 * 
	 * @param job
	 *            the job to submit
	 * @param vsite
	 *            target Vsite
	 * @param user
	 *            consigner of the job
	 * @return reply object
	 */
	public Reply nonPolling(AbstractJob job, Vsite vsite, User user) {
		return nonPolling(job, vsite, user, new Vector());
	}

	/**
	 * Submit the job in non-polling mode and strem some files with it
	 * 
	 * @param job
	 *            the job to submit
	 * @param vsite
	 *            target Vsite
	 * @param user
	 *            consigner of the job
	 * @param streamedFiles
	 *            list of files to be streamed
	 * @return reply object
	 */
	public Reply nonPolling(AbstractJob job, Vsite vsite, User user,
			Vector streamedFiles) {
		Reply reply = null;
		try {
			Connection connection = ResourceManager.getConnectionManager()
					.lookupConnection(vsite, ResourceManager.getUser(vsite));
			reply = doRequest(job, vsite, user, connection, streamedFiles);
		}
		catch (SSLPeerUnverifiedException e) {
			logger.log(Level.SEVERE, "Non-polling request failed.", e);
			ResourceManager.getResourceCache().addUnavailableVsite(
					vsite,
					"Identity " + ResourceManager.getUser(vsite).getName()
							+ " has no permission at " + vsite);
		}
		// this one will be thrown in doRequest
		catch (IOException e) {
			logger.log(Level.SEVERE, "Non-polling request failed.", e);
			ResourceManager.getResourceCache().addUnavailableVsite(
					vsite,
					"Identity " + ResourceManager.getUser(vsite).getName()
							+ " could not contact " + vsite);
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "Non-polling request failed.", e);
		}
		return reply;
	}

	/**
	 * Submit a job in non-polling mode and directly store file exports in
	 * destination directory without copying them to spooling dir. This will
	 * only work for single files in the stream. Caution: If non-polling
	 * connection times out the request will automatically switch into polling
	 * mode.
	 * 
	 * @param job
	 *            the job to submit
	 * @param vsite
	 *            target Vsite
	 * @param user
	 *            consigner of the job
	 * @param directStore
	 *            File export that will be stored directly to destination dir
	 * @param isAscii
	 *            true if export is ascii format
	 * @return reply object
	 */
	public Reply nonPolling(AbstractJob job, Vsite vsite, User user,
			FileExport directStore, boolean isAscii) throws Exception {
		Reply reply = null;
		Connection connection = null;
		/*
		 * if (vsite.getType().getDescription().equals("GS")) { connection = new
		 * GSConnection( vsite.getAddress(), ResourceManager.getUser(vsite),
		 * directStore, isAscii); } else {
		 */
		connection = new Connection(vsite.getAddress(), ResourceManager
				.getUser(vsite), directStore, isAscii);
		//		}
		reply = doRequest(job, vsite, user, connection, new Vector());
		return reply;
	}

	/**
	 * Submit a job in polling mode. Caution: This method will not fetch outcome
	 * or delete jobs
	 * 
	 * @param job
	 *            Job to be submitted
	 * @param vsite
	 *            destination vsite
	 * @param user
	 *            endorser
	 * @return NJS reply
	 * @throws Exception
	 */
	public Reply polling(AbstractJob job, Vsite vsite, User user)
			throws Exception {
		return polling(job, vsite, user, new Vector());
	}

	/**
	 * Submit a job in polling mode. Caution: This method will not fetch outcome
	 * or delete jobs
	 * 
	 * @param job
	 *            Job to be submitted
	 * @param vsite
	 *            destination vsite
	 * @param user
	 *            endorser
	 * @param streamedFiles
	 *            Vector of files to be streamed with AJO
	 * @return NJS reply
	 * @throws Exception
	 */
	public Reply polling(AbstractJob job, Vsite vsite, User user,
			Vector streamedFiles) throws Exception {
		Connection connection = ResourceManager.getConnectionManager()
				.lookupConnection(vsite, ResourceManager.getUser(vsite));
		// Limit the lifetime of Service AJOs, default 30 s
		job.setTerminationTime(new Date(System.currentTimeMillis()
				+ connection.getAjoLifetime()));
		ConsignJob cj = JobConverter.convert(job, true, true);
		// send consign job an streamed files
		return connection.sendMessage(cj, streamedFiles);
	}

	private Reply doRequest(AbstractJob job, Vsite vsite, User user,
			Connection connection, Vector streamedFiles)
			throws StreamCorruptedException, ClassNotFoundException,
			IOException, KeyStoreException, SignatureException, Exception {
		interrupted = false;
		// Limit the lifetime of Service AJOs, default 30 s
		job.setTerminationTime(new Date(System.currentTimeMillis()
				+ connection.getAjoLifetime()));
		ConsignJob cj = JobConverter.convert(job, false, true);
		// send consign job an streamed files
		Reply reply = connection.sendMessage(cj, streamedFiles);
		// request has been switched to polling due to NJS time out
		if (reply instanceof ConsignJobReply) {
			int returnCode = reply.getLastEntry().getReturnCode();
			logger.info("Switching to polling mode for vsite: "
					+ vsite.getName());
			Outcome result = null;
			while ((result == null || !result.getStatus().isEquivalent(
					AbstractActionStatus.DONE))
					&& !interrupted) {
				try {
					Thread.sleep(5000);
				}
				catch (InterruptedException e) {
					logger.log(Level.WARNING, "Thread interrupted.", e);
				}
				logger.fine("Polling vsite: " + vsite.getName());
				RetrieveOutcome ro = new RetrieveOutcome(job.getAJOId(), vsite);
				JobConverter.convert(ro, true);
				reply = connection.sendMessage(ro, null);
				returnCode = reply.getLastEntry().getReturnCode();
				if (returnCode != 0) {
					throw new IOException("Request failed." + returnCode
							+ " reason: " + reply.getLastEntry().getComment());
				}
				if (reply instanceof RetrieveOutcomeReply) {
					RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) connection
							.getReply();
					result = (Outcome) (new ObjectInputStream(
							new ByteArrayInputStream(roReply.getOutcome())))
							.readObject();
				}
			}
			// acknowledge outcome
			RetrieveOutcomeAck roa = new RetrieveOutcomeAck(job.getAJOId(),
					vsite);
			JobConverter.convert(roa, true);
			connection.sendMessage(roa, null);
		}
		else if (!(reply instanceof RetrieveOutcomeReply)) {
			UnicoreResponse resp = reply.getLastEntry();
			logger.severe("Error consigning an AJO:" + resp.getReturnCode()
					+ ", Reason: " + resp.getComment());
		}
		return reply;
	}
}