package org.unicore.ajo;

import java.util.*;

/**
 * An ActionGroup is an ordered collection of AbstractActions. 
 * <p>
 * The AbstractActions 
 * will be executed in the order defined by the dependency relationships.
 * (The implementation is a directed acyclic graph). ActionGroups are
 * executed in the Uspace of their parent AJO, unless they are instances
 * of AbstractJob, when they are consigned to another Vsite and have their
 * own Uspace.
 *
 * @see org.unicore.outcome.ActionGroup_Outcome
 *
 * @author M. Drescher (Pallas)
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3 (AJO 1 as ActionGraph)
 *
 * @version ActionGroup.java 1.9 10/30/02 
 *
 **/
public class ActionGroup extends AbstractAction {

    static final long serialVersionUID = 6117278589085234895L;

    /**
     * Thrown when a dependency could not be added to an ActionGroup. The dependency
     * was either invalid (predecessor or successor AbstractActions are null or
     * not both in the ActionGroup) or would introduce cycles into the ActionGroup.
     *
     **/
    public static class InvalidDependencyException extends Exception {
        static final long serialVersionUID = 400L;
        public InvalidDependencyException() {
            super();
        }
        public InvalidDependencyException(String message) {
            super(message);
        }
    }

    public ActionGroup() {
        this("");
    }

    /**
     * Create a new ActionGroup (with no actions to perform).
     * <p>
     * @param name The name of the ActionGroup
     * @param extra_information Client supplied extra information about the ActionGroup
     *
     **/
    public ActionGroup( String name,
                        byte[] extra_information
                      ) {
        super(name,extra_information);

        nodes = new Vector();
        edges = new Vector();
        successors = new Vector();

    }

    /**
     * Create a new ActionGroup (with no actions to perform).
     *
     **/
    public ActionGroup(String name) {
        this(name, (byte[]) null);
    }

    /*
       Simple implementation of a DAG. Represent both
       the nodes and edges as Java Vectors.
    */
    private Vector nodes;
    private Vector edges;
    private Vector successors;

    /**
     * Add an <tt>AbstractAction</tt> to the group. This will have no dependencies.
     * <p>
     * <b>Pre:</b>  The <tt>AbstractAction</tt> is not in the group.
     * <br>
     * <b>Post:</b> The <tt>AbstractAction</tt> has been added to the group without 
     *              any dependencies.
     * <p>
     * @param action The <tt>AbstractAction</tt> to be added to the group.
     * 
     **/
    public void add(AbstractAction action) {
        nodes.addElement(action);
    }

    private boolean has_cycles;

    public boolean hasCyles() {return has_cycles;}

    /**
     * Add a dependency to the group.
     * <p>
     * <b>Pre:</b>  canAdd(aDependency) and not isIn(aDependency)
     * <br>
     * <b>Post:</b> The dependency is in the group (once).
     * <p>
     * @deprecated AJO 4 Use addDependency(Dependency dependency) instead, <em>or</em>
     *             always check that the new dependency is valid and will not introduce cycles by
     *             calling canAdd(Dependency dependency) first.
     **/
    public void add(Dependency dependency) {
        edges.addElement(dependency);

        // Add the successor to the Vector, Vector is a bag
        // and so multiple entries represent the number of dependencies
        successors.addElement(dependency.getSuccessor());
    }

    /**
     * Add a dependency to the group.
     *
     * @throws InvalidDependencyException The new dependency is invalid or would create a cycle in the ActionGroup
     **/
    public void addDependency(Dependency dependency) throws InvalidDependencyException {
        if(!canAdd(dependency)) {
            throw new InvalidDependencyException("The new dependency is invalid or would create a cycle in  the ActionGroup");
        }

        edges.addElement(dependency);

        // Add the successor to the Vector, Vector is a bag
        // and so multiple entries represent the number of dependencies
        successors.addElement(dependency.getSuccessor());
    }

    /**
     * Add a dependency between the two AbstractActions (adding the AbstractActions if necessary)
     *
     * @throws InvalidDependencyException The new dependency would create a cycle in the ActionGroup
     *
     **/
    public void addDependency(AbstractAction predecessor, AbstractAction successor) throws InvalidDependencyException {

        Dependency dependency = new Dependency(predecessor,successor);

        try {
            addDependency(dependency);
        }
        catch(InvalidDependencyException idex) {
            // Why'd it go wrong?
            if(predecessor == null) throw new InvalidDependencyException("Predecessor AbstractAction is null");
            if(successor   == null) throw new InvalidDependencyException("Successor AbstractAction is null");

            if(!isIn(predecessor)) add(predecessor);
            if(!isIn(successor))   add(successor);

            try {
                addDependency(dependency);
            }
            catch(InvalidDependencyException idexidex) {
                throw new InvalidDependencyException("The new dependency would create a cycle in  the ActionGroup");
            }

        }
    }

    /**
     * Test if the <tt>Dependency</tt> can be added to the group. For
     * this to be possible both the parent and child <tt>AbstractAction</tt>s
     * must already be in the group and, as we are representing a DAG, the
     * <tt>Dependency</tt> must not create cycles.
     * <p>
     * <b>Pre:</b>  True.
     * <br>
     * <b>Post:</b> Returns true iff all <tt>AbstractAction</tt>s refered to 
     *              in the dependency are in the group and it's addition 
     *              creates no cycles and the <tt>Dependency</tt> has
     *              distinct predecessor and successor.
     * <p>
     **/
    public boolean canAdd(Dependency dependency) {

        AbstractAction pred = dependency.getPredecessor();
        AbstractAction succ = dependency.getSuccessor();

        if( pred == null || succ == null ) {
            return false;
        }
        else if( pred.equals(succ) ) {
            return false;
        }
        else if ( !( isIn(pred) && isIn(succ) ) ) {
            return false;
        }
        else {
            // Does this dependency introduce any cycles?
            // We **know** that there are no cycles in the DAG
            // now so we only need to look for an existing path from
            // succ to pred.

            CycleFinder c = new CycleFinder(pred);
            return !c.find(succ);
        }
    }

    class CycleFinder {
        AbstractAction start; // if we return to this we have a cycle
        Dependency d[];

        public CycleFinder(AbstractAction s) {
            start = s;
            // Copy dependencies out (cleaner/more efficient)
            d = new Dependency[edges.size()];
            edges.copyInto(d);
        }

        public boolean find(AbstractAction from) {

            boolean cycle_found = false;

            for(int i=0; i < d.length; i++) {
                if(d[i].getPredecessor().equals(from)) {
                    if(d[i].getSuccessor().equals(start)) {
                        cycle_found = true; 
                    }
                    else {
                        cycle_found = find(d[i].getSuccessor());
                    }
                    if(cycle_found) break;
                }
            }
            return cycle_found;
        }
    }




    /**
     * Get an enumeration over all the <tt>AbstractAction</tt>s in the
     * group.
     * <p>
     * <b>Pre:</b>  True.
     * <br>
     * <b>Post:</b> Returns a, possibly empty, <tt>Enumeration</tt>, of all 
     *              <tt>AbstractAction</tt>s contained in the group.
     * <p>
     **/
    public Enumeration getActions() {
        return nodes.elements();
    }

    /**
     * Get an enumeration over all the <tt>Dependencies</tt>s in the
     * group.
     * <p>
     * <b>Pre:</b>  True.
     * <br>
     * <b>Post:</b> Returns a, possibly empty, <tt>Enumeration</tt> of all 
     *              <tt>Dependencies</tt>s contained in the group.
     * <p>
     **/
    public Enumeration getDependencies() {
        return edges.elements();
    }

    /**
     * Test if an <tt>AbstractAction</tt> is in the group.
     * <p>
     * <b>Pre:</b>  True.
     * <br>
     * <b>Post:</b> Return true iff the <tt>AbstractAction</tt> is 
     *              in the group.
     * <p>
     *
     **/
    public boolean isIn(AbstractAction action) {
        return nodes.contains(action);
    }

    /**
     * Test if an <tt>Dependency</tt> is in the group.
     * <p>
     * <b>Pre:</b>  True.
     * <br>
     * <b>Post:</b> Return true iff the <tt>Dependency</tt> is 
     *              in the group.
     * <p>
     *
     **/
    public boolean isIn(Dependency dependency) {
        return edges.contains(dependency);
    }

    /**
     * Remove an <tt>AbstractAction</tt> from the group.
     * <p>
     * <b>Pre:</b>  True.
     * <br>
     * <b>Post:</b> The <tt>AbstractAction</tt> is not in 
     *              the group and any dependencies that relied transitively 
     *              on that <tt>AbstractAction</tt> have been replaced 
     *              with direct dependencies.
     * <p>
     **/
    public void remove(AbstractAction action) {

        if( nodes.removeElement(action) ) {
            // The AbstractAction was in the DAG
            // We now need to find out which dependencies it was
            // involved in

            Vector in_pred = new Vector(edges.size());
            Vector in_succ = new Vector(edges.size());

            for( int i=edges.size()-1; i >= 0 ; i--) {
                // ... loop backwards as we are removing elements

                Dependency d = (Dependency) edges.elementAt(i);

                if(d.getPredecessor().equals(action)) {
                    in_pred.addElement(d);
                    remove(d); // remove the old dependency from group
                }
                else if (d.getSuccessor().equals(action)) {
                    in_succ.addElement(d);
                    remove(d); // remove the old dependency from group
                }

            }

            // Now add the new dependencies
            for(int s=0; s < in_succ.size(); s++) {
            for(int p=0; p < in_pred.size(); p++) {
                Dependency d = new Dependency(
                    ((Dependency) in_succ.elementAt(s)).getPredecessor(),
                    ((Dependency) in_pred.elementAt(p)).getSuccessor() 
                                             );
                if(!isIn(d)) add(d);
            }
            }

        }

        // Note. If the AbstractAction was not in the DAG, then
        // the precondition on addDependency ensures that there 
        // are no depnedencies involving this AbstractAction. So
        // do nothing here.

    }

    /**
     * Remove an <tt>Dependency</tt> from the group.
     * <p>
     * <b>Pre:</b>  True.
     * <br>
     * <b>Post:</b> The dependency is not in the group.
     * <p>
     **/
    public void remove(Dependency dependency) {
        edges.removeElement(dependency);

        successors.removeElement(dependency.getSuccessor());
    }

    /**
     * Clear the group of all actions and, since without actions they
     * have no meaning, all dependencies.
     * <p>
     * <b>Pre:</b>  True.
     * <br>
     * <b>Post:</b> The group is empty.
     * <p>
     **/
    public void removeAllActions() {
        nodes.removeAllElements();
        removeAllDependencies();
    }

    /**
     * Clear the group of all dependencies.
     *<p>
     * <b>Pre:</b>  True.
     *<br>
     * <b>Post:</b> There are no dependencies in the group (actions remain).
     *<p>
     **/
    public void removeAllDependencies() {
        edges.removeAllElements();
        successors.removeAllElements();
    }


    /**
     * Return the AbstractActions that are ready to run,
     * that is they are not a successor in any dependency. An 
     * ActionGroup can be walked by using this method on a copy
     * and removing those visited.
     * <p>
     * This method considers only its own DAG so it returns 
     * any child ActionGroups but does
     * not return the AbstractActions in the child ActionGroups.
     * <p>
     * <b>Pre:</b> True.
     * <br>
     * <b>Post:</b> The enumeration over any ready actions
     * <p>
     **/
    public Enumeration ready() {

        // We have maintained an accurate list of all successor
        // Actions in successors. 

        Vector result = new Vector();

        Enumeration e = getActions();
        while(e.hasMoreElements()) {
            Object action = e.nextElement();
            if( !successors.contains(action) ) {
                // This action is not a successor,
                // so must be ready
                result.addElement(action);
            }
        }

        return result.elements();
    }

    public boolean containsCycles() {

        // create a map of each node to a list of successors
        AbstractAction[] n = new AbstractAction[nodes.size()];
        Set[] succs = new Set[nodes.size()];

        for(int i = 0; i < nodes.size(); i++) {
            n[i] = (AbstractAction)nodes.get(i);

            if(n[i] instanceof ActionGroup) {
                if( ((ActionGroup)n[i]).containsCycles() ) return true;
            }
            else if(n[i] instanceof ConditionalAction) {
                ConditionalAction ca = (ConditionalAction)n[i];
                if(ca.getTrueBranch()  instanceof ActionGroup && ((ActionGroup)ca.getTrueBranch()).containsCycles() ) return true;
                if(ca.getFalseBranch() instanceof ActionGroup && ((ActionGroup)ca.getFalseBranch()).containsCycles() ) return true;
            }

            succs[i] = new HashSet();
            Iterator j = edges.iterator();
            while(j.hasNext()) {
                Dependency d = (Dependency)j.next();
                if(d.getPredecessor().equals(n[i])) succs[i].add(d.getSuccessor());
            }
        }

        // Remove all leaf nodes (no succs), repeatedly until none left
        boolean removed;
        int count = n.length;
        do {
            removed = false;
            for(int i = 0; i < n.length; i++) {
                if(n[i] != null) {
                    if(succs[i].size() == 0) {
                        for(int j = 0; j < n.length; j++) {
                            succs[j].remove(n[i]);
                        }
                        removed = true;
                        count--;
                        n[i] = null;
                    }
                }
                else {
                }
            }
        } while(removed);

        // If there are any left, then must have a cycle
        return count > 0;
    }
        

}
//
// Change history:
//          11 Oct 2002 SvdB Fixed cycles
//          18 Mar 2002 SvdB V4 no arg constructor
//          20 Dec 2000 SvdB Removed clone
//           5 May 2000 SvdB Renamed (from ActionGraph), made an AbstractAction 
//           7 Dec 1998 SvdB Second release to Unicore 
//           2 Sep 1998 SvdB added ready, clone methods 
//          18 Jun 1998 SvdB canAdd checks and rejects null Actions 
//          12 Jun 1998 SvdB Initial release to Unicore 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
