package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * ImportTask copies files into the Uspace.
 * <p>
 * The files imported from Xspace, the location determined by
 * instance of {@link org.unicore.resources.Home}, 
 * {@link org.unicore.resources.Root}, {@link org.unicore.resources.Temp} or
 * {@link org.unicore.resources.StorageServer} found in the resources (default is
 * the root of the Home fielsystem).
 * <p>
 * If any of the file names are globbed (*, ? and []), then
 * all the files that match the globbing at run time will be
 * imported. The copy is recursive, the contents of any named directories
 * and all their subdirectories are copied to the Uspace.
 * <p>
 * The destination Portfolio is created in the Uspace, the subdirectory determined
 * by the instance of {@link org.unicore.resources.USpace}
 * found in the resources (default is the Uspace root). 
 *
 * @see org.unicore.outcome.ImportTask_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 1
 *
 * @version ImportTask.java 1.8 09/18/02 
 * 
 **/
public final class ImportTask extends FileTransfer {

    static final long serialVersionUID = 3331869012248937540L;

    /**
     * Create a new ImportTask.
     * <p>
     * @param name The name of the ImportTask
     * @param extra_information Client supplied extra information about the ImportTask
     * @param resources Resources needed to execute the ImportTask
     * @param status The use that will be made of the imported files
     * @param files The names of the files to import
     *
     * @deprecated AJO 4.0 Use SymbolicLink taks for optimisations
     *
     **/
    public ImportTask( String name,
                       byte[] extra_information,
                       ResourceSet resource_limits,
                       FileImportStatus status, 
                       String[] files
                     ) {

        super(name,extra_information,resource_limits);

        this.status = status;
        this.files = files;
    }

    /**
     * Create a new ImportTask.
     * <p>
     * @param name The name of the ImportTask
     * @param resources Resources needed to execute the ImportTask
     * @param status The use that will be made of the imported files
     * @param files The names of the files to import
     *
     * @deprecated AJO 4.0 Use SymbolicLink taks for optimisations
     *
     **/
    public ImportTask( String name,
                       ResourceSet resource_limits,
                       FileImportStatus status, 
                       String[] files
                     ) {

        this(name,(byte[]) null,resource_limits, status, files);
    }

    /**
     * Create a new ImportTask.
     * <p>
     * @param name The name of the ImportTask
     * @param extra_information Client supplied extra information about the ImportTask
     * @param resources Resources needed to execute the ImportTask
     * @param files The names of the files to import
     * @param overwrite Whether the Import should overwrite any existing files
     *
     **/
    public ImportTask( String name,
                       byte[] extra_information,
                       ResourceSet resource_limits,
                       String[] files,
                       boolean overwrite
                     ) {

        super(name,extra_information,resource_limits);

        this.files = files;
        this.overwrite = overwrite;
    }

    /**
     * Create a new ImportTask.
     * <p>
     * @param name The name of the ImportTask
     * @param resources Resources needed to execute the ImportTask
     * @param files The names of the files to import
     * @param overwrite Whether the Import should overwrite any existing files
     *
     **/
    public ImportTask( String name,
                       ResourceSet resource_limits,
                       String[] files,
                       boolean overwrite
                     ) {

        this(name,(byte[]) null,resource_limits, files, true);
    }

    /**
     * Create a new ImportTask.
     * <p>
     * @param name The name of the ImportTask
     *
     **/
    public ImportTask(String name) {
        this(name,(byte[]) null, (ResourceSet) null, (String[]) null, true);
    }

    public ImportTask() {
        this("");
    }

    private String[] files;
    /**
     * Returns the names of the files that will be imported.
     * <p>
     * If any of the file names are globbed (*, ? and []), then
     * all the files that match the globbing at run time will be
     * imported.
     *
     **/
    public String[] getFiles() {return files;}

    /**
     * Set the names of the files that will be imported.
     *
     **/
    public void setFiles(String[] files) {this.files = files;}

    /**
     * Add a file to be imported.
     *
     **/
    public void addFile(String name) {
        if(files == null) {
            files = new String[1];
            files[0] = name;
        }
        else {
            String[] temp = new String[files.length+1];
            System.arraycopy(files,0,temp,0,files.length);
            temp[files.length] = name;
            files = temp;
        }
    }

    private Portfolio portfolio;
    /**
     * Get the Portfolio representing the imported files.
     *
     **/
    public Portfolio getPortfolio() {
        if(portfolio == null) portfolio = new Portfolio("Import");
        return portfolio;
    }

    private FileImportStatus status;
    /**
     * Return the use that will be made of the imported files.
     *
     * @deprecated AJO 4.0 Use SymbolicLink taks for optimisations
     *
     **/
    public FileImportStatus getStatus() {return status;}

    /**
     * Set the use that will be made of the imported files.
     *
     * @deprecated AJO 4.0 Use SymbolicLink taks for optimisations
     *
     **/
    public void setStatus(FileImportStatus status) {this.status = status;}

    boolean overwrite;
    /**
     * Overwrite existing destination files?
     * <p>
     * If no overwriting is allowed (this method returns false), then the import will
     * fail if the copy would overwrite an existing file. If any source file is a
     * directory, then the test will be applied to each file in the directory (that
     * is, writing files into existing directories is always allowed).
     * <p>
     * Overwriting existing files or writing into existing directories is always
     * subject to permissions on the existing files or directories and the task
     * will fail if there is no suitable write permission.
     *
     **/
    public boolean willOverwrite() {return overwrite;}

    /**
     * Set overwrite policy.
     *
     **/
    public void setOverwrite(boolean overwrite) {this.overwrite = overwrite;}

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor etc
//          16 Nov 2000 SvdB Made consistent with MakePortfolio (removed specification)
//           5 May 2000 SvdB V3.0
//           7 Dec 1998 SvdB Second release to Unicore
//          12 Jun 1998 SvdB Initial release to Unicore
//             
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
