package org.unicore.outcome;

import org.unicore.Unicore;
import org.unicore.ajo.Ticket;
import org.unicore.AAIdentifier;

import org.unicore.sets.ResourceSet;

import java.util.Date;

/**
 * This class encapsutates Quality of Service information for the Unicore Task. 
 *
 * QoS information includes expected start time, end time and cost for the run of the Task.
 * It also includes a Ticket which should be included in the {@link org.unicore.resources.QoSCheck} sent
 * with the Task.
 *
 * @author A. Bekele (fecit)
 * @author D. Snelling (fecit)
 *
 * @see org.unicore.ajo.CheckQoS
 *
 * @since AJO 3.6
 *
 * @version Estimate.java 1.6 10/10/02
 * 
 **/
public class Estimate extends Unicore {

    static final long serialVersionUID = 234099624848157922L;

    private double cost;       
    private String units;
    
    private Date earliestStartTime;               
    private Date latestStartTime;               

    private Date earliestEndTime;               
    private Date latestEndTime;               
    
    private Ticket ticket;        
    private AAIdentifier tid;     
    
    /** 
     * Creates an new Estimate instance.
     *
     * @param cost This represents the cost for the run of the Unicore
     * Task represented by this instance. The cost value is represented by
     * a double.
     * @param units The units of cost
     *
     * @param est The earliest expected start time of the Unicore Task.
     * @param lst The latest expected start time of the Unicore Task.
     * @param eet The earliest expected end time of the Unicore Task.
     * @param let The latest expected end time of the Unicore Task.
     * @param ticket The ticket for the Unicore Task.
     * @param tid The Identifier for the Unicore Task.
     * @param resources The resources for which this estimate was made
     *
     **/
    public Estimate(double cost, String units, Date est, Date lst, Date eet, Date let, Ticket ticket, AAIdentifier tid, ResourceSet resources) {
    
        this.cost              = cost;
        this.units             = units;
        this.earliestStartTime = est;
        this.latestStartTime   = lst;
        this.earliestEndTime   = eet;
        this.latestEndTime     = let;
        this.ticket            = ticket;
        this.tid               = tid;
        this.resources         = resources;
    }
    
    public Estimate(){
        this(0.0, (String)null, (Date) null, (Date) null, (Date) null, (Date) null, (Ticket) null, (AAIdentifier) null , (ResourceSet) null);
    }
    
    public void setTID(AAIdentifier tid) {
        this.tid = tid;
    }
    
    public void setCost(double cost) {
        this.cost = cost;
    }
    
    public void setUnits(String units) {
        this.units = units;
    }
    
    public void setTicket(Ticket ticket){
        this.ticket = ticket; 
    }
    
    /** 
     * Returns the cost of running the Unicore Task
     *
     * @return Value of the cost of running the Unicore Task
     *
     **/
    public double getCost(){
           return this.cost;
    }
    
    /** 
     * Returns the units of the cost estimate.
     *
     * @return the units of cost for this estimate.
     *
     **/
    public String getUnits(){
           return this.units;
    }
    
    /** 
     * Returns the earliest expected start time of the Unicore Task
     *
     * @return Date that represents the earliest expected start time of the Unicore Task.
     *
     **/
    public Date getEarliestStartTime(){
        return this.earliestStartTime;
    }
    
    /**
     * Set the earliest expected start time of the Unicore Task.
     *
     * @param time The earliest expected start time of the Unicore Task
     *
     **/
    public void setEarliestStartTime(Date t) {
        this.earliestStartTime = t; 
    }
    
    /** 
     * Returns the latest expected start time of the Unicore Task
     *
     * @return Date that represents the latest expected start time of the Unicore Task.
     *
     **/
    public Date getLatestStartTime(){
        return this.latestStartTime;
    }
    
    /**
     * Set the latest expected start time of the Unicore Task.
     *
     * @param time The latest expected start time of the Unicore Task
     *
     **/
    public void setLatestStartTime(Date t) {
        this.latestStartTime = t; 
    }
    
    /** 
     * Returns the earliest expected end time of the Unicore Task
     *
     * @return Date that represents the earliest expected end time of the Unicore Task.
     *
     **/
    public Date getEarliestEndTime(){
        return this.earliestEndTime;
    }
    
    /**
     * Set the earliest expected end time of the Unicore Task.
     *
     * @param time The earliest expected end time of the Unicore Task
     *
     **/
    public void setEarliestEndTime(Date t) {
        this.earliestEndTime = t; 
    }
    
    /** 
     * Returns the latest expected end time of the Unicore Task
     *
     * @return Date that represents the latest expected end time of the Unicore Task.
     *
     **/
    public Date getLatestEndTime(){
        return this.latestEndTime;
    }
    
    /**
     * Set the latest expected end time of the Unicore Task.
     *
     * @param time The latest expected end time of the Unicore Task
     *
     **/
    public void setLatestEndTime(Date t) {
        this.latestEndTime = t; 
    }
    
    /** 
     * Returns the ticket token associated with the QoS for the Unicore Task.
     *
     * @return Ticket token
     *
     **/
    public Ticket getTicket(){
        return this.ticket;
    }

    /** 
     * Returns the identifier for the Unicore Task
     *
     * @return Identifier for the Unicore Task
     *
     **/
    public AAIdentifier getTID(){
        return this.tid;
    }

    private ResourceSet resources;

    public ResourceSet getResources() {return resources;}

    public void setResources(ResourceSet resources) {this.resources = resources;}
}
//
// Change history:
//          10 Oct 2002 SvdB Added resourecs
//          28 Jun 2002 SvdB Added time windows for start and end
//           6 Dec 2001 DFS Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
