package org.unicore.resources;

/**
 * Describes a resource that has some measurable aspect.
 * <p>
 * Parent of all classes used to describe resources which represent a (roughly)
 * measurable quantity of usable resource, e.g. memory, floating point operations, etc.
 * <p>
 * A CapacityResource in a Resource Description has a minimum, maximum and default value.
 * <p>
 * A CapacityResource in a Resource Resquest fills in the request field to
 * request an amount of the CapacityResource. Any request
 * that does not contain a given capacity resource is granted the site default.
 * <p>
 * CapacityResources are sent as job totals, a requested amount summed
 * over all processors allocated to the job. A Resource Description assumes
 * a single processor job.
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version CapacityResource.java 1.13 06/11/02
 *
 **/
public abstract class CapacityResource extends Resource {

    static final long serialVersionUID = 6154414659164805106L;

    public CapacityResource() {
        this("",0.0,0.0,0.0);
    }

    /**
     * Create a new CapacityResource.
     * <p>
     * This constructor suggests a rule that
     * new CapacityResources are created as Resource Descriptions.
     * A Resource Resquest clones an existing instance
     * and sets the desired request amount.
     *
     * @param description A description ofthe CapacityResource
     * @param maxrequest Maximum amount of resource allowed
     * @param defaultrequest Amount of resource granted if no specific request received
     * @param minrequest Minimum amount of resource allowed
     *
     **/
    public CapacityResource (String description,
                             double maxrequest,
                             double defaultrequest,
                             double minrequest) {
        super(description);

        setMaxRequest(maxrequest);
        setMinRequest(minrequest);
        setDefaultRequest(defaultrequest);

        setRequest(defaultrequest);
    }

    protected String units;
    /**
     * Return the units in which requests are measured. 
     * <p>
     * This is set by the definition of the final object type.
     *
     **/
    public  String getUnits () { return units; }

    private double maxrequest;
    /**
     * Return the maximum request allowed for this resource. 
     * <p>
     * As set by the Vsite.
     *
     **/
    public  double getMaxRequest ()                  { return maxrequest; }

    /**
     * Change the maximum request allowed for this resource. 
     * <p>
     * Should only be changed by a Vsite.
     *
     **/
    public  void   setMaxRequest (double maxrequest) { this.maxrequest = maxrequest; }

    private double defaultrequest;
    /**
     * Return the default request set for this resource. 
     * <p>
     * As set by the Vsite.
     *
     **/
    public  double getDefaultRequest ()                  { return defaultrequest; }

    /**
     * Change the default request set for this resource.
     * <p>
     * Should only be changed by a Vsite.
     *
     **/
    public  void   setDefaultRequest (double defaultrequest) { 
        setRequest(defaultrequest);
        this.defaultrequest = defaultrequest; 
    }

    private double minrequest;
    /**
     * Return the minumum request allowed for this resource. 
     * <p>
     * As set by the Vsite.
     *
     **/
    public  double getMinRequest ()                      { return minrequest; }

    /**
     * Change the minimum request allowed for this resource. 
     * <p>
     * Should only be changed by a Vsite.
     *
     **/
    public  void   setMinRequest (double minrequest) { this.minrequest = minrequest; }

    private double request;
    /**
     * Return the amount of this resource requested by the containing Task. 
     * <p>
     * Set by the client.
     *
     **/
    public  double getRequest ()               { return request; }

    /**
     * Set the amount of this resource requested by the containing Task. 
     * <p>
     * Set by the client.
     *
     **/
    public  void   setRequest (double request) { this.request = request; }

    /**
     * Is this resource of the same Class? 
     * <p>
     * CapacityResources can 
     * refer to the same physical resource (are equal) or they
     * be references to a type of resource - for example when
     * requesting Nodes the type is important but the 
     * max, min or default values are not relevant.
     *
     * @deprecated AJO 4.0 equals now does this
     *
     **/
    public boolean equivalences(CapacityResource c) {
        return equals(c);
    }

    /**
     * CapacityResources are equal if they represent the same resource.
     * This usually means that the type is the same but occasionally also
     * involves other paramaters.
     * <p>
     *<em>AJO 4.0 No longer considers the limits (max, min, default) </em>
     * <p>
     **/
     public boolean equals(Object o) {

        if(o == null) return false;

        if(o.getClass() == getClass()) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Return a hash code for this CapacityResource. 
     **/
    public int hashCode() {
        return 998822222;
    }

}
//
// Change history:
//          15 Mar 2002 SvdB V4, no arg constructor, deprecated equivalences
//          06 Oct 2000 SvdB request is initialised to DefaultRequest
//          21 Aug 2000 SvdB Change minimum to min - consistent with max 
//          14 Aug 2000 DFS added minimum request field and fixed bug in equals.
//          13 Jul 2000 SvdB override equals
//          21 Jan 2000 DFS First release with V3.0 resource model. 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
//
//     This software was developed as part of the UNICORE and UNICORE Plus
//     Seamless Computing Projects and the authors gratefully acknowledge the 
//     efforts of all our partners.
