package org.unicore.sets;

import org.unicore.*;
import org.unicore.ajo.*;
import org.unicore.outcome.*;
import java.util.*;

/**
 * This class contains a map from Vsites to the Estimate sets.
 *
 * Each Estimate set contains the Estimates for the tasks
 * in the brokered AJO. Each EstimateSet represents a different bid from
 * the Vsite.
 *
 * @author A. Bekele (fecit)
 * @author D. Snelling (fecit)
 * @author Sven van den Berghe (fecit)
 *
 * @since AJO 3.6
 *
 * @version  VsiteEstimateMap.java 1.3 09/09/02
 * 
 **/

public class VsiteEstimateMap extends Unicore {

    private Map map;
   
    /**
     * Creates a new VsiteEstimateMap with a 1st Vsite and a 1st EstimateSet.
     *
     **/
    public VsiteEstimateMap (Vsite vsite, EstimateSet est_set) {
        this();
        addEstimateSet(vsite,est_set);
    }
    
    /**
     * Create an empty VsiteEstimateMap.
     *
     **/
    public VsiteEstimateMap () {
        map = new HashMap (); 
    }
    
    /**
     * Adds a new EstimateSet associated with a Vsite.
     * If est_set equals an EstimateSet already associated with
     * the Vsite, only one copy is retained.
     *
     * Both must be non-null
     *
     *
     **/
    public void addEstimateSet (Vsite vsite, EstimateSet est_set) {
        if(vsite != null && est_set != null) {
            if (map.containsKey(vsite)) {
                ((HashSet) map.get(vsite)).add(est_set);
            } else {
                HashSet sets = new HashSet ();
                sets.add(est_set);
                map.put (vsite, sets);
            }
        }
    }

    /**
     * Remove all estimates for a Vsite.
     *
     * @param vsite Unicore Vsite
     *
     **/
    public void removeEstimateSets (Vsite vsite) {
        map.remove (vsite);
    }

    /**
     * Removes all entries in the Map.
     *
     * <p>
     * <b>Pre:</b> true 
     * <b>Post:</b> The map is empty
     *
     **/
    public void removeAll() {map.clear();}

    /**
     * Returns all the EstimateSets for this Vsite as an enumeration.
     *
     **/
    public EstimateSetEnumeration getEstimateSetEnumeration (Vsite vsite) {
        return new EstimateSetEnumeration (new Vector ((HashSet) (map.get (vsite)))); 
    }
    
    /**
     * Returns an Enumeration over the Vsites for which 
     * EstimateSets are held. 
     *
     **/
    public VsiteEnumeration getVsiteEnumeration () {
        return new VsiteEnumeration (new Vector(map.keySet()));
    }
}
//
// Change history:
//          19 Jul 2002 SvdB Mapped Vistes not names
//          30 May 2002 DFS  Converted to multiple EstimateSets per Vsite 
//          28 May 2002 SvdB Constructor matches set type
//          21 Feb 2002 SvdB Added removeAll method
//          12 Dec 2001 DFS  Initial release to Unicore 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
