package org.unicore.upl;

import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.AJOIdentifier;

/**
 * A ConsignJob is sent by a user to a Vsite
 * to ask for some work to be done.
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2.9
 *
 * @version ConsignJob.java 1.8 05/13/02  
 * 
 **/
public final class ConsignJob extends ServerRequest {

    /**
     * Create an empty ConsignJob
     *
     **/
    public ConsignJob() {
        this(null, null, null, null, null, null, false, true);
    }

    /**
     * Create a ConsignJob
     *
     * @param target The Identifier of the AJO being consigned
     * @param vsite Where to execute the AJO
     * @param endorser The User who created the AJO (and who will be used to execute it)
     * @param ajo The AJO to execute (serialised)
     * @param signature The signature of the AJO
     * @param sso Site specific security object
     * @param has_streamed Does the Request hace Portfolios coming as streamed data?
     * @param polling Should the request be executed in polling mode?
     *
     **/
    public ConsignJob(AJOIdentifier target,
                      Vsite vsite,
                      User endorser,
                      byte[] ajo,
                      byte[] signature,
                      byte[] sso,
                      boolean has_streamed,
                      boolean polling) {
        super(target,vsite,sso);
        this.endorser = endorser;
        this.ajo = ajo;
        this.signature = signature;
        this.has_streamed = has_streamed;

        this.polling = polling;
    }

    private byte[] ajo;
    /**
     * Return the work to be done on the Vsite. 
     *
     **/
    public byte[] getAJO() {return ajo;}

    /**
     * Set the work to be done on the Vsite. 
     *
     **/
    public void setAJO(byte[] ajo) {this.ajo = ajo;}

    private byte[] signature;
    /**
     * Return the signature of the serialised AJO.
     * <p>
     * The signing
     * algorithm is agreed elsewhere (Where?)
     *
     **/
    public byte[] getSignature() {return signature;}

    /**
     * Set the signature of the serialised AJO.
     *
     * {@link org.unicore.utility.ConsignForm}
     * will compute the signature of the serialised AJO (see {@link org.unicore.ajo.AbstractJob#getSignature})
     * <p>
     * The signature algorithm is MD5withRSA.
     *
     **/
    public void setSignature(byte[] signature) {this.signature = signature;}

    // NOTE The alternative to the above two byte arrays is to
    //      use java.security.SignedObject which encapsulates
    //      this information plus some algorithm information.
    //      It claims to store the object serialised. The advantage
    //      of doing as two arrays is that we can choose not to sign.

    private User endorser;
    /**
     * Return the AJO's endorser.
     * <p>
     * The Unicore user under whose
     * authority this job will be executed. Incarnated
     * tasks will use the Xlogin corresponding to this
     * Ulogin. The AJO's signature must be correctly
     * verified by this Ulogin.
     *
     **/
    public User getEndorser() {return endorser;}
    /**
     * Set the AJO's endorser (this must be done by the client).
     *
     **/
    public void setEndorser(User endorser) {this.endorser = endorser;}

    boolean has_streamed;
    /**
     * Is this consign followed by streamed data?
     *
     **/
    public boolean hasStreamed() {return has_streamed;}

    /**
     * Set whether this consign is followed by streamed data.
     *
     **/
    public void setStreamed(boolean has_streamed) {this.has_streamed = has_streamed;}

    private boolean polling;
    /**
     * Return the style of interaction to be used in fetching the Outcome.
     *
     **/
    public boolean isPolling() {return polling;}

    /**
     * Set the style of interaction to be used in fetching the
     * Outcome of the AJO. 
     * <p>
     * If this is not set, then the client is asking
     * for a non-Polling style and expects a RetrieveOutcomeReply
     * in response. If it is set, then the style is Polling whereby
     * the client expects a ConsignJobReply and will poll for
     * completion using RetrieveOutcome requests.
     *
     **/
    public void setPolling(boolean polling) {this.polling = polling;}

    static final long serialVersionUID = 6270214020139528884L;

}
//
// Change history:
//          15 Mar 2002, SvdB V4 changes, moved SSO to ServerRequest
//          13 Feb 2001, SvdB Pay some attention to the user's polling desire!
//          29 Jul 2000, SvdB did it properly
//          15 May 2000, SvdB Added choice of polling style
//           2 May 2000, SvdB Changes for V3 
//           4 Feb 2000, SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
